;***************************************;***************************************
;
;   Author              :	jeff bachiochi
;   Company             :  	imagine that
;	Web Site			:	www.imaginethatnow.com
;   Filename            :  	Graphic Touch.s 
;   Date                :  	2/14/2013
;   File Version        :  	1.2                                                   
;
;***************************************;***************************************
;
;	Version 0.1	=	Initial work file
;	Version 1.1	=	print to all orientations
;	Version 1.1	=	BMP to all orientations
;	Version 1.2	=	Project Release
;
;   Additional Notes:                         
;                                        
;  	@32MHz instruction time = 62.6nS                                           
;	Revision 4 chips have a bug in the TX FIFO
;
;	be careful what you have connected to MCLR... requires enabling high voltage
;	 MCLR to program (MCLR)
;
;	Make surer to program the device in high voltage mode or MCLR will not be disabled
;
;	Orientation chooses mode
;	Portrait mode = 240x320
;	Landscape mode = 320x240
;	UpsideDown = 180 degree rotation
;	Mirror = Flip on axis (left/right)
;
;***************************************;***************************************
;
        .equ __24FV32KA302, 1
        .include "p24FV32KA302.inc"
	.equ	Debug,	0					; 1=Debug, 0=Production
;
;***************************************;***************************************
;
;	Configuration bits:
;
;***************************************;***************************************
;{
        config __FOSCSEL, FNOSC_FRCPLL & SOSCSRC_DIG  ; use the Internal Clock
        								; as the system clock
                                        
        config __FOSC, OSCIOFNC_OFF  	; no clock out                                      

        config __FWDT, FWDTEN_OFF  & WINDIS_OFF	; Turn off Watchdog Timer

	.if	Debug==1
	        config __FPOR,  BORV_LPBOR & PWRTEN_ON & MCLRE_ON
	.else        
	        config __FPOR,  BORV_LPBOR & PWRTEN_ON & MCLRE_OFF
	.endif	
                                        ; Set Brown-out Reset voltage, use MCLR
                                        ; and set Power-up Timer to 16msecs
                                            
        config __FGS, GSS0_OFF        	; Set Code Protection Off for the 
                                        ;General Segment
		config __FICD,	ICS_PGx1		; Use 
;}
;***************************************;***************************************
;
;	Program Specific Constants (literals used in code)
;
;***************************************;***************************************
;{
;	PORTA
	.equ	MinusX,	0					; Touch X-
	.equ	PlusY,	1					; Touch Y+
	.equ	PlusX,	2					; Touch X+
	.equ	MinusY,	3					; Touch Y-
	.equ	SD_CS,	4					; SD Card Chip Select
	.equ	JP1,	5					; PIC reset/JP1 (OFF)1=normal, (ON) 0=inverted
;	.equ	CAP,	6					; external cap for core
	.equ	RST,	7					; LCD reset
;
;	PORTB
	.equ	JP2,	0					; PGD/JP2 (OFF)1=9600, (ON)0=115200
	.equ	BL,		1					; PGC/Backlight
	.equ	RX,		2					; serial in
	.equ	CD,		3					; 1=control, 0=data
	.equ	CS,		4					; 1=enable
	.equ	WR_STROBE,		5					; 1=Write Strobe
	.equ	RD_STROBE,		6					; 1=Read Strobe
	.equ	TX,		7					; serial out
	.equ	D0,		8					; data0
	.equ	D1,		9					; data1
	.equ	D2 ,	10					; data2 (SDO)
	.equ	D3 ,	11					; data3 (SCK)
	.equ	D4,		12					; data4
	.equ	D5,		13					; data5 (SDI)
	.equ	D6,		14					; data6
	.equ	D7,		15					; data7
;
    .equ 	UARTBuffer_Length, 64  		; Buffer Size
    .equ 	CommandBuffer_Length, 64	; Buffer Size
    .equ	TouchMinimum,	0x10		; A/D minimum above zero (no touch)
    .equ	CharacterRowSpacing,	8	; pixels/character
    .equ	CharacterColumnSpacing,	6	; pixels/character    
    .equ	ButtonColumns,	4			; characters
    .equ	ButtonRows,		3			; characters
    .equ	ButtonRowSpacing,	0		; row pixels between buttons 
    .equ	ButtonColumnSpacing,	0	; column pixels between buttons   
    .equ	OriginOffsetX,	4			; button origin row offset in pixels
    .equ	OriginOffsetY,	4			; button origin column offset in pixels
    .equ	RowOffset,	(ButtonRows*CharacterRowSpacing)+ButtonRowSpacing
    .equ	ColumnOffset,	(ButtonColumns*CharacterColumnSpacing)+ButtonColumnSpacing  
	.equ	Media_Retries,	4
	.equ	DelayCount,		8
	.equ	BusyCount,		8
	.equ	InitialOrientation,	0		; 0=Landscape, 1=Portrait
	.equ	InitialUpsideDown,	0		; 0=RightsideUp, 1=UpsideDown
	.equ	InitialMirror,	0			; 0=Not Mirrored, 1=Mirrored
;
;	TFT ILI9320
;    
    .equ	TFT_PWidth,	240				; pixels
    .equ	TFT_PHeight,	320			; pixels
;    .equ	TFT_Rotation,	0		
    .equ	TFT_PositionX,	0
    .equ	TFT_PositionY,	0
    .equ	TFT_TextSize,	1
    .equ	TFT_Black,		0x0000
    .equ	TFT_Blue,		0x001F
    .equ	TFT_Green,		0x07E0
    .equ	TFT_Cyan,		0x07FF	
    .equ	TFT_Red,		0xF800
    .equ	TFT_Magenta,	0xF81F
    .equ	TFT_Yellow,		0xFFE0
    .equ	TFT_White,		0xFFFF
	.equ	TFT_IndexRegister,	0
	.equ	TFT_StatusRegister,	0
	.equ	TFT_DriverCodeRegister,	0
	.equ	TFT_StartOscRegister,	0    
	.equ	TFT_DriverOutputControl1Register,	1
	.equ	TFT_LCDDrivingControlRegister,	2
	.equ	TFT_EntryModeRegister,	3
	.equ	TFT_ResizeControlRegister,	4     
	.equ	TFT_DisplayControl1Register,	7
	.equ	TFT_DisplayControl2Register,	8
	.equ	TFT_DisplayControl3Register,	9     
	.equ	TFT_DisplayControl4Register,	0xA    
	.equ	TFT_RGBDisplayInterfaceControl1Register,	0xC
	.equ	TFT_FrameMakerPositionRegister,	0xD
	.equ	TFT_RGBDisplayInterfaceControl2Register,	0xF    
	.equ	TFT_PowerControl1Register,	0x10
	.equ	TFT_PowerControl2Register,	0x11
	.equ	TFT_PowerControl3Register,	0x12
	.equ	TFT_PowerControl4Register,	0x13    
	.equ	TFT_HorizontalGRAMAddressRegister,	0x20
	.equ	TFT_VerticalGRAMAddressRegister,	0x21
	.equ	TFT_WriteDataToGRAMRegister,	0x22
	.equ	TFT_PowerControl7Register,	0x29     
	.equ	TFT_FrameRateAndColorControlRegister,	0x2B
	.equ	TFT_GammaControl1Register,	0x30
	.equ	TFT_GammaControl2Register,	0x31
	.equ	TFT_GammaControl3Register,	0x32
	.equ	TFT_GammaControl4Register,	0x35    
	.equ	TFT_GammaControl5Register,	0x36     
	.equ	TFT_GammaControl6Register,	0x37    
	.equ	TFT_GammaControl7Register,	0x38
	.equ	TFT_GammaControl8Register,	0x39
	.equ	TFT_GammaControl9Register,	0x3C
	.equ	TFT_GammaControl10Register,	0x3D    
	.equ	TFT_HorizontalAddressStartPositionRegister,	0x50
	.equ	TFT_HorizontalAddressEndPositionRegister,	0x51
	.equ	TFT_VerticalAddressStartPositionRegister,	0x52
	.equ	TFT_VerticalAddressEndPositionRegister,	0x53    
	.equ	TFT_DriverOutputControl2Register,	0x60
	.equ	TFT_BaseImageDisplayControlRegister,	0x61
	.equ	TFT_VerticalScrollControlRegister,	0x6A
	.equ	TFT_PartialImage1DisplayPositionRegister,	0x80
	.equ	TFT_PartialImage1AreaStartLineRegister,	0x81
	.equ	TFT_PartialImage1AreaEndLineRegister,	0x82
	.equ	TFT_PartialImage2DisplayPositionRegister,	0x83    
	.equ	TFT_PartialImage2AreaStartLineRegister,	0x84     
	.equ	TFT_PartialImage2AreaEndLineRegister,	0x85    
	.equ	TFT_PanalInterfaceControl1Register,	0x90
	.equ	TFT_PanalInterfaceControl2Register,	0x92
	.equ	TFT_PanalInterfaceControl3Register,	0x93
	.equ	TFT_PanalInterfaceControl4Register,	0x95    
	.equ	TFT_PanalInterfaceControl5Register,	0x97     
	.equ	TFT_PanalInterfaceControl6Register,	0x98
;
;	SystemFLAGS
	.equ	U1RXEmpty,	0				; 1=No Characters available
	.equ	U1RXCharacterReady,	1    	; 1=Character Removed (U1RXCharacter)
	.equ	Response,	2				; 0=OK, 1=?
	.equ	DataExpected,	3			; 0=none, 1=Data Expected (BitMap)
	.equ	FirstSector,4				; 1=Yes,	0=No
	.equ	Palette,	5				; 1=Yes,	0=No
	.equ	Pixels,		6				; 1=Yes, 	0=No
;	
;	LCDFLAGS	
	.equ	WrapAround,	0				; 1=wrap
	.equ	Match,	1					; 1=Match
	.equ	NewPush,	2				; 0=New Push, 1=Still pushed
	.equ	Comma,		3				; 0=no comma yet, 1=comma found
	.equ	InvertDisplay,	4			; 0=Normal, 1=Invert
	.equ	SerialLogic,	5			; 0=Invert,	1=Normal
	.equ	SerialBaud,		6			; 0=115200, 1=9600
	.equ	OrientationDisplay,	7		; 0=240x320 (Portrait)
										; 1=320x240 (Landscape)
	.equ	BackLightDisplay,	8		; 1=ON, 0=OFF
	.equ	MirrorDisplay,	9			; 1=Mirror, 0=Normal
	.equ	UpsideDownDisplay,	10		; 1=UpsideDown, 0=Normal
	.equ	BMP2Large,	11				; 0=OK, 1=Too Large for LCD
;			
;	TouchFLAGS	
	.equ	ReadingX,	0				; 1=touch X inprocess
	.equ	ReadingY,	1				; 1=touch Y inprocess
	.equ	ReadingZ1,	2				; 1=touch Z1 (X) inprocess
	.equ	ReadingZ2,	3				; 1=touch Z2 (Y) inprocess
	.equ	TouchReady,	4				; 1=all samples finished
	.equ	Button1,	5				; 1=touched
	.equ	Button2,	6				; 1=touched	
	.equ	Calibrate,	7				; 1=request calibration
;	
;	SDFlags
	.equ	MediaMode,	0				; 1=read, 0=write
	.equ	BadMedia,	1				; 1=Bad, 0=OK
	.equ	MediaReady,	2				; 1=Ready, 0=Not Ready
;	
;	R1 flags
	.equ	InIdleState,	0
	.equ	EraseState,	1
	.equ	IllegalCOmmand,	2
	.equ	CRCError,	3
	.equ	EraseError,	4
	.equ	AddressError,	5
	.equ	ParameterError,	6
;}
;***************************************;***************************************
;
;	Global Declarations:
;
;***************************************;***************************************
;{
        .global	_wreg_init       		; Provide global scope to _wreg_init 
                                 		; routine. In order to call this routine
                                 		;  from a C file, place "wreg_init" in 
                                 		; an "extern" declarationin the C file.

        .global __reset          		; The label for the first line of code. 

        .global __T1Interrupt    		; Declare Timer 1 ISR name global
        .global __U1TXInterrupt    		; Declare UART1 TX ISR name global
        .global __U1RXInterrupt    		; Declare UART1 RX ISR name global  
        .global __T2Interrupt    		; Declare Timer2 ISR name global  
        .global __SPI1Interrupt			; Declare SPI ISR as global 
        .global __ADC1Interrupt			; Declare A/D ISR as global         
;}
;***************************************;***************************************
;
;	Global Declarations:
;
;***************************************;***************************************
;{
        .section 	.nbss, bss, near
TEMP32H:		.space	2
TEMP32L:		.space	2
;     
U1TXBuffer_Head: 	.space 	2   		; U1TXBuffer Pointer (in) 
U1TXBuffer_Tail: 	.space 	2   		; U1TXBuffer Pointer (out)
U1RXBuffer_Head: 	.space 	2   		; U1RXBuffer Pointer (in) 
U1RXBuffer_Tail: 	.space 	2   		; U1RXBuffer Pointer (out)
;
CommandBufferPointer: .space 	2  		; Command Buffer Pointer
SystemFlags:	.space	2				; System SDFlags
LCDFlags:		.space	2				; LCD SDFlags
TouchFlags:		.space	2				; Touch SDFlags
SDFlags:		.space	2
;
Media_Retry_Count:	.space	2
SPI_CMD_Delay:	.space	2
SPI_CMD_Length:	.space	2
SPI_CMD_inputbuffer_pointer:	.space	2
SPI_CMD_outputbuffer_pointer:	.space	2
SPI_CMD_inputbuffer:	.space	6
SPI_CMD_outputbuffer:	.space	6
;
SPI_CMD_Response_Length:	.space	2
SPI_CMD_Response_Delay:		.space	2
SPI_CMD_Response_inputbuffer_pointer:	.space	2
SPI_CMD_Response_inputbuffer:	.space	6
;
SPI_Data_Length:	.space 	2
SPI_Data_Delay:		.space	2
SPI_Data_buffer_pointer:	.space	2
;
SPI_Data_Response_Length:	.space	2
SPI_Data_Response_Delay:	.space	2
SPI_Data_Response_inputbuffer_pointer: .space	2
SPI_Data_Response_inputbuffer: .space	6
;
File_Name:	.space	12
File_ATT:	.space	2
File_NTRES:	.space	2
File_CTimeF:	.space	2
File_CTime:	.space	2
File_CDate:	.space	2
File_LADate:	.space	2
File_FClusterH:	.space	2
File_WTime:	.space	2
File_WDate:	.space	2
File_FClusterL:	.space	2
File_Size:	.space	4
FileLength:	.space	4
;
BitMapType:	.space	2
BitMapFileSizeL:	.space	2
BitMapFileSizeH:	.space	2
BitMapReservedL:	.space	2
BitMapReservedH:	.space	2
BitMapOffsetL:	.space	2
BitMapOffsetH:	.space	2
BitMapHeaderSizeL:	.space	2
BitMapHeaderSizeH:	.space	2
BitMapWidthL:	.space	2
BitMapWidthH:	.space	2
BitMapHeightL:	.space	2
BitMapHeightH:	.space	2
BitMapPlanes:	.space	2
BitMapBitsPerPixel:	.space	2
BitMapCompressionL:	.space	2
BitMapCompressionH:	.space	2
BitMapSizeL:	.space	2
BitMapSizeH:	.space	2
BitMapHorizontalResL:	.space	2
BitMapHorizontalResH:	.space	2
BitMapVerticalResL:	.space	2
BitMapVerticalResH:	.space	2
BitMapColorsL:	.space	2
BitMapColorsH:	.space	2
BitMapImportantColorsL:	.space	2
BitMapImportantColorsH:	.space	2
BitMapColorTableSizeL:	.space	2
BitMapColorTableSizeH:	.space	2
BitMapByteCountL:	.space	2
BitMapByteCountH:	.space	2
BMP_Width_Count:	.space	2
BMP_Width_DataMax:	.space	2
BMP_ColumnMax:	.space	2
BMP_Height_Count:	.space	2
BMP_RowMax:	.space	2
;
BlockAddressH:	.space	2
BlockAddressL:	.space	2
VBSH:	.space	2
VBSL:	.space	2
NumDirEntries:	.space	2
NumDirInSector:	.space	2
NumSecInCluster:	.space	2
FATSectorCount:	.space	2
NumFATs:	.space	2
NumSecInFAT:	.space	2
FATEntryValue:	.space	2
FATOffsetCount:	.space	2
FATEntry:	.space	2
ReservedSectorCount:	.space	2
FATRegionSectorH:	.space	2
FATRegionSectorL:	.space	2
RootDirSector:	.space	2
LastDirectoryCluster:	.space	2
SectorOfInterestH:	.space	2
SectorOfInterestL:	.space	2
SectorCount:	.space 2
DirectoryCount:	.space	2
NumSecInClusterCount:	.space	2
DataOffsetCount:	.space	2
ClusterOfInterest:	.space	2
StringBuffer:	.space	14
;
TFT_WR_STROBE_Value:	.space	2				; word to write to TRF
TFT_RD_STROBE_Value:	.space	2				; word read from TFT
TEMPW:			.space	2
TFTW_Width:		.space	2			
TFTW_Height:	.space	2
Width:			.space	2			
Height:			.space	2
TFTW_PositionX:	.space	2
TFTW_PositionY:	.space	2
TFTW_OriginX:	.space	2
TFTW_OriginY:	.space	2
TFTW_ButtonX:	.space	2
TFTW_CharacterX:	.space	2
TFTW_CharacterY:	.space	2
TFTW_TextColor:	.space	2
TFTW_FillColor:	.space	2
TFTW_X:			.space	2
TFTW_X1:		.space	2
TFTW_X2:		.space	2
TFTW_Y:			.space	2
TFTW_Y1:		.space	2
TFTW_Y2:		.space	2
TFTW_PixelColor:.space	2
TFTW_PixelX:	.space	2
TFTW_PixelY:	.space	2
TFTW_RectangleColor:	.space	2
TFTW_RectangleX1:	.space	2
TFTW_RectangleX2:	.space	2
TFTW_RectangleY1:	.space	2
TFTW_RectangleY2:	.space	2
TFTW_BFColor:	.space	2
TFTW_BBColor:	.space	2
TFTW_TFColor:	.space	2
TFTW_TBColor:	.space	2
TFTW_EEPROM:	.space	2
TFTW_BitMapWidth:	.space	2
TFTW_BitMapHeight:	.space	2
TFTW_BitMapX:	.space	2
TFTW_BitMapY:	.space	2
;
U1TXCharacter:	.space	1
U1RXCharacter:	.space	1
TFT_IR:			.space	1				; Index Register
;TFTB_Rotation:	.space	1		
TFTB_TextSize:	.space	1
TFTB_Character:	.space	1
TFTB_Count:		.space	1
BitCount:		.space	1
TFTB_Row:		.space	1
TFTB_RowCount:	.space	1
TFTB_RowSize:		.space	1
TFTB_RowSizeCount:	.space	1
TFTB_Column:	.space	1
TFTB_ColumnCount:	.space	1
TFTB_ColumnSize:	.space	1
TFTB_ColumnSizeCount:	.space	1
TEMPB:			.space	1
Line:			.space	1
TFTB_TouchX:	.space	1
TFTB_TouchY:	.space	1
TFTB_TouchZ1:	.space	1
TFTB_TouchZ2:	.space	1
Last_TouchX:	.space	1
Last_TouchY:	.space	1
Rotated_TouchX:	.space	1
Rotated_TouchY:	.space	1
ButtonID:		.space	1				; inc as button commands are entered
TFTB_XMin:	.space	1
TFTB_XMax:		.space	1
TFTB_XDivisor:	.space	1
TFTB_YMin:	.space	1
TFTB_YMax:		.space	1
TFTB_YDivisor:	.space	1
TouchID:		.space	1
TFTB_KeypadRow:	.space	1
TFTB_KeypadRowCount:	.space	1
TFTB_KeypadColumn:	.space	1
TFTB_KeypadColumnCount:	.space	1
TFTB_CharacterWidth:	.space	1
TFTB_CharacterHeight:	.space	1
TFTB_Red:	.space	1
TFTB_Green:	.space	1
TFTB_Blue:	.space	1
TFTB_Char1:	.space	1
TFTB_Char2:	.space	1
TFTB_Char3:	.space	1
TFTB_Char4:	.space	1
TFTB_Char5:	.space	1
TFTB_Char6:	.space	1
;
		.align	2
U1TXBuffer:			.space	UARTBuffer_Length	; U1TXBuffer 
U1RXBuffer:			.space	UARTBuffer_Length	; U1RXBuffer
CommandBuffer:		.space	CommandBuffer_Length	; Command Buffer
TouchMatrix:		.space	16			; Button # Matrix X=0
TouchMatrix1:		.space	16			; Button # Matrix X=1
TouchMatrix2:		.space	16			; Button # Matrix X=2
TouchMatrix3:		.space	16			; Button # Matrix X=3
TouchMatrix4:		.space	16			; Button # Matrix X=4
TouchMatrix5:		.space	16			; Button # Matrix X=5
TouchMatrix6:		.space	16			; Button # Matrix X=6
TouchMatrix7:		.space	16			; Button # Matrix X=7
TouchMatrix8:		.space	16			; Button # Matrix X=8
TouchMatrix9:		.space	16			; Button # Matrix X=9
TouchMatrixA:		.space	16			; Button # Matrix X=10
TouchMatrixB:		.space	16			; Button # Matrix X=11
TouchMatrixC:		.space	16			; Button # Matrix X=12
TouchMatrixD:		.space	16			; Button # Matrix X=13
TouchMatrixE:		.space	16			; Button # Matrix X=14
TouchMatrixF:		.space	16			; Button # Matrix X=15
		.align 4
SPI_Data_buffer:	.space	516
ColorTable1:		.space 512
TopOfMemoryUsage:	.space	1
;}
;***************************************;***************************************
;
;	EEPROM
;
;	Initial data for Touch Pad Calibration 
;
;***************************************;***************************************
;{
	.section	*,eedata
TouchEEXData:
 	.word 0x20C0						; TouchXMin, TouchXMax,
TouchEEYData:
	.word 0x20C0		 				; TouchYMin, TouchYMax
;}
;***************************************;***************************************
;
;	Code Section in Program Memory
;
;***************************************;***************************************
;{
.text                             		; Start of Code section
__reset:
        mov 	#__SP_init,	W15       	; Initalize the Stack Pointer
        mov 	#__SPLIM_init,	W0     	; Init the Stack Pointer Limit Register
        mov 	W0, SPLIM
        nop                       		; Add NOP to follow SPLIM initialization
;        
        call	_wreg_init           	; Call _wreg_init subroutine
;        
;***************************************
;
init_CLKDIV:
		mov		#0x0000,	W0			; 1:1 (8MHz w/PLL=32MHz clk)
		mov		W0,	CLKDIV
;		
;***************************************		
;
init_PORTA:
 		mov		#0x000f,	W0			; 1=analog, 0=digital
 										; 15:8=00000000, 7:0=00001111	
 		mov		W0,	ANSA       			; analog/digital
; 		
 		mov		#0x0000,	W0			; 1=open drain, 0=digital
 		 								; 15:8=0, 7:0=0 
 		mov		W0,	ODCA       			; open drain/digital
; 		
 		mov		#0xFF6F,	W0			; 1=input, 0=output
 										; 15:8=FF, 7:0=01101111 
 										; RES=O, SD_CS=O
 		mov		W0,	TRISA       		; input/output
; 		 		 
 		mov		#0x0010,	W0			; 1=high, 0=low
 										; 15:8=0, 7:0=00010000 
 		mov		W0,	LATA       			; RES=O, SD_CS=1 		
; 
 		mov		#0x0000,	W0			; 1=Interrupt On Change, 0=no IOC
 										; 15:8=00000000, 7:0=00000000
  		mov		W0,	CNEN1
; 		 		 
 		mov		#0xF800,	W0			; 1=PullUp enabled, 0=no PullUp
 										; 15:8=11111000, 7:0=00000000
  		mov		W0,	CNPU1 		
;		
;***************************************		
;		
init_PORTB:
 		mov		#0x0000,	W0			; 1=analog, 0=digital
 										; 15:8=00000000, 7:0=00000000	
 		mov		W0,	ANSB       			; analog/digital
; 		
 		mov		#0x0000,	W0			; 1=open drain, 0=digital
 										; 15:8=0, 7:0=0		
 		mov		W0,	ODCB       			; open drain/digital
;
	.if Debug	==	1	
;
 		mov		#0xFF07,	W0			; 1=input, 0=output
 										; 15:8=FF, 7:0=00000111, 
 										; TX1=0, RD_STROBE=O, WR_STROBE=O, CS=0, CD=0
;
	.else
;	
 		mov		#0xFF05,	W0			; 1=input, 0=output
 										; 15:8=FF, 7:0=00000101, 
 										; TX1=0, RD_STROBE=O, WR_STROBE=O, CS=0, CD=0, BL=0 
; 		 		 
	.endif
; 																				
 		mov		W0,	TRISB       		; input/output
;	
 		mov		#0x0080,	W0			; 1=high, 0=low
 										; 15:8=0, 7:0=10000000
 										; TX=1
		mov		W0,	LATB 										
; 		 		 
 		mov		#0x0000,	W0			; 1=Interrupt On Change, 0=no IOC
 										; CN31:22=00000000, CB21:16=00000000
 		mov		W0,	CNEN2
; 	
 		mov		#0x0010,	W0			; 1=PullUp enabled, 0=no PullUp
 										; CN15:8=00000000, CN7:0=00010000 										
		mov		W0,	CNPU1				; JP2=PU
;		
;***************************************		
;
init_Varaibles:
		mov		#U1TXBuffer_Head,	W0
		mov		#TopOfMemoryUsage,	W1
;
init_Varaibles_Loop:
		clr.b	[W0++]
		cpseq	W0,		W1
		goto	init_Varaibles_Loop
;		
		mov		#U1TXBuffer,	W0
		mov		W0,	U1TXBuffer_Head
		mov		W0,	U1TXBuffer_Tail	
		mov		#U1RXBuffer,	W0
		mov		W0,	U1RXBuffer_Head
		mov		W0,	U1RXBuffer_Tail
		mov		#CommandBuffer,	W0
		mov		W0,	CommandBufferPointer
		mov		#1,		W0
		mov.b	WREG,	ButtonID
		mov		#CharacterColumnSpacing,	W0
		mov.b	WREG,	TFTB_CharacterWidth
		mov		#CharacterRowSpacing,	W0
		mov.b	WREG,	TFTB_CharacterHeight
;						
		btsc	PORTA,		#JP1		; skip next if jumpered (invert UART)
		bset	LCDFlags,		#SerialLogic	; else (normal UART)
		btsc	PORTB,		#JP2		; skip next if jumpered (115200 BAUD)
		bset	LCDFlags,		#SerialBaud	; else 9600 BAUD
;
		bset	LCDFlags,		#WrapAround		
;		
;***************************************		
;
init_Timer1:							; LCD timer	(delay)
		mov		#0x8020,	W0			; TON=0, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=10, x, TSYNC=0, TCS=0, x
		mov		W0,		T1CON			; tick=16MHz/64=4uS
;
		mov		#5000,	W0	
		mov		W0,		PR1				; 4*5000=20mS
;		
;***************************************		
;
init_Timer2:							; Touch scan
		mov		#0x0030,	W0			; TON=0, x=0, TSIDL=0, x, x, x, x, x,
										; x, TGATE=0, TCKPS1:0=11, x, T32=0, TCS=0, x
		mov		W0,		T2CON			; tick=16MHz/256=16uS
;
		mov		#3125,	W0	
		mov		W0,		PR2				; 16*3125=50mS
;
		bclr	IFS0,	#T2IF			; clear any interrupt		
;		
;***************************************		
;						   
init_UART1:
 		mov		#0x8008,	W0			; UARTEN=1, x, USIDL=0, IREN=0,
										; RTSMD=0, x, UEN1=0, UEN0=0,
										; WAKE=0, LPBACK=0, ABAUD=0, RXINV=0,
										; BRGH=1, PDSEL1=0, PDSEL0=0, STSEL=0
 		mov		W0,	U1MODE      		; U1 Status	
 		btss	LCDFlags,	#SerialLogic	; skip next if normal serial selected
 		bset	U1MODE,		#RXINV
;
 		mov		#0x0510,	W0			; UTXISEL1=0, UTXINV=0, UTXISEL0=0, x,
 										; UTXBRK=0, UTXEN=1, UTXBF=0, TRMT=1,
										; URXISEL1:=00, ADDEN=0, RIDLE=1,
										; PERR=0, FERR=0, OERR=0, URXDA=0
 		mov		W0,	U1STA      			; U2 Status	
 		btss	LCDFlags,	#SerialLogic	; skip next if normal serial selected
 		bset	U1STA,		#UTXINV 					  
;
		mov 	#34,	W0				; @32MHz 32=115200 baud
 		btsc	LCDFlags,	#SerialBaud		; skip next if high baud selected	
		mov 	#416,	W0				; @32MHz 32=9600 baud		
		mov		W0,	U1BRG
;
		bclr	IFS0,	#U1RXIF			; clear any interrupt
;
 		btss	LCDFlags,	#SerialLogic	; skip next if normal serial selected
 		bset	CNPD1,		#CN6PDE		; pull down on RX
;
 		btsc	LCDFlags,	#SerialLogic	; skip next if inverted serial selected 		
 		bset	CNPU1,		#CN6PUE		; pull up on RX 		
;		
;***************************************		
;						   
init_SPI:
 		mov		#0x0062,	W0			; x, x, x, DISSCK=0,
										; DISSDO=0, MODE16=0, SMP=0, CKE=0,
										; SSEN=1, CKP=1, MSTEN=0, SPRE=0,
										; SPRE1:0=00, PPRE1:0=10
		mov		W0,	SPI1CON1
;		
 		mov		#0x0000,	W0			; FRMEN=0, SPIFSD=0, SPIFPOL=0, x
 										; x, x, x, x,
 										; x, x, x, x,
 										; x, x, SPIFE=0, SPIBEN=0
		mov		W0,	SPI1CON2
;
 		mov		#0x0004,	W0			; SPIEN=0, x, SPISIDL=0, x,
										; x, SPIBEC2:0=000,
 										; SRMPT=0, SPIROV=0, SRXMPT=0,
 										; SISEL2:0=001, SPITBF=0, SPIRBF=0		
		mov		W0,	SPI1STAT			; leave disabled for now
;		
;***************************************		
;		
init_A_D:
		mov		#0x0270,	W0			; ADON=0, x=0, ADSIDL=0, DMABM=0,
										; DMAEN=0, MODE12=0, FORM1:0=10,  
										; SSRC3:0=0111,  
										; x=0, ASAM=0, SAMP=0, DONE=0
		mov		W0,		AD1CON1			; A/D Control Register 1
;
		mov		#0x0000,W0				; CH0NB=000, CH0SB=00000, 
										; CH0NA=000, CH0SA=00000
		mov		W0,		AD1CHS			; A/D Sample Select Register 		
;												
		mov		#0x0800,W0				; PVCFG1:0=00, NVCFG0=0, OFFCAL=0,
										; BUFREGEN=1, CSCNA=0, x=0, x=0,
										; BUFS=0, SMPI4=00000, BUFM=0, ALTS=0
		mov		W0,		AD1CON2			; A/D Control Register 2
;		
		mov		#0x1F3F,W0				; ADRC=0, EXTSAM=0, PUMPEN=0, SAMC4=11111,
										; ADCS7=00111111
		mov		W0,		AD1CON3			; A/D Control Register 3
;
		mov		#0x0000,W0				; CHH15=0, CHH14=0, CHH13=0, CHH12=0,
										; CHH11=0, CHH10=0, CHH9=0, CHH8=0,
										; CHH7=0, CHH6=0, CHH5=0, CHH4=0,
										; CHH3=0, CHH2=0, CHH1=0, CHH0=0,
		mov		W0,		AD1CSSL			; A/D Scan Select Register (Low Word)
;
		bclr	IFS0,	#AD1IF			; clear any interrupts
		bset	AD1CON1,#ADON			; enable A/D
;
		call	Read_TouchX				; prepares PORTA and starts a sample
;		
;***************************************		
;
init_TFT:
		mov		#TFT_PWidth,	W0
		mov		W0,	Width
		mov		#TFT_PHeight,	W0
		mov		W0,	Height	
		mov		#TFT_PositionX,	W0
		mov		W0,	TFTW_PositionX
		mov		#TFT_PositionY,	W0
		mov		W0,	TFTW_PositionY
		mov		#TFT_White,	W0
		mov		W0,	TFTW_TextColor
		mov		#TFT_Black,	W0
		mov		W0,	TFTW_FillColor
;		
		mov		#InitialOrientation,	W0
		btsc	W0,		#0				; skip next if 240x320 Portrait
		goto	init_TFT_L
;
init_TFT_P:
		bclr	LCDFlags,	#OrientationDisplay	; force Landscape
		mov		#TFT_PHeight,	W0
		mov		W0,		Width
		mov		#TFT_PWidth,	W0
		mov		W0,		Height
		mov		#InitialUpsideDown,	W0
		btsc	W0,		#0				; skip next if RightsideUp
		goto	init_TFT_P_UpsideDown
;
init_TFT_P_RightsideUp:
		bclr	LCDFlags,	#UpsideDownDisplay	; RightsideUp
		mov		#InitialMirror,	W0
		btsc	W0,		#0				; skip next if not Mirrored
		goto	init_TFT_P_RightsideUp_Mirrored
;		
init_TFT_P_RightsideUp_NotMirrored:
		bclr	LCDFlags,		#MirrorDisplay
		mov		#0x1030,	W0			; 0x1030
		mov		W0,		TEMPW		
		goto	init_TFT_Continue


init_TFT_P_RightsideUp_Mirrored:
		bset	LCDFlags,		#MirrorDisplay	
		mov		#0x1020,	W0			; 0x1020
		mov		W0,		TEMPW		
		goto	init_TFT_Continue	
		
init_TFT_P_UpsideDown:
		bset	LCDFlags,	#UpsideDownDisplay	; force UpsideDown
		mov		#InitialMirror,	W0
		btsc	W0,		#0				; skip next if not Mirrored
		goto	init_TFT_P_UpsideDown_Mirrored
;		
init_TFT_P_UpsideDown_NotMirrored:
		bclr	LCDFlags,		#MirrorDisplay
		mov		#0x1000,	W0			; 0x1000
		mov		W0,		TEMPW
		goto	init_TFT_Continue


init_TFT_P_UpsideDown_Mirrored:
		bset	LCDFlags,		#MirrorDisplay	
		mov		#0x1010,	W0			; 0x0010
		mov		W0,		TEMPW		
		goto	init_TFT_Continue	
;
init_TFT_L:
		bset	LCDFlags,	#OrientationDisplay	; force Portrait
		mov		#TFT_PHeight,	W0
		mov		W0,		Height
		mov		#TFT_PWidth,	W0
		mov		W0,		Width
		mov		#InitialUpsideDown,	W0
		btsc	W0,		#0				; skip next if RightsideUp
		goto	init_TFT_L_UpsideDown
;
init_TFT_L_RightsideUp:
		bclr	LCDFlags,	#UpsideDownDisplay	; RightsideUp
		mov		#InitialMirror,	W0
		btsc	W0,		#0				; skip next if not Mirrored
		goto	init_TFT_L_RightsideUp_Mirrored
;		
init_TFT_L_RightsideUp_NotMirrored:
		bclr	LCDFlags,		#MirrorDisplay
		mov		#0x1028,	W0			; 0x1028
		mov		W0,		TEMPW
		goto	init_TFT_Continue


init_TFT_L_RightsideUp_Mirrored:
		bset	LCDFlags,		#MirrorDisplay	
		mov		#0x1008,	W0			; 0x1008
		mov		W0,		TEMPW		
		goto	init_TFT_Continue	
		
init_TFT_L_UpsideDown:
		bset	LCDFlags,	#UpsideDownDisplay	; force UpsideDown
		mov		#InitialMirror,	W0
		btsc	W0,		#0				; skip next if not Mirrored
		goto	init_TFT_L_UpsideDown_Mirrored
;		
init_TFT_L_UpsideDown_NotMirrored:
		bclr	LCDFlags,		#MirrorDisplay
		mov		#0x1018,	W0			; 0x1018
		mov		W0,		TEMPW
		goto	init_TFT_Continue


init_TFT_L_UpsideDown_Mirrored:
		bset	LCDFlags,		#MirrorDisplay	
		mov		#0x1038,	W0			; 0x1038
		mov		W0,		TEMPW		
		goto	init_TFT_Continue
;
init_TFT_Continue:
		mov		#TFT_TextSize,	W0
		mov.b	WREG,	TFTB_TextSize
;
		bset	LATB,	#CS
		bset	LATB,	#CD
		bset	LATB,	#RD_STROBE
		bset	LATB,	#WR_STROBE
;
		call	Reset_TFT
;
		mov		#TFT_StartOscRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0001,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#3125,	W0				; 16uS * 3125 = 50mS	
		call	Delay		
;
		mov		#TFT_DriverOutputControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0100,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_LCDDrivingControlRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0700,	W1
 		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_EntryModeRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		TEMPW,		W1
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_ResizeControlRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_DisplayControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0202,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_DisplayControl3Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_DisplayControl4Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_RGBDisplayInterfaceControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT						   
;
		mov		#TFT_FrameMakerPositionRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_RGBDisplayInterfaceControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PowerControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT    
;
		mov		#TFT_PowerControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0007,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PowerControl3Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PowerControl4Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT				   		
;
		mov		#12500,	W0				; 16uS * 12500 = 200mS	
		call	Delay		
;
		mov		#TFT_PowerControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x1690,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT    
;
		mov		#TFT_PowerControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0227,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#3125,	W0				; 16uS * 3125 = 50mS	
		call	Delay		
;
		mov		#TFT_PowerControl3Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x001A,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#3125,	W0				; 16uS * 3125 = 50mS	
		call	Delay		
;
		mov		#TFT_PowerControl4Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x1800,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_PowerControl7Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x002A,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#3125,	W0				; 16uS * 3125 = 50mS	
		call	Delay		
;
		mov		#TFT_GammaControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_GammaControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl3Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl4Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0206,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl5Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0808,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl6Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0007,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl7Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0201,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl8Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl9Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_GammaControl10Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT																		
;
		mov		#TFT_HorizontalGRAMAddressRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_VerticalGRAMAddressRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_HorizontalAddressStartPositionRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_HorizontalAddressEndPositionRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x00EF,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_VerticalAddressStartPositionRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_VerticalAddressEndPositionRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x013F,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT 	
;
		mov		#TFT_DriverOutputControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0xA700,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT			
;
		mov		#TFT_BaseImageDisplayControlRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0003,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_VerticalScrollControlRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PanalInterfaceControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0010,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_PanalInterfaceControl2Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PanalInterfaceControl3Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0003,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PanalInterfaceControl4Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0110,	W1
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PanalInterfaceControl5Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		mov		#TFT_PanalInterfaceControl6Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0000,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		call	Set_Rotation
;		
		call	FullScreenWindow
;
		mov		#TFT_Black,	W0
		mov		W0,		TFTW_FillColor
		call	ClearScreen
;
		mov		#TFT_DisplayControl1Register,	W0
		mov.b	WREG,		TFT_IR
		mov		#0x0133,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		call	ImagineThat
;		
	.if Debug	==	0
;	
		bset	LCDFlags,	#BackLightDisplay
		bset	LATB,	#BL	
;		
	.endif
;	
		call	InitTouchCalibration			
;		
;***************************************		
;
init_Interrupts:
		mov		#0x2C80,	W0			; NVMIE=0, x, AD1IE=1, U1TXIE=0,
										; U1RXIE=1, SPI1IE=1, SPF1IE=0, T3IE=0,
										; T2IE=1, OC2IE=0, IC2IE=0, x,
										; T1IE=0, OC1IE=0, IC1IE=0, INT0IE=0
		mov		W0,		IEC0
;
		mov		#0x0000,	W0			; U2TXIE=0, U2RXIE=0, INT2IE=0, T5IE=0,
										; T4IE=0, x, OC3IE=0, x,
										; x, x, x, INT1IE=0,
										; CNIE=0, CMIE=0, MI2C1IE=0, SI2C1IE=0
		mov		W0,		IEC1
;		
;***************************************		
;
		bset	T2CON,	#TON
;
		goto	main
;}
;***************************************;***************************************
;
;	Main Code Section 					; Service the RX buffer and LCD Touch 
;
;***************************************;***************************************
;{
main:
;***************************************;
main_RX:
		call	U1RXBuffer_Remove
;cls

		btss	SystemFlags,	#U1RXCharacterReady
		goto	main_RX_Exit
;
		bclr	SystemFlags,	#U1RXCharacterReady
		call	FillCommandBuffer			
;
main_RX_Exit:
;***************************************;
main_Touch:
		btss	TouchFlags,	#TouchReady
		goto	main_Touch_Exit
;
		bclr	TouchFlags,	#TouchReady
		call	CheckTouch		
;		
main_Touch_Exit:
;***************************************;
main_Exit:
		goto	main
;}
;***************************************;***************************************
;
;	Subroutine: Initialization of W0:14 registers to 0x0000
;
;	Requires
;		none
;	Changes
;		W0
;		W14
;	Calls
;		none
;
;***************************************;***************************************
;{
_wreg_init:
        clr 	W0
        mov 	W0,	W14
        repeat 	#12
        mov 	W0, [++W14]
        clr 	W14
        return
;}
;***************************************;***************************************
;
;	Subroutine: Read word from EEPROM
;
;
;	DateLastEdited:	3/28/2013
;
;	Requires
;		TBLPAG	-	EEPROM page address	
;		W1		-	EEPROM data address
;	Changes
;		W0
;		W1
;		TFTW_EEPROM
;	Calls
;		none
;
;***************************************;***************************************
;{
ReadEEPROM:
		tblrdl	[W1++],	W0				; Get EEPROM data
		mov		W0,		TFTW_EEPROM		; data
;			
ReadEEPROM_Exit:
        return 
;}
;***************************************;***************************************
;
;	Subroutine: Write word to EEPROM
;
;
;	DateLastEdited:	3/28/2013
;
;	Requires
;		TBLPAG	-	EEPROM page address	
;		data written to EEPROM data address
;	Changes
;		W0
;		W1
;		NVMCON.WR
;		NVMKEY
;	Calls
;		none
;
;***************************************;***************************************
;{
WriteEEPROM:
		disi 	#5						; Disable Interrupts For 5 instructions
		mov 	#0x55, 	W0 				; Issue Unlock Sequence
		mov 	W0, 	NVMKEY 
		mov 	#0xAA, 	W1
		mov 	W1, 	NVMKEY 
		bset 	NVMCON, #WR 			; Perform Write/Erase operations
		nop 
		nop
;		 
WriteEEPROM_Exit:
		btsc	NVMCON, #WR 			; skip next if finished
		goto	WriteEEPROM_Exit
;		
        return
;}
;***************************************;***************************************
;
;	Subroutine: Initialize Touch Calibration
;
;
;	DateLastEdited:	3/28/2013
;
;	Requires
;		data written to EEPROM data address
;	Changes
;		W0
;		W1
;		TBLPAG
;		TFTB_XMax
;		TFTB_XMin
;		TFTB_XDivisor
;		TFTB_YMax
;		TFTB_YMin
;		TFTB_YDivisor
;	Calls
;		ReadEEPROM
;
;***************************************;***************************************
;{       
InitTouchCalibration:
		mov		#tblpage(TouchEEXData), W1	; Set TBLPAG to the page that 
		mov		W1, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEXData), W1	; Make a pointer to table
		call	ReadEEPROM
;		
		mov		TFTW_EEPROM,	W0
		mov.b	WREG,	TFTB_XMax		; maximum X value
		swap.w	W0						; swap bytes
		mov.b	WREG,	TFTB_XMin		; minimum X value
		sub.b	TFTB_XMax,	WREG		; WREG=TFTB_XMax-TFTB_XMin
		and.b	#0xF0,	W0
		swap.b	W0						; swap nibbles
		mov.b	WREG,	TFTB_XDivisor	; TFTB_Divisor=XSpan/16
;
		mov		#tblpage(TouchEEYData), W1	; Set TBLPAG to the page that 
		mov		W1, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEYData), W1	; Make a pointer to table
		call	ReadEEPROM
;		
		mov		TFTW_EEPROM,	W0
		mov.b	WREG,	TFTB_YMax		; maximum Y value
		swap.w	W0						; swap bytes
		mov.b	WREG,	TFTB_YMin		; minimum Y value
		sub.b	TFTB_YMax,	WREG		; WREG=TFTB_YMax-TFTB_YMin
		and.b	#0xF0,	W0
		swap.b	W0						; swap nibbles
		mov.b	WREG,	TFTB_YDivisor	; TFTB_YDivisor=YSpan/16
;				
InitTouchCalibration_Exit:
		return                        
;}
;***************************************;***************************************
;
;	Subroutine: Check Touch for finger pressure
;
;
;	DateLastEdited:	3/28/2013
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		LCDFlags.NewPush
;		Last_TouchX
;		Last_TouchY
;	Calls
;		SendTouch
;
;***************************************;***************************************
;{        
CheckTouch:
		mov		#TouchMinimum,	W0
		cp.b	TFTB_TouchZ2
		bra		GTU,	CheckTouch_Touch
;
		com		W0,		W0
		cp.b	TFTB_TouchZ1
		bra		GEU,	CheckTouch_NoTouch		
;
CheckTouch_Touch:
		mov.b	Last_TouchX,	WREG
		mov		W0,		W1
		and.b	#0xF0,	W1
		mov.b	TFTB_TouchX,	WREG
		and.b	#0xF0,	W0		
		cp.b	W0,		W1
		bra		NZ,		CheckTouch_Save
;
		mov.b	Last_TouchY,	WREG
		mov		W0,		W1
		and.b	#0xF0,	W1
		mov.b	TFTB_TouchY,	WREG
		and.b	#0xF0,	W0		
		cp.b	W0,		W1
		bra		NZ,		CheckTouch_Save
;
		btsc	LCDFlags,	#NewPush
		goto	CheckTouch_Exit
;						
		call	SendTouch
		bset	LCDFlags,	#NewPush
		goto	CheckTouch_Exit
;
CheckTouch_NoTouch:
		bclr	LCDFlags,	#NewPush
		goto	CheckTouch_Exit		
;		
CheckTouch_Save:
		mov.b	TFTB_TouchX,	WREG
		mov.b	WREG,	Last_TouchX
		mov.b	TFTB_TouchY,	WREG
		mov.b	WREG,	Last_TouchY
;
CheckTouch_Exit:				
		return  
;}
;***************************************;***************************************
;
;	Subroutine: Send Touch 				- Send Button message if Matrix Lookup
;										 is something other than 0 (no button)
;										 (based on TFTB_TouchX and TFTB_TouchY
;										 measurements)
;
;	DateLastEdited:	3/28/2013
;
;	Requires
;		TFTB_XDivisor
;		TFTB_TouchX
;		TFTB_XMin
;		TFTB_XMax
;		TFTB_YDivisor
;		TFTB_TouchY
;		TFTB_YMin
;		TFTB_YMax
;		TFTW_EEPROM
;	Changes
;		W0
;		W1
;		TouchID
;		TouchFlags.Button1
;		TouchFlags.Button2
;		TouchFlags.Calibrate
;		NVMCON.WREN
;	Calls
;		ReadEEPROM
;		WriteEEPROM
;		InitTouchCalibration
;		CalibrateDone		
;		InvertToggle
;		Delay		
;		InvertToggle
;		SendMessage
;		SendDecimalWord
;		SendCRLF
;
;***************************************;***************************************
;{        
SendTouch:
		push.d	W2
;		
		clr		W0
		mov.b	TFTB_XDivisor,	WREG
		mov		W0,		W2
		clr		W0
		mov.b	TFTB_TouchX,	WREG
		mov		W0,		W1
		clr		W0
		mov.b	TFTB_XMin,	WREG
		sub		W1,		W0,		W1
		cp0		W1
		bra		NN,		SendTouch_X_Min
;		
		clr		W1
;
SendTouch_X_Min:
		clr		W0
		mov.b	TFTB_XMin,	WREG
		sub.b	TFTB_XMax,	WREG
		dec.b	W0,		W0
		cp  	W0,		W1
		bra		NN,		SendTouch_X_Max
;
		mov.b	W0,		W1		
;
SendTouch_X_Max:
		mov		W1,		W0		
		repeat	#17
		div.u	W0,		W2
		mov		W0,		W3
;				
		clr		W0
		mov.b	TFTB_YDivisor,	WREG
		mov		W0,		W2
		clr		W0
		mov.b	TFTB_TouchY,	WREG
		mov		W0,		W1
		clr		W0
		mov.b	TFTB_YMin,	WREG
		sub 	W1,		W0,		W1
		cp0		W1
		bra		NN,		SendTouch_Y_Min
;		
		clr		W1
;
SendTouch_Y_Min:
		clr		W0
		mov.b	TFTB_YMin,	WREG
		sub.b	TFTB_YMax,	WREG
		dec.b	W0,		W0		
		cp		W0,		W1
		bra		NN,		SendTouch_Y_Max
;
		mov.b	W0,		W1		
;
SendTouch_Y_Max:
		mov		W1,		W0		
		repeat	#17
		div.u	W0,		W2
		swap.b	W0						; swap nibbles		
		ior.b	W0,		W3,		W0		;combine X and Y as MatrixOffset
;		
		mov		#TouchMatrix,	W1
		add		W0,		W1,		W1		;TouchMatrix address + MatrixOffset
		mov.b	[W1],	W0
 		mov.b	WREG,	TouchID
;
		cp0.b	TouchID
		bra		Z,		SendTouch_Exit	; branch if TouchID=0
;
		btss	TouchFlags,	#Calibrate	; skip next if request for calibration
		goto	SendTouch_Continue
;
SendTouch_Chk4ID1:
		mov		#1,		W0
		cp.b	TouchID
		bra		NZ,		SendTouch_Chk4ID2
;
SendTouch_SaveID1Data:
		bset	TouchFlags,	#Button1
		mov		#tblpage(TouchEEXData), W1	; Set TBLPAG to the page that 
		mov		W1, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEXData), W1	; Make a pointer to table
		call	ReadEEPROM
		mov		TFTW_EEPROM,	W2		; Get EEPROM data TouchXMinMax
		mov		#tblpage(TouchEEYData), W1	; Set TBLPAG to the page that 
		mov		W1, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEYData), W1	; Make a pointer to table		
		call	ReadEEPROM
		mov		TFTW_EEPROM,	W3		; Get EEPROM data TouchYMinMax
;
		mov		#0x00FF,	W0			
		and		W0,		W2,		W2		; just Xmax
		clr		W0
		mov.b	TFTB_TouchX,	WREG
		swap	W0
		ior		W0,		W2,		W2		; replace Xmin
		mov		#tblpage(TouchEEXData), W1	; Set TBLPAG to the page that 
		mov		W1, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEXData), W1	; Make a pointer to table		
		bset	NVMCON,	#WREN			; enable 1 write		
		tblwtl	W2,		[W1++]			; Write Data		
		call	WriteEEPROM
;
		mov		#0x00FF,	W0			
		and		W0,		W3,		W3		; just Ymax
		clr		W0
		mov.b	TFTB_TouchY,	WREG
		swap	W0
		ior		W0,		W3,		W3		; replace Ymin
		bset	NVMCON,	#WREN			; enable 1 write
		tblwtl	W3,		[W1]			; Write Data		
		call	WriteEEPROM							
;
SendTouch_Chk4ID2:				
		mov		#2,		W0
		cp.b	TouchID
		bra		NZ,		SendTouch_Chk4Done
;
SendTouch_SaveID2Data:		
		bset	TouchFlags,	#Button2
		mov		#tblpage(TouchEEXData), W2	; Set TBLPAG to the page that 
		mov		W2, _TBLPAG 	    	; contains the EEPROM Data
		mov		#tbloffset(TouchEEXData), W2	; Make a pointer to table
		call	ReadEEPROM
		mov		TFTW_EEPROM,	W2		; Get EEPROM data TouchXMinMax
		call	ReadEEPROM
		mov		TFTW_EEPROM,	W3		; Get EEPROM data TouchYMinMax
;
		mov		#0xFF00,	W0			
		and		W0,		W1,		W1		; just Xmin		
		clr		W0
		mov.b	TFTB_TouchX,	WREG
		ior		W0,		W1,		W1		; replace Xmax
		bset	NVMCON,	#WREN			; enable 1 write		
		tblwtl	W1,		[W2++]			; Write Data		
		call	WriteEEPROM
;
		mov		#0xFF00,	W0			
		and		W0,		W3,		W3		; just Ymin
		clr		W0
		mov.b	TFTB_TouchY,	WREG
		ior		W0,		W3,		W3		; replace Ymax
		bset	NVMCON,	#WREN			; enable 1 write		
		tblwtl	W3,		[W2]			; Write Data		
		call	WriteEEPROM	
;
SendTouch_Chk4Done:
		btss	TouchFlags,	#Button1
		goto	SendTouch_Continue
;
		btss	TouchFlags,	#Button2
		goto	SendTouch_Continue
;
		bclr	TouchFlags,	#Calibrate
		call	InitTouchCalibration
;		
		call	CalibrateDone		
;
SendTouch_Continue:
		call	InvertToggle
;
		mov		#12500,	W0				; 16uS * 12500 = 200mS	
		call	Delay		
;
		call	InvertToggle
;							
		mov		#tblpage(Button_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Button_Command), W1	; Make a pointer to table
		call	SendMessage
;
		clr		W0
		mov.b	TouchID,	WREG
		call	SendDecimalWord
;
		call	SendCRLF
;		
SendTouch_Exit:
		pop.d	W2
		return		      
;}
;***************************************;***************************************
;
;	Subroutine: Fill Command Buffer     - remove received characters and
;										 build a command
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		U1RXCharacter
;	Changes
;		W0
;		W1
;		CommandBufferPointer
;	Calls
;		MatchACommand
;		SendMessage
;		SendCRLF
;		SendCharacter
;
;***************************************;***************************************
;{        
FillCommandBuffer:
;
FillCommandBuffer_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1RXCharacter
		bra		Z,		FillCommandBuffer_CR
		
FillCommandBuffer_Chk4LF:
		mov		#0x0A,	W0				; CR
		cp.b	U1RXCharacter
		bra		Z,		FillCommandBuffer_Exit	; just dump it		
;
FillCommandBuffer_Chk4LT_Space:
		mov		#' ',	W0				; space
		cp.b	U1RXCharacter
		bra		LTU,	FillCommandBuffer_Illegal
;		
FillCommandBuffer_Chk4GT_Tilde:		
		mov		#'~',	W0				; ~
		cp.b	U1RXCharacter
		bra		GTU,	FillCommandBuffer_Illegal
;		
FillCommandBuffer_Legal:
		mov.b	U1RXCharacter,	WREG
		mov		CommandBufferPointer,	W1
		mov.b	W0,		[W1++]
		mov		#CommandBuffer+CommandBuffer_Length,	W0
		cp		W0,		W1
		bra		Z,		FillCommandBuffer_Illegal	; branch if end of Command Buffer
;		
		mov		W1,		CommandBufferPointer	
		goto	FillCommandBuffer_Exit			
;
FillCommandBuffer_CR:
		mov.b	U1RXCharacter,	WREG
		mov		CommandBufferPointer,	W1
		mov.b	W0,		[W1++]
		mov		W1,		CommandBufferPointer
		call	MatchACommand
;
		btss	LCDFlags,		#Match	; skip next if at least one command		
		goto	FillCommandBuffer_Illegal	; that's all, must not be a command
;
		bclr	LCDFlags,		#Match	; clear the match		
		mov		#CommandBuffer,	W0
		mov		W0,	CommandBufferPointer
		mov		#tblpage(OK_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(OK_Command), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;				
		goto	FillCommandBuffer_Exit		
;
FillCommandBuffer_Illegal:
		mov		#CommandBuffer,	W0
		mov		W0,	CommandBufferPointer
		mov		#'?',	W0
		mov.b	WREG,	U1TXCharacter	
		call	SendCharacter
;
		call	SendCRLF
;				
		goto	FillCommandBuffer_Exit						
;
FillCommandBuffer_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Match a Command			- look for command matches 
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		
;	Changes
;		W0
;		W1
;		LCDFlags.Match
;		CommandBufferPointer
;	Calls
;		Help
;		CompareStrings
;		ClearScreen
;		Pixel
;		NextY
;		NextX
;		ImagineThat
;		FColor
;		BColor
;		ATX
;		ATY
;		BMP
;		Button
;		Bitmap
;		TextSize
;		Orientation
;		Wrap
;		Display
;		Invert
;		Backlight
;		Mirror
;		Upside Down
;		Print
;		Test
;
;***************************************;***************************************
;{ 		
MatchACommand:
		bclr	LCDFlags,		#Match
;***************************************
MatchACommand_Chk4QM:		
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer
		mov		#'?',	W1
		cp.b	W1,		[W0]
		bra		NZ,		MatchACommand_Chk4QM_Exit
;
MatchACommand_QM:
		call	Help
;
		goto	MatchACommand_Exit
;				
MatchACommand_Chk4QM_Exit:		
;***************************************
MatchACommand_Chk4CLS:
		mov		#tblpage(CLS_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(CLS_Command), W1		; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,	#Match		
		goto	MatchACommand_Chk4CLS_Exit
;
MatchACommand_CLS:
		call	ClearScreen
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4CLS_Exit:
;***************************************
MatchACommand_Chk4Pixel:
		mov		#tblpage(Pixel_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Pixel_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Pixel_Exit
;
MatchACommand_Pixel:
		call	Pixel
;			
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Pixel_Exit:
;***************************************
MatchACommand_Chk4LF:
		mov		#tblpage(CLS_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(CLS_Command), W1		; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4LF_Exit
;
MatchACommand_LF:
		call	NextY
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4LF_Exit:
;***************************************
MatchACommand_Chk4CR:
		mov		#tblpage(CLS_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(CLS_Command), W1		; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4CR_Exit
;
MatchACommand_CR:
		call	NextX
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4CR_Exit:
;***************************************
MatchACommand_Chk4IT:
		mov		#tblpage(IT_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(IT_Command), W1		; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4IT_Exit
;
MatchACommand_IT:
		call	ImagineThat
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4IT_Exit:
;***************************************
MatchACommand_Chk4FColor:
		mov		#tblpage(FColor_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(FColor_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4FColor_Exit
;
MatchACommand_FColor:
		call	FColor
;		
		goto	MatchACommand_Exit
;
MatchACommand_Chk4FColor_Exit:		
;***************************************
MatchACommand_Chk4BColor:
		mov		#tblpage(BColor_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BColor_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4BColor_Exit
;
MatchACommand_BColor:
		call	BColor
;		
		goto	MatchACommand_Exit
;
MatchACommand_Chk4BColor_Exit:		
;***************************************
MatchACommand_Chk4ATX:
		mov		#tblpage(ATX_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(ATX_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4ATX_Exit
;
MatchACommand_ATX:
		call	ATX
;		
		goto	MatchACommand_Exit
;
MatchACommand_Chk4ATX_Exit:		
;***************************************
MatchACommand_Chk4ATY:
		mov		#tblpage(ATY_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(ATY_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4ATY_Exit
;
MatchACommand_ATY:
		call	ATY
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4ATY_Exit:	
;***************************************
MatchACommand_Chk4BMP:
		mov		#tblpage(BMP_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BMP_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4BMP_Exit
;
MatchACommand_BMP:
		call	BMP
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4BMP_Exit:	
;***************************************
MatchACommand_Chk4Button:
		mov		#tblpage(Button_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Button_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Button_Exit
;
MatchACommand_Button:
		call	Button
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Button_Exit:
;***************************************
MatchACommand_Chk4BitMap:
		mov		#tblpage(BitMap_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BitMap_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4BitMap_Exit
;
MatchACommand_BitMap:
		call	BitMap
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4BitMap_Exit:		
;***************************************
MatchACommand_Chk4TextSize:
		mov		#tblpage(TextSize_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(TextSize_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4TextSize_Exit
;
MatchACommand_TextSize:
		call	TextSize
;				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4TextSize_Exit:		
;***************************************
MatchACommand_Chk4Orientation:
		mov		#tblpage(Orientation_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Orientation_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Orientation_Exit
;
MatchACommand_Orientation:
		call	Orientation		
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Orientation_Exit:	
;***************************************
MatchACommand_Chk4Wrap:
		mov		#tblpage(Wrap_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Wrap_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Wrap_Exit
;
MatchACommand_Wrap:
		call	Wrap	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Wrap_Exit:	
;***************************************
MatchACommand_Chk4Display:
		mov		#tblpage(Display_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Display_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Display_Exit
;
MatchACommand_Display:
		call	Display	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Display_Exit:	
;***************************************
MatchACommand_Chk4Invert:
		mov		#tblpage(Invert_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Invert_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Invert_Exit
;
MatchACommand_Invert:
		call	Invert	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Invert_Exit:
;***************************************
MatchACommand_Chk4BackLight:
		mov		#tblpage(BackLight_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BackLight_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4BackLight_Exit
;
MatchACommand_BackLight:
		call	BackLight	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4BackLight_Exit:
;***************************************
MatchACommand_Chk4Mirror:
		mov		#tblpage(Mirror_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Mirror_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Mirror_Exit
;
MatchACommand_Mirror:
		call	Mirror	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Mirror_Exit:
;***************************************
MatchACommand_Chk4UpsideDown:
		mov		#tblpage(UpsideDown_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(UpsideDown_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4UpsideDown_Exit
;
MatchACommand_UpsideDown:
		call	UpsideDown	
; 				
		goto	MatchACommand_Exit
;
MatchACommand_Chk4UpsideDown_Exit:	
;***************************************
MatchACommand_Chk4Print:
		mov		#tblpage(Print_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Print_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer		
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Print_Exit	; that's all, must not be a command
;
MatchACommand_Print:
		call	Print	
; 
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Print_Exit:
;***************************************	
MatchACommand_Chk4Test:
		mov		#tblpage(Test_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Test_Command), W1	; Make a pointer to table
		mov		#CommandBuffer,	W0
		mov		W0,		CommandBufferPointer
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchACommand_Chk4Test_Exit	; that's all, must not be a command
;
MatchACommand_Test:
		call	Test	
;		 
		goto	MatchACommand_Exit
;
MatchACommand_Chk4Test_Exit:		
;***************************************
MatchACommand_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Compare Strings			; look for a match between the selected
;										 string and the Command Buffer
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;		W1	
;	Changes
;		W0
;		W1
;		LCDFlags.Match
;	Calls
;		none
;
;***************************************;***************************************
;{		
CompareStrings:
		push.d	W2
		mov		CommandBufferPointer,	W3		; get the start of the Character Buffer	
;
CompareStrings_Loop:	
		tblrdl.b	[W1++],	W0				; get Command Character
		cp0.b	W0						; skip next if it's NOT the end (null)
		bra		Z,		CompareStrings_Match	; else it is, so we've matched
										; this command
;
		mov		W0,		W2				; put this character into W2
		mov		#'=',	W0				; '='	
		cp.b	W0,		[W3]			; compare '=' to CommandBuffer Character
		mov.b	[W3++],	W0				; get Character (no status change)
		bra		Z,		CompareStrings_Continue	; branch if Character is '='
		
		bclr	W0,		#5				; force these to UPPER CASE
;
CompareStrings_Continue:		
		cp.b	W0,		W2				; compare the two characters
		bra		NZ,		CompareStrings_Exit	; branch if they don't match
;
		goto	CompareStrings_Loop		; else they match try the next character
;
CompareStrings_Match:
		mov		W3,		CommandBufferPointer	; save the Command Buffer pointer
		bset	LCDFlags,	#Match
;
CompareStrings_Exit:
		pop.d	W2
		return
;}
;***************************************;***************************************
;
;	Subroutine: Pixel					- display TFTW_PixelColor @
;										 TFTW_PositionX and TFTW_PositionY
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;		TFTW_BitMapX
;		TFTW_BitMapY
;		TFTB_TextSize
;		TFTW_BitMapWidth
;		TFTW_BitMapHeight
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		LCDFlags.Match
;		TFTW_BitMapX
;		TFTW_BitMapY
;		TFTW_PixelColor
;		TFTW_CharacterX	
;		TFTW_CharacterY
;	Calls
;		MatchAColor
;
;***************************************;***************************************
;{		
Pixel:
		push.d	W2
		push.d	W4		
		clr		W4
 		mov		CommandBufferPointer,		W5		
;
Pixel_Loop:
 		mov.b	[W5++],	W0
 		mov.b	WREG,	U1TXCharacter
;
Pixel_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		Pixel_Done 
;
Pixel_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		Pixel_Illegal	; illegal 		
;
Pixel_Chk4LE9:
		mov		#'9',	W0				; 9
		cp.b	U1TXCharacter
		bra		LEU,		Pixel_Number	; 0-9, go on
;
Pixel_Chk4LTA:
		mov		#'A',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		Pixel_Illegal	; illegal		
;
Pixel_Chk4GTF:
		mov		#'F',	W0				; 9
		cp.b	U1TXCharacter
		bra		GTU,		Pixel_Illegal	; illegal		 		
;
Pixel_Alpha:
		mov		#7,	W0
		sub.b	U1TXCharacter			; adjust to : through ?
				
Pixel_Number:
		mov		#0x10,	W0
		mul.uu	W0,		W4,		W2
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		W0,		W2,		W4
		goto	Pixel_Loop
;		
Pixel_Chk4Name:
		call	MatchAColor
;
		btss	LCDFlags,	#Match	
		goto	Pixel_Illegal	; illegal				
;		
		mov		W0,		W4
;		
Pixel_Done:
		mov		W4,		TFTW_PixelColor	
		mov		TFTW_BitMapX,	W0
		mov		W0,		TFTW_CharacterX		
		mov		TFTW_BitMapY,	W0
		mov		W0,		TFTW_CharacterY		
		call	DrawPixel
;
		clr		W0
		mov.b	TFTB_TextSize,	WREG
		add		TFTW_BitMapX
;		
		clr		W0
		mov.b	TFTB_TextSize,	WREG
		mov		TFTW_BitMapWidth,	W1
		mul.uu	W0,		W1,		W0
		mov		TFTW_BitMapX,	W1
		cp		W1,		W0
		bra		GEU,	Pixel_Done_NewRow
;		
		goto	Pixel_Exit
;
Pixel_Done_NewRow:
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_BitMapX
		clr		W0
		mov.b	TFTB_TextSize,	WREG
		add		TFTW_BitMapY
;
		clr		W0
		mov.b	TFTB_TextSize,	WREG
		mov		TFTW_BitMapHeight,	W1
		mul.uu	W0,		W1,		W0
		mov		TFTW_BitMapY,	W1
		cp		W1,		W0
		bra		GEU,	Pixel_Done_TopRow
;
		goto	Pixel_Exit		
;
Pixel_Done_TopRow:
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_BitMapY		
		goto	Pixel_Exit
;		
Pixel_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	Pixel_Exit			 
;
Pixel_Exit:
		pop.d	W4
		pop.d	W2			
		return
;}
;***************************************;***************************************
;
;	Subroutine: FColor					- assign or send TFTB_TextColor
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		TEMPW
;		LCDFlags.Match
;		TFTW_TextColor
;	Calls
;		MatchAColor
;		SendMessage
;		SendHexWord
;		SendCRLF
;
;***************************************;***************************************
;{		
FColor:		
		push.d	W2
		push.d	W4		
		clr		W4
 		mov		CommandBufferPointer,		W5		
;
FColor_Loop:
 		mov.b	[W5++],	W0
 		mov.b	WREG,	U1TXCharacter
;
FColor_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		FColor_Done 
;
FColor_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		FColor_QM
;
FColor_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		FColor_Illegal	; illegal 		
;
FColor_Chk4LE9:
		mov		#'9',	W0				; 9
		cp.b	U1TXCharacter
		bra		LEU,		FColor_Number	; 0-9, go on
;
FColor_Chk4LTA:
		mov		#'A',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		FColor_Chk4Name	; possible color by name 		
;
FColor_Chk4GTF:
		mov		#'F',	W0				; 9
		cp.b	U1TXCharacter
		bra		GTU,		FColor_Chk4Name	; possible color by name		 		
;
FColor_Alpha:
		mov		#7,	W0
		sub.b	U1TXCharacter			; adjust to : through ?
				
FColor_Number:
		mov		#0x10,	W0
		mul.uu	W0,		W4,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		W0,		W4,		W4
		goto	FColor_Loop
;		
FColor_Chk4Name:
		call	MatchAColor
;
		btss	LCDFlags,	#Match	
		goto	FColor_Illegal	; illegal				
;		
		mov		W0,		W4
;		
FColor_Done:
		mov		W4,		TFTW_TextColor		
		goto	FColor_Exit
;		
FColor_QM:
		mov		#tblpage(FColor_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(FColor_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TFTW_TextColor,	WREG
		call	SendHexWord
;
		call	SendCRLF
;		
		goto	FColor_Exit		
;		
FColor_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	FColor_Exit			 
;
FColor_Exit:
		pop.d	W4
		pop.d	W2			
		return
;}
;***************************************;***************************************
;
;	Subroutine: BColor					- assign or send TFTB_FillColor
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		TEMPW
;		LCDFlags.Match
;		TFTW_FillColor
;	Calls
;		MatchAColor
;		SendMessage
;		SendHexWord
;		SendCRLF
;
;***************************************;***************************************
;{		
BColor:		
		push.d	W2
		push.d	W4		
		clr		W4
 		mov		CommandBufferPointer,		W5
;
BColor_Loop:
 		mov.b	[W5++],	W0
 		mov.b	WREG,	U1TXCharacter
;
BColor_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		BColor_Done 
;
BColor_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		BColor_QM
;
BColor_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		BColor_Illegal	; illegal 		
;
BColor_Chk4LE9:
		mov		#'9',	W0				; 9
		cp.b	U1TXCharacter
		bra		LEU,		BColor_Number	; 0-9, go on
;
BColor_Chk4LTA:
		mov		#'A',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		BColor_Chk4Name	; possible color by name 		
;
BColor_Chk4GTF:
		mov		#'F',	W0				; 9
		cp.b	U1TXCharacter
		bra		GTU,		BColor_Chk4Name	; possible color by name		 		
;
BColor_Alpha:
		mov		#7,	W0
		sub.b	U1TXCharacter			; adjust to : through ?
				
BColor_Number:
		mov		#0x10,	W0
		mul.uu	W0,		W4,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		W0,		W4,		W4
		goto	BColor_Loop
;		
BColor_Chk4Name:
		call	MatchAColor
;
		btss	LCDFlags,	#Match	
		goto	BColor_Illegal	; illegal				
;		
		mov		W0,		W4
;		
BColor_Done:
		mov		W4,		TFTW_FillColor		
		goto	BColor_Exit
;		
BColor_QM:
		mov		#tblpage(BColor_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BColor_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TFTW_FillColor,	WREG
		call	SendHexWord
;
		call	SendCRLF
;		
		goto	BColor_Exit		
;		
BColor_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	BColor_Exit			 
;
BColor_Exit:
		pop.d	W4
		pop.d	W2		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Match A Color			- look for color matches 
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		LCDFlags.Match
;	Changes
;		W0
;		W1
;	Calls
;		CompareStrings
;
;***************************************;***************************************
;{
MatchAColor:
		bclr	LCDFlags,		#Match
;***************************************
MatchAColor_Chk4BLACK:
		mov		#tblpage(Black_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Black_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4BLACK_Exit
;
MatchAColor_BLACK:
		mov		#TFT_Black,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4BLACK_Exit:
;***************************************
MatchAColor_Chk4BLUE:
		mov		#tblpage(Blue_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Blue_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4BLUE_Exit
;
MatchAColor_BLUE:
		mov		#TFT_Blue,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4BLUE_Exit:
;***************************************
MatchAColor_Chk4RED:
		mov		#tblpage(Red_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Red_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4RED_Exit
;
MatchAColor_RED:
		mov		#TFT_Red,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4RED_Exit:
;***************************************
MatchAColor_Chk4GREEN:
		mov		#tblpage(Green_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Green_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4GREEN_Exit
;
MatchAColor_GREEN:
		mov		#TFT_Green,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4GREEN_Exit:
;***************************************
MatchAColor_Chk4MAGENTA:
		mov		#tblpage(Magenta_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Magenta_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4MAGENTA_Exit
;
MatchAColor_MAGENTA:
		mov		#TFT_Magenta,	W0
		goto	MatchACommand_Exit
;
MatchAColor_Chk4MAGENTA_Exit:
;***************************************
MatchAColor_Chk4CYAN:
		mov		#tblpage(Cyan_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Cyan_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4CYAN_Exit
;
MatchAColor_CYAN:
		mov		#TFT_Cyan,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4CYAN_Exit:
;***************************************
MatchAColor_Chk4YELLOW:
		mov		#tblpage(Yellow_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Yellow_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4YELLOW_Exit
;
MatchAColor_YELLOW:
		mov		#TFT_Yellow,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4YELLOW_Exit:
;***************************************
MatchAColor_Chk4WHITE:
		mov		#tblpage(White_Color), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(White_Color), W1		; Make a pointer to table
		call	CompareStrings
;
		btss	LCDFlags,		#Match		
		goto	MatchAColor_Chk4WHITE_Exit
;
MatchAColor_WHITE:
		mov		#TFT_White,	W0
		goto	MatchAColor_Exit
;
MatchAColor_Chk4WHITE_Exit:
;***************************************
MatchAColor_Exit:
		return						
;}
;***************************************;***************************************
;
;	Subroutine: ATX						- assign or send TFTB_PositionX
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		TEMPW
;		CommandBufferPointer
;		U1TXCharacter
;		TFTW_PositionX
;		LCDFlags,	#Match
;	Calls
;		SendMessage
;		SendDecimalWord
;		SendCRLF
;
;***************************************;***************************************
;{		
ATX:		
		push.d	W2
		clr		TEMPW
;
ATX_Loop:
 		mov		CommandBufferPointer,		W1			
 		mov.b	[W1++],	W0
 		mov		W1,		CommandBufferPointer
 		mov.b	WREG,	U1TXCharacter
;
ATX_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		ATX_Done 
				;
ATX_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		ATX_QM
;
ATX_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		ATX_Illegal	; illegal 		
;
ATX_Chk4GT9:
		mov		#'9',	W0				; 9
		cp.b	U1TXCharacter
		bra		GTU,		ATX_Illegal	; illegal 		
;		
ATX_Number:
		mov		#10,	W0
		mov		TEMPW,	W1
		mul.uu	W0,		W1,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		TEMPW
		goto	ATX_Loop		
;		
ATX_Done:
		mov		Width,	W0
		mov		TEMPW,		W1
		cp		W1,		W0
		bra		GTU,	ATX_Illegal	
;
		mov		W1,		TFTW_PositionX		
		goto	ATX_Exit
ATX_QM:
		mov		#tblpage(ATX_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(ATX_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TFTW_PositionX,	WREG
		call	SendDecimalWord
;
		call	SendCRLF
;		
		goto	ATY_Exit		
;		
ATX_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	ATX_Exit			 
;
ATX_Exit:
		pop.d	W2	
		return		
;}
;***************************************;***************************************
;
;	Subroutine: ATY						- assign or send TFTB_PositionY
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		TEMPW
;		CommandBufferPointer
;		U1TXCharacter
;		TFTW_PositionY
;		LCDFlags,	#Match
;	Calls
;		SendMessage
;		SendDecimalWord
;		SendCRLF
;
;***************************************;***************************************
;{		
ATY:	
		push.d	W2
		clr		TEMPW
;
ATY_Loop:
 		mov		CommandBufferPointer,		W1			
 		mov.b	[W1++],	W0
 		mov		W1,		CommandBufferPointer
 		mov.b	WREG,	U1TXCharacter
;
ATY_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		ATY_Done 
;
ATY_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		ATY_QM
;
ATY_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		ATY_Illegal	; illegal 		
;
ATY_Chk4GT9:
		mov		#'9',	W0				; 9
		cp.b	U1TXCharacter
		bra		GTU,		ATY_Illegal	; illegal 		
;		
ATY_Number:
		mov		#10,	W0
		mov		TEMPW,	W1
		mul.uu	W0,		W1,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		TEMPW
		goto	ATY_Loop		
;		
ATY_Done:
		mov		Height,	W0
		mov		TEMPW,		W1
		cp		W1,		W0
		bra		GTU,	ATY_Illegal	
;
		mov		W1,		TFTW_PositionY		
		goto	ATY_Exit
;		
ATY_QM:
		mov		#tblpage(ATY_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(ATY_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TFTW_PositionY,	WREG
		call	SendDecimalWord
;
		call	SendCRLF
;		
		goto	ATY_Exit		
;		
ATY_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	ATY_Exit			 
;
ATY_Exit:
		pop.d	W2	
		return
;}
;***************************************;***************************************
;
;	Subroutine: BMP						- Get BMP file from SD Card and display
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		TEMPW
;		LCDFlags.Match
;	Calls
;		Search4File
;
;***************************************;***************************************
;{		
BMP:	
		push.d	W2
 		mov		CommandBufferPointer,	W1
 		mov		#StringBuffer,	W2
 		clr		W3
;
BMP_Loop:
 		mov.b	[W1],	W0
		cp.b	W0,		#0x0D
		bra		Z,		BMP_Next 		; no more characters		
; 		
 		mov.b	[W1++],	W0
 		btsc	W0,		#6				; skip next if upper case
 		bclr	W0,		#5				; else force to uppercase
 		mov.b	W0,		[W2++]
 		inc.b	W3,		W3
 		mov		#8,		W0
 		cpseq.b	W3,		W0				; maximum of 8 characters
		goto	BMP_Loop
;
BMP_Next:
 		mov		#StringBuffer+8,	W1
 		cpseq	W1,		W2
 		goto	BMP_Fill
 ;
 		mov		#'B',	W0
 		mov.b	W0,		[W2++]
 		mov		#'M',	W0
 		mov.b	W0,		[W2++] 		
 		mov		#'P',	W0
 		mov.b	W0,		[W2++] 
 		mov		#0x00,	W0
 		mov.b	W0,		[W2++] 				
 ;
 		call	Search4File			 	; try and find this file
					
		goto	BMP_Exit
;
BMP_Fill:
 		mov		#' ',	W0				; space
 		mov.b	W0,		[W2++]
 		goto	BMP_Next	
;		
BMP_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	BMP_Exit			 
;
BMP_Exit:
		pop.d	W2	
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Button					- display user's button request @
;										 TFTW_PositionX  and TFTW_PositionY 
;										 based on TFTW_TextSize, TFTW_Rotation,
;										 TFTW_TextColor and TFTW_FillColor
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		TEMPW
;		LCDFlags.Comma
;		LCDFlags.Match
;		CommandBufferPointer
;		U1TXCharacter
;	Calls
;		Display_Button
;
;***************************************;***************************************
;{		
Button:		
		push.d	W2
		clr		TEMPW
		bclr	LCDFlags,	#Comma
;
Button_Loop:
 		mov		CommandBufferPointer,		W1			
 		mov.b	[W1++],	W0
 		mov		W1,		CommandBufferPointer
 		mov.b	WREG,	U1TXCharacter
;
Button_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		Button_Done 
;
Button_Chk4Comma:
		mov		#0x2C,	W0				; ','
		cp.b	U1TXCharacter
		bra		Z,		Button_Comma 		
;
Button_Chk4LT0:
		mov		#'0',	W0				; '0'
		cp.b	U1TXCharacter
		bra		LTU,	Button_Illegal	; illegal 		
;
Button_Chk4GT9:
		mov		#'9',	W0				; '9'
		cp.b	U1TXCharacter
		bra		GTU,	Button_Illegal	; illegal 		
;		
Button_Number:
		mov		#10,	W0
		mov		TEMPW,	W1
		mul.uu	W0,		W1,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		TEMPW
		goto	Button_Loop
;
Button_Comma:
		btsc	LCDFlags,	#Comma
		goto	Button_Illegal	; multiple commas illegal
;		
		mov		#3,		W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		LTU,	Button_Illegal	; <3 illegal
;
		mov		#50,	W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		GTU,	Button_Illegal	; >50 illegal
;	
		mov		W1,		W0
		mov.b	WREG,	TFTB_KeypadRowCount	
		bset	LCDFlags,	#Comma
		clr		TEMPW	
		goto	Button_Loop			
;		
Button_Done:
		mov		#3,		W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		LTU,	Button_Illegal	; <3 illegal
;
		mov		#50,	W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		GTU,	Button_Illegal	; >50 illegal
		
		mov		W1,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
		call	Display_Button
;
		goto	Button_Exit								
;		
Button_Illegal:
 		bclr	LCDFlags,	#Match
 		goto	Button_Exit			 
;
Button_Exit:
		pop.d	W2	
		return
;}
;***************************************;***************************************
;
;	Subroutine: BitMap					- display user's bitmap request @
;										 TFTW_PositionX  and TFTW_PositionY 
;										 based on TFTW_TextSize, and
;										 TFTW_Rotation
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;		TFTW_PositionX
;		TFTW_PositionY	
;	Changes
;		W0
;		W1
;		TEMPW
;		CommandBufferPointer
;		U1TXCharacter
;		LCDFlags.Comma
;		LCDFlags.Match
;		TFTW_BitMapWidth
;		TFTW_BitMapHeight
;		TFTW_BitMapX
;		TFTW_BitMapY
;	Calls
;		none
;
;***************************************;***************************************
;{		
BitMap:		
		push.d	W2
		clr		TEMPW
		bclr	LCDFlags,	#Comma
;
BitMap_Loop:
 		mov		CommandBufferPointer,		W1			
 		mov.b	[W1++],	W0
 		mov		W1,		CommandBufferPointer
 		mov.b	WREG,	U1TXCharacter
;
BitMap_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		BitMap_Done 
;
BitMap_Chk4Comma:
		mov		#0x2C,	W0				; ','
		cp.b	U1TXCharacter
		bra		Z,		BitMap_Comma 		
;
BitMap_Chk4LT0:
		mov		#'0',	W0				; '0'
		cp.b	U1TXCharacter
		bra		LTU,	BitMap_Illegal	; illegal 		
;
BitMap_Chk4GT9:
		mov		#'9',	W0				; '9'
		cp.b	U1TXCharacter
		bra		GTU,	BitMap_Illegal	; illegal 		
;		
BitMap_Number:
		mov		#10,	W0
		mov		TEMPW,	W1
		mul.uu	W0,		W1,		W2
		mov		W2,		TEMPW
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		add		TEMPW
		goto	BitMap_Loop
;
BitMap_Comma:
		btsc	LCDFlags,	#Comma
		goto	BitMap_Illegal	; multiple commas illegal
;		
		mov		#1,		W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		LTU,	BitMap_Illegal	; <1 illegal
;
		mov		W1,		TFTW_BitMapWidth	
		bset	LCDFlags,	#Comma
		clr		TEMPW	
		goto	BitMap_Loop			
;		
BitMap_Done:
		mov		#1,		W0
		mov		TEMPW,	W1
		cp		W1,		W0
		bra		LTU,	BitMap_Illegal	; <1 illegal
;
		mov		W1,		TFTW_BitMapHeight
;
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_BitMapX
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_BitMapY		
;
		goto	BitMap_Exit								
;		
BitMap_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	BitMap_Exit			 
;
BitMap_Exit:
		pop.d	W2	
		return				
;}
;***************************************;***************************************
;
;	Subroutine: TextSize				- assign or send TFTB_TextSize
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		CommandBufferPointer
;		U1TXCharacter
;		TFTB_TextSize
;		LCDFlags.Match
;	Calls
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
TextSize:		
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
TextSize_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		TextSize_QM 		
;
TextSize_Chk4LT0:
		mov		#'0',	W0				; ?
		cp.b	U1TXCharacter
		bra		LTU,		TextSize_Illegal	; illegal 		
;
TextSize_Chk4GT9:
		mov		#'9',	W0				; ?
		cp.b	U1TXCharacter
		bra		GTU,		TextSize_Illegal	; illegal 		
;		
TextSize_Number:
		mov		#0x30,	W0
		sub.b	U1TXCharacter,	WREG
		mov.b	WREG,	TFTB_TextSize	
		goto	TextSize_Exit
;		
TextSize_QM:
		mov		#tblpage(TextSize_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(TextSize_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	TFTB_TextSize,	WREG
		add.b	#0x30,	W0
		mov.b	WREG,	U1TXCharacter	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	TextSize_Exit
;		
TextSize_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	TextSize_Exit			 
;
TextSize_Exit:
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Orientation				; assign or send TFTB_Rotation
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		CommandBufferPointer
;		U1TXCharacter
;		TFTB_TextSize
;		LCDFlags.Match
;		LCDFlags.OrientationDisplay
;	Calls
;		ChangeOrientation
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
Orientation:		
 		mov		CommandBufferPointer,	W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Orientation_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		Orientation_QM 		
;
		bclr	U1TXCharacter,	#5		; force upper case
Orientation_Chk4P:
		mov		#'P',	W0				; P
		cp.b	U1TXCharacter
		bra		Z,		Orientation_P		
;
Orientation_Chk4L:
		mov		#'L',	W0				; L
		cp.b	U1TXCharacter
		bra		Z,		Orientation_L
;		
		goto	Orientation_Illegal	; illegal 		
;		
Orientation_L:
		bset	LCDFlags,		#OrientationDisplay
		call	ChangeOrientation
;					
		goto	Orientation_Exit
;
Orientation_P:
		bclr	LCDFlags,		#OrientationDisplay
		call	ChangeOrientation
;										
		goto	Orientation_Exit
;		
Orientation_QM:
		mov		#tblpage(Orientation_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Orientation_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,		#OrientationDisplay	; skip next if Landscape
		goto	Orientation_QM_P
;
Orientation_QM_L:		
		mov		#tblpage(Landscape), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Landscape), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		goto	Orientation_Exit
;		
Orientation_QM_P:		
		mov		#tblpage(Portrait), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Portrait), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		goto	Orientation_Exit
;		
Orientation_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	Orientation_Exit			 
;
Orientation_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: ChangeOrientation			; rotate the Display image 90 degrees
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		TFT_IR
;		LCDFlags.OrientationDisplay
;		TFT_WR_STROBE_Value
;	Calls
;		Write_TFT	
;		Set_Rotation
;
;***************************************;***************************************
;{
ChangeOrientation:
		mov		#TFT_EntryModeRegister,	W0
		mov.b	WREG,	TFT_IR		
		call	Read_TFT
;		
		btss	TFT_RD_STROBE_Value,	#3	; skip next if Is Landscape
		goto	ChangeOrientation_IsP 
;
ChangeOrientation_IsL:
		btss	TFT_RD_STROBE_Value,	#4	; skip next if UpsideDown		
		goto	ChangeOrientation_IsL_RightsideUp
;
ChangeOrientation_IsL_UpsideDown:
		btsc	TFT_RD_STROBE_Value,	#5	; skip next if not Mirrored
		goto	ChangeOrientation_IsL_UpsideDown_Mirrored
;
ChangeOrientation_IsL_UpsideDown_NotMirrored:
		btsc	LCDFlags,	#OrientationDisplay	; skip next if requesting Portrait
		goto	ChangeOrientation_Exit	; else already Landscape
;		
ChangeOrientation_ChangeToP_UpsideDown_NotMirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0000,		W0
		ior		TFT_RD_STROBE_Value,	WREG
		mov		W0,		TFT_WR_STROBE_Value	; 0x0000
		goto	ChangeOrientation_DoIt		
;
ChangeOrientation_IsL_UpsideDown_Mirrored:
		btsc	LCDFlags,	#OrientationDisplay	; skip next if requesting Portrait
		goto	ChangeOrientation_Exit	; else already Landscape
;
ChangeOrientation_ChangeToP_UpsideDown_Mirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0010,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0010
		goto	ChangeOrientation_DoIt
;
ChangeOrientation_IsL_RightsideUp:		
		btsc	TFT_RD_STROBE_Value,	#5	; skip next if Mirrored		
		goto	ChangeOrientation_IsL_RightsideUp_NotMirrored
;
ChangeOrientation_IsL_RightsideUp_Mirrored:
		btsc	LCDFlags,	#OrientationDisplay	; skip next if requesting Portrait
		goto	ChangeOrientation_Exit	; else already Landscape
;
ChangeOrientation_ChangeToP_RightsideUp_Mirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0020,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0020
		goto	ChangeOrientation_DoIt
;		
ChangeOrientation_IsL_RightsideUp_NotMirrored:
		btsc	LCDFlags,	#OrientationDisplay	; skip next if requesting Portrait
		goto	ChangeOrientation_Exit	; else already Landscape
;
ChangeOrientation_ChangeToP_RightsideUp_NotMirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value		
		mov		#0x0030,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0030
		goto	ChangeOrientation_DoIt
;
ChangeOrientation_IsP:
		btsc	TFT_RD_STROBE_Value,	#5	; skip next if UpsideDown		
		goto	ChangeOrientation_IsP_RightsideUp
;
ChangeOrientation_IsP_UpsideDown:
		btsc	TFT_RD_STROBE_Value,	#4	; skip next if not Mirrored
		goto	ChangeOrientation_IsP_UpsideDown_Mirrored
;
ChangeOrientation_IsP_UpsideDown_NotMirrored:
		btss	LCDFlags,	#OrientationDisplay	; skip next if requesting Landscape
		goto	ChangeOrientation_Exit	; else already Portrait
;
ChangeOrientation_ChangeToL_UpsideDown_NotMirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0018,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0018
		goto	ChangeOrientation_DoIt		
;
ChangeOrientation_IsP_UpsideDown_Mirrored:
		btss	LCDFlags,	#OrientationDisplay	; skip next if requesting Landscape
		goto	ChangeOrientation_Exit	; else already Portrait
;
ChangeOrientation_ChangeToL_UpsideDown_Mirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0038,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0038
		goto	ChangeOrientation_DoIt
;
ChangeOrientation_IsP_RightsideUp:		
		btsc	TFT_RD_STROBE_Value,	#4	; skip next if Mirrored		
		goto	ChangeOrientation_IsP_RightsideUp_NotMirrored
;
ChangeOrientation_IsP_RightsideUp_Mirrored:
		btss	LCDFlags,	#OrientationDisplay	; skip next if requesting Landscape
		goto	ChangeOrientation_Exit	; else already Portrait
;
ChangeOrientation_ChangeaToL_RightsideUp_Mirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0008,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0008
		goto	ChangeOrientation_DoIt
;		
ChangeOrientation_IsP_RightsideUp_NotMirrored:
		btss	LCDFlags,	#OrientationDisplay	; skip next if requesting Landscape
		goto	ChangeOrientation_Exit	; else already Portrait
;
ChangeOrientation_ChangeaToL_RightsideUp_NotMirrored:
		mov		#0xFFC7,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value		
		mov		#0x0028,		W0
		ior		TFT_RD_STROBE_Value,	WREG		
		mov		W0,		TFT_WR_STROBE_Value	; 0x0028
		goto	ChangeOrientation_DoIt
;
ChangeOrientation_DoIt:
		mov		#TFT_EntryModeRegister,	W0
		mov.b	WREG,	TFT_IR		
		call	Write_TFT	
;
ChangeOrientation_Exit:
 		call	Set_Rotation
;							
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Wrap				; Enable/Disable Wrap
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		CommandBufferPointer
;		U1TXCharacter
;		TFTB_TextSize
;		LCDFlags.Match
;		LCDFlags.WrapAround
;	Calls
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
Wrap:		
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Wrap_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		Wrap_QM 		
;
Wrap_Chk4LT0:
		mov		#'0',	W0				; ?
		cp.b	U1TXCharacter
		bra		LTU,		Wrap_Illegal	; illegal 		
;
Wrap_Chk4GT9:
		mov		#'1',	W0				; ?
		cp.b	U1TXCharacter
		bra		GTU,		Wrap_Illegal	; illegal 		
;		
Wrap_Number:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		Wrap_Number_0
;
Wrap_Number_1:
		bset	LCDFlags,		#WrapAround					
		goto	Wrap_Exit
;		
Wrap_Number_0:
		bclr	LCDFlags,		#WrapAround					
		goto	Wrap_Exit
;		
Wrap_QM:
		mov		#tblpage(Wrap_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Wrap_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,		#WrapAround		; skip next if enabled
		goto	Wrap_QM_0
;
Wrap_QM_1:
		mov		#'1',	W0
		mov		W0,		U1TXCharacter
		goto	Wrap_QM_Continue
;		
Wrap_QM_0:		
		mov		#'0',	W0
		mov		W0,		U1TXCharacter
		goto	Wrap_QM_Continue
;
Wrap_QM_Continue:	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	Wrap_Exit
;		
Wrap_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	Wrap_Exit			 
;
Wrap_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Invert				; Invert the display colors
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		TFT_WR_STROBE_Value
;		LCDFlags.InvertDisplay
;		LCDFlags.Match
;	Calls
;		Read_TFT
;		Write_TFT
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
Invert:		
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Invert_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		Invert_QM 		
;
Invert_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		Invert_Illegal	; illegal 		
;
Invert_Chk4GT9:
		mov		#'1',	W0				; 1
		cp.b	U1TXCharacter
		bra		GTU,		Invert_Illegal	; illegal 		
;		
Invert_Number:
		mov		#TFT_BaseImageDisplayControlRegister,	W0
		mov.b	WREG,		TFT_IR
		call	Read_TFT
;
		mov		TFT_RD_STROBE_Value,	W0
		mov		#0xFFFC,		W1			
		and		W0,		W1,		W1
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		Invert_Number_0
;
Invert_Number_1:
		mov		#2,		W0				; set bit 1, clear bit 0			
		ior		W0,		W1,		W0
		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		bset	LCDFlags,		#InvertDisplay															
		goto	Invert_Exit
;		
Invert_Number_0:
		mov		#3,		W0				; set bit 1 and 0			
		ior		W0,		W1,		W0
		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		bclr	LCDFlags,		#InvertDisplay	
		goto	Invert_Exit
;		
Invert_QM:
		mov		#tblpage(Invert_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Invert_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,		#InvertDisplay	; skip next if enabled
		goto	Invert_QM_0
;
Invert_QM_1:
		mov		#'1',	W0
		mov		W0,		U1TXCharacter
		goto	Invert_QM_Continue
;		
Invert_QM_0:		
		mov		#'0',	W0
		mov		W0,		U1TXCharacter
		goto	Invert_QM_Continue
;
Invert_QM_Continue:	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	Invert_Exit
;		
Invert_Illegal:
 		bclr	LCDFlags,	#Match			
		goto	Invert_Exit			 
;
Invert_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: InvertToggle			; reverse  colors
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		TFT_IR
;		TFT_TFT_WR_STROBE_Value
;		LCDFlags.InvertDisplay
;	Calls
;		Read_TFT
;		Write_TFT
;
;***************************************;***************************************
;{		
InvertToggle:
		mov		#TFT_BaseImageDisplayControlRegister,	W0
		mov.b	WREG,		TFT_IR
		call	Read_TFT
;		
		mov		TFT_RD_STROBE_Value,	W0
		mov		#0xFFFC,		W1			
		and		W0,		W1,		W1
		btss	LCDFlags,		#InvertDisplay	; skip next if inverted	(FLAG=1)
		goto	InvertToggle_0
;
InvertToggle_1:
		mov		#3,		W0				; set bit 1 and 0			
		ior		W0,		W1,		W0
		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		bclr	LCDFlags,		#InvertDisplay
		goto	InvertToggle_Exit
;		
InvertToggle_0:
		mov		#2,		W0				; set bit 1, clear bit 0			
		ior		W0,		W1,		W0
		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		bset	LCDFlags,		#InvertDisplay
		goto	InvertToggle_Exit
;
InvertToggle_Exit:
		return					
;}
;***************************************;***************************************
;
;	Subroutine: BackLight				; Enable/Disable BackLight
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		LCDFlags.BackLightDisplay
;		LCDFlags.Match
;		LATB.BL
;	Calls
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
BackLight:		
 		mov		CommandBufferPointer,	W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
BackLight_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		BackLight_QM 		
;
BackLight_Chk4LT0:
		mov		#'0',	W0				; ?
		cp.b	U1TXCharacter
		bra		LTU,	BackLight_Illegal	; illegal 		
;
BackLight_Chk4GT9:
		mov		#'1',	W0				; ?
		cp.b	U1TXCharacter
		bra		GTU,	BackLight_Illegal	; illegal 		
;		
BackLight_Number:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		BackLight_Number_0
;
BackLight_Number_1:
		bset	LCDFlags,	#BackLightDisplay
		bset	LATB,	#BL																	
		goto	BackLight_Exit
;		
BackLight_Number_0:
		bclr	LCDFlags,	#BackLightDisplay
		bclr	LATB,	#BL			
		goto	BackLight_Exit
;		
BackLight_QM:
		mov		#tblpage(BackLight_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BackLight_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,	#BackLightDisplay	; skip next if enabled
		goto	BackLight_QM_0
;
BackLight_QM_1:
		mov		#'1',	W0
		mov		W0,		U1TXCharacter
		goto	BackLight_QM_Continue
;		
BackLight_QM_0:		
		mov		#'0',	W0
		mov		W0,		U1TXCharacter
		goto	BackLight_QM_Continue
;
BackLight_QM_Continue:	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	BackLight_Exit
;		
BackLight_Illegal:
 		bclr	LCDFlags,	#Match
 		goto	BackLight_Exit			 
;
BackLight_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Mirror				; Reverse the Display Image
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		LCDFlags.MirrorDisplay
;		LCDFlags.Match
;	Calls
;		ChangeMirror
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
Mirror:		
 		mov		CommandBufferPointer,	W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Mirror_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		Mirror_QM 		
;
Mirror_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,	Mirror_Illegal	; illegal 		
;
Mirror_Chk4GT9:
		mov		#'1',	W0				; 1
		cp.b	U1TXCharacter
		bra		GTU,	Mirror_Illegal	; illegal 		
;		
Mirror_Number:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		Mirror_Number_0
;
Mirror_Number_1:
		bset	LCDFlags,	#MirrorDisplay	; now mirror
		call	ChangeMirror
;		
		goto	Mirror_Exit		
;		
Mirror_Number_0:
		bclr	LCDFlags,	#MirrorDisplay	; now normal
		call	ChangeMirror
;					
		goto	Mirror_Exit
;		
Mirror_QM:
		mov		#tblpage(Mirror_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Mirror_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,	#MirrorDisplay	; skip next if enabled
		goto	Mirror_QM_0
;
Mirror_QM_1:
		mov		#'1',	W0
		mov		W0,		U1TXCharacter
		goto	Mirror_QM_Continue
;		
Mirror_QM_0:		
		mov		#'0',	W0
		mov		W0,		U1TXCharacter
		goto	Mirror_QM_Continue
;
Mirror_QM_Continue:	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	Mirror_Exit
;		
Mirror_Illegal:
 		bclr	LCDFlags,	#Match
 		goto	Mirror_Exit			 
;
Mirror_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: ChangeMirror			; mirror the Display image
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none	
;	Changes
;		W0
;		W1
;		TFT_IR
;		LCDFlags.MirrorDisplay
;		LCDFlags.Match
;		TFT_RD_STROBE_Value
;	Calls
;		Read_TFT
;		Write_TFT	
;		Set_Rotation
;
;***************************************;***************************************
;{
ChangeMirror:
		mov		#TFT_EntryModeRegister,		W0
		mov.b	WREG,	TFT_IR
		call	Read_TFT
;
		btsc	TFT_RD_STROBE_Value,	#3		; Is Portrait
		goto	ChangeMirror_IsL
;
ChangeMirror_IsP:
		btss	TFT_RD_STROBE_Value,	#5		; Is RightsideUp
		goto	ChangeMirror_IsP_UpsideDown
;
ChangeMirror_IsP_RightsideUp:
		btsc	TFT_RD_STROBE_Value,	#4		; Is Mirrored
		goto	ChangeMirror_IsP_RightsideUp_NotMirrored		
;
ChangeMirror_IsP_RightsideUp_Mirror:
		btsc	LCDFlags,	#MirrorDisplay	; skip next if Not Mirrored
		goto	ChangeMirror_Exit		;  else already Mirrored
;
ChangeMirror_ChangeToP_RightsideUp_NotMirror:		
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0030,	W0			; 0x0030
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt				
;
ChangeMirror_IsP_RightsideUp_NotMirrored:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	ChangeMirror_Exit		;  else already Not Mirrored
;
ChangeMirror_ChangeToP_RightsideUp_Mirror:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0020,	W0			; 0x0020
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt
;		
ChangeMirror_IsP_UpsideDown:		
		btss	TFT_RD_STROBE_Value,	#4		; Is Mirrored
		goto	ChangeMirror_IsP_UpsideDown_NotMirrored		
;
ChangeMirror_IsP_UpsideDown_Mirror:
		btsc	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	ChangeMirror_Exit		;  else already Mirrored
;
ChangeMirror_ChangeToP_UpsideDown_NotMirror:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0000,	W0			; 0x0000
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt	
;
ChangeMirror_IsP_UpsideDown_NotMirrored:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirror
		goto	ChangeMirror_Exit		;  else already Not Mirrored
;
ChangeMirror_ChangeToP_UpsideDown_Mirror:	
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value	
		mov		#0x0010,	W0			; 0x0010
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt	
;
ChangeMirror_IsL:
		btsc	TFT_RD_STROBE_Value,	#4		; Is RightsideUp
		goto	ChangeMirror_IsL_UpsideDown
;
ChangeMirror_IsL_RightsideUp:
		btsc	TFT_RD_STROBE_Value,	#5		; Is Mirror
		goto	ChangeMirror_IsL_RightsideUp_NotMirrored		
;
ChangeMirror_IsL_RightsideUp_Mirror:
		btsc	LCDFlags,	#MirrorDisplay	; skip next if Not Mirror
		goto	ChangeMirror_Exit		;  else already Mirrored
;
ChangeMirror_ChangeToL_RightsideUp_NotMirror:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0020,	W0			; 0x0020
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt				
;
ChangeMirror_IsL_RightsideUp_NotMirrored:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirror
		goto	ChangeMirror_Exit		;  else already NotMirrored
;
ChangeMirror_ChangeToL_RightsideUp_Mirror:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0000,	W0			; 0x0000
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt
;		
ChangeMirror_IsL_UpsideDown:		
		btss	TFT_RD_STROBE_Value,	#5		; Is Mirror
		goto	ChangeMirror_IsL_UpsideDown_NotMirrored		
;
ChangeMirror_IsL_UpsideDown_Mirror:
		btsc	LCDFlags,	#MirrorDisplay	; skip next if Not Mirror
		goto	ChangeMirror_Exit		;  else already Mirrored
;
ChangeMirror_ChangeToL_UpsideDown_NotMirror:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0010,	W0			; 0x0010
		ior		TFT_RD_STROBE_Value,	WREG
		goto	ChangeMirror_DoIt	
;
ChangeMirror_IsL_UpsideDown_NotMirrored:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirror
		goto	ChangeMirror_Exit		;  else already NotMirrored
;
ChangeMirror_ChangeToL_UpsideDown_Mirror:	
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0030,	W0			; 0x0030
		ior		TFT_RD_STROBE_Value,	WREG			
		goto	ChangeMirror_DoIt	
;		
ChangeMirror_DoIt:		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT	
;
ChangeMirror_Exit:
		call	Set_Rotation
;							
		return				
;}
;***************************************;***************************************
;
;	Subroutine: UpsideDown				; 180 degree rotation of the Display
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		LCDFlags.UpsideDownDisplay
;		LCDFlags.Match
;	Calls
;		ChangeUpsideDown
;		SendMessage
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
UpsideDown:		
 		mov		CommandBufferPointer,	W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
UpsideDown_Chk4QM:
		mov		#'?',	W0				; ?
		cp.b	U1TXCharacter
		bra		Z,		UpsideDown_QM 		
;
UpsideDown_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,	UpsideDown_Illegal	; illegal 		
;
UpsideDown_Chk4GT9:
		mov		#'1',	W0				; 1
		cp.b	U1TXCharacter
		bra		GTU,	UpsideDown_Illegal	; illegal 		
;		
UpsideDown_Number:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		UpsideDown_Number_0
;
UpsideDown_Number_1:
		bset	LCDFlags,	#UpsideDownDisplay	; request upsidedown
		call	ChangeUpsideDown
;
		goto	UpsideDown_Exit				
;		
UpsideDown_Number_0:
		bclr	LCDFlags,	#UpsideDownDisplay	; request rightsideup			
		call	ChangeUpsideDown
;
		goto	UpsideDown_Exit	
;		
UpsideDown_QM:
		mov		#tblpage(UpsideDown_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(UpsideDown_Command), W1	; Make a pointer to table
		call	SendMessage
;
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if enabled
		goto	UpsideDown_QM_0
;
UpsideDown_QM_1:
		mov		#'1',	W0
		mov		W0,		U1TXCharacter
		goto	UpsideDown_QM_Continue
;		
UpsideDown_QM_0:		
		mov		#'0',	W0
		mov		W0,		U1TXCharacter
		goto	UpsideDown_QM_Continue
;
UpsideDown_QM_Continue:	
		call	SendCharacter
;
		call	SendCRLF
;		
		goto	UpsideDown_Exit
;		
UpsideDown_Illegal:
 		bclr	LCDFlags,	#Match
 		goto	UpsideDown_Exit			 
;
UpsideDown_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: ChangeDown				; flip the Display image 180 degrees
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none	
;	Changes
;		W0
;		W1
;		TFT_IR
;		LCDFlags.UpsideDownDisplay
;		LCDFlags.Match
;		TFT_RD_STROBE_Value
;	Calls
;		Read_TFT
;		Write_TFT	
;		Set_Rotation
;
;***************************************;***************************************
;{
ChangeUpsideDown:
		mov		#TFT_EntryModeRegister,		W0
		mov.b	WREG,	TFT_IR
		call	Read_TFT
;
		btsc	TFT_RD_STROBE_Value,	#3		; Is Portrait 
		goto	ChangeUpsideDown_IsL
;
ChangeUpsideDown_IsP:
		btsc	TFT_RD_STROBE_Value,	#5		; Is UpsideDown 
		goto	ChangeUpsideDown_IsP_RightsideUp
;
ChangeUpsideDown_IsP_UpsideDown:
		btss	TFT_RD_STROBE_Value,	#4		; Is Mirrored
		goto	ChangeUpsideDown_IsP_UpsideDown_NotMirrored
;
ChangeUpsideDown_IsP_UpsideDown_Mirrored:
		btsc	LCDFlags,	#UpsideDownDisplay	; skip next if RightsideUp
		goto	ChangeMirror_Exit		;  else already UpsideDown
;		
ChangeUpsideDown_ChangeToP_RightsideUp_Mirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0020,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0020
		goto	ChangeUpsideDown_DoIt		

ChangeUpsideDown_IsP_UpsideDown_NotMirrored:
		btsc	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	ChangeMirror_Exit		;  else already RightsideUp
;		
ChangeUpsideDown_ChangeToP_RightsideUp_NotMirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0030,	W0				
		ior		TFT_RD_STROBE_Value,	WREG		; 0x0030
		goto	ChangeUpsideDown_DoIt				
;
ChangeUpsideDown_IsP_RightsideUp:
		btsc	TFT_RD_STROBE_Value,	#4		; Is Mirrored
		goto	ChangeUpsideDown_IsP_RightsideUp_NotMirrored
;
ChangeUpsideDown_IsP_RightsideUp_Mirrored:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if RightsideUp
		goto	ChangeMirror_Exit		;  else already UpsideDown
;		
ChangeUpsideDown_ChangeToP_UpsideDown_Mirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0010,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0010
		goto	ChangeUpsideDown_DoIt

ChangeUpsideDown_IsP_RightsideUp_NotMirrored:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	ChangeMirror_Exit		;  else already RightsideUp
;		
ChangeUpsideDown_ChangeToP_UpsideDown_NotMirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value	
		mov		#0x0000,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0000
		goto	ChangeUpsideDown_DoIt
;
ChangeUpsideDown_IsL:
		btss	TFT_RD_STROBE_Value,	#4		; Is UpsideDown
		goto	ChangeUpsideDown_IsL_RightsideUp
;
ChangeUpsideDown_IsL_UpsideDown:
		btss	TFT_RD_STROBE_Value,	#5		; Is Mirrored
		goto	ChangeUpsideDown_IsL_UpsideDown_NotMirrored
;
ChangeUpsideDown_IsL_UpsideDown_Mirrored:
		btsc	LCDFlags,	#UpsideDownDisplay	; skip next if RightsideUp
		goto	ChangeMirror_Exit		;  else already UpsideDown
;
ChangeUpsideDown_ChangeToL_RightsideUp_Mirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0000,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0000
		goto	ChangeUpsideDown_DoIt		

ChangeUpsideDown_IsL_UpsideDown_NotMirrored:
		btsc	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	ChangeMirror_Exit		;  else already RightsideUp
;		
ChangeUpsideDown_ChangeToL_RightsideUp_NotMirrored:	
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value	
		mov		#0x0020,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0020
		goto	ChangeUpsideDown_DoIt				
;
ChangeUpsideDown_IsL_RightsideUp:
		btsc	TFT_RD_STROBE_Value,	#5		; Is UpsideDown
		goto	ChangeUpsideDown_IsL_RightsideUp_NotMirrored
;
ChangeUpsideDown_IsL_RightsideUp_Mirrored:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if RightsideUp
		goto	ChangeMirror_Exit		;  else already UpsideDown
;
ChangeUpsideDown_ChangeToL_UpsideDown_Mirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value
		mov		#0x0030,	W0
		ior		TFT_RD_STROBE_Value,	WREG		; 0x0030
		goto	ChangeUpsideDown_DoIt

ChangeUpsideDown_IsL_RightsideUp_NotMirrored:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	ChangeMirror_Exit		;  else already RightsideUp
;
ChangeUpsideDown_ChangeToL_UpsideDown_NotMirrored:
		mov		#0xFFCF,	W0			; mask of good bits
		and		TFT_RD_STROBE_Value	
		mov		#0x0010,	W0
		ior		TFT_RD_STROBE_Value,	WREG	; 0x0010
		goto	ChangeUpsideDown_DoIt
;
ChangeUpsideDown_DoIt:		
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT	
;
ChangeUpsideDown_Exit:
		call	Set_Rotation
;														
		return										
;}
;***************************************;***************************************
;
;	Subroutine: Display				; Enable/Disable Display
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer	
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		TFT_IR
;		LCDFlags.Match
;		TFT_WR_STROBE_Value
;	Calls
;		Read_TFT
;		Write_TFT
;
;***************************************;***************************************
;{		
Display:		
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Display_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		Display_Illegal	; illegal 		
;
Display_Chk4GT9:
		mov		#'1',	W0				; 1
		cp.b	U1TXCharacter
		bra		GTU,		Display_Illegal	; illegal 		
;		
Display_Number:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		Z,		Display_Number_0
;
Display_Number_1:
		mov		#7,		W0
		mov		W0,		TFT_IR
		call	Read_TFT
;
		mov		TFT_RD_STROBE_Value,	W0
		mov		#3,		W1
		ior		W0,		W1,		W0
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;							
		goto	Display_Exit
;		
Display_Number_0:
		mov		#7,		W0
		mov		W0,		TFT_IR
		call	Read_TFT
;
		mov		TFT_RD_STROBE_Value,	W0
		mov		#0xFFFD,	W1
		and		W0,		W1,		W0
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;							
		goto	Display_Exit
;		
Display_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	Display_Exit			 
;
Display_Exit:
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Print					- display user's string from
;										 @CommandBufferPointer to TFTW_PositionX  
;										 and TFTW_PositionY based on
; 										 TFTW_TextSize, TFTW_Rotation,
;										 TFTW_TextColor and TFTW_FillColor
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;		TFTW_PositionX
;		TFTW_PositionY
;	Changes
;		W0
;		W1
;		U1TXCharacter
;		CommandBufferPointer
;		TFTW_X
;		TFTW_Y
;		TFTB_Character
;	Calls
;		DisplayACharacter
;
;***************************************;***************************************
;{		
Print:	
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
 		mov		W1,		CommandBufferPointer
 ;		
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y 		
;
Print_Chk4CR:
		mov		#0x0D,	W0				; CR
		cp.b	U1TXCharacter
		bra		Z,		Print_Exit
;
		mov.b	U1TXCharacter,	WREG
		mov.b	WREG,	TFTB_Character
		call	DisplayACharacter
;
		goto	Print								
;
Print_Exit:	
		return
;}
;***************************************;***************************************
;
;	Subroutine: Test 					- perform testing routines
;
;										 0=Display Character Set
;										 1=Display a numeric keypad
;										 2=TFT LCD Register Dump 
;										 3=SD Card Directory
;										 4=Calibrate the Touchscreen
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		CommandBufferPointer
;	Changes
;		W0
;		W1
;		CommandBufferPointer
;		U1TXCharacter
;		LCDFlags.Match
;	Calls
;		ClearScreen
;		Display_Character_Set
;		Display_Keypad
;		Send_Registers
;		Send_Files
;		CalibrateScreen
;
;***************************************;***************************************
;{		
Test:		
 		mov		CommandBufferPointer,		W1
 		mov.b	[W1++],		W0
 		mov.b	WREG,	U1TXCharacter
;
Test_Chk4LT0:
		mov		#'0',	W0				; 0
		cp.b	U1TXCharacter
		bra		LTU,		Test_Illegal	; illegal 		
;
Test_Chk4GT9:
		mov		#'4',	W0				; 1
		cp.b	U1TXCharacter
		bra		GTU,		Test_Illegal	; illegal 		
;		
Test_Chk4_0:
		mov		#0x30,	W0
		cp.b	U1TXCharacter
		bra		NZ,		Test_Chk4_1
;
Text_0:
		call	ClearScreen
;		
		call	Display_Character_Set
;			
		goto	Test_Exit
;
Test_Chk4_1:
		mov		#0x31,	W0
		cp.b	U1TXCharacter
		bra		NZ,		Test_Chk4_2
;		
Test_1:		
		call	ClearScreen
;		
		call	Display_Keypad
;				
		goto	Test_Exit
;
Test_Chk4_2:	
		mov		#0x32,	W0
		cp.b	U1TXCharacter
		bra		NZ,		Test_Chk4_3
;		
Test_2:		
		call	Send_Registers
;				
		goto	Test_Exit
;
Test_Chk4_3:
		mov		#0x33,	W0
		cp.b	U1TXCharacter
		bra		NZ,		Test_Chk4_4
;		
Test_3:		
		call	Send_Files
;				
		goto	Test_Exit
;
Test_Chk4_4:							
		mov		#0x34,	W0
		cp.b	U1TXCharacter
		bra		NZ,		Test_Chk4_5
;		
Test_4:		
		call	CalibrateScreen
;				
		goto	Test_Exit
;
Test_Chk4_5:							; illegal		
;		
Test_Illegal:
 		bclr	LCDFlags,	#Match		
		goto	Test_Exit			 
;
Test_Exit:
		return										
;}
;***************************************;***************************************
;
;	Subroutine: Reset of TFT LCD
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none
;	Changes
;		W0
;		LATA.RST
;	Calls
;		Delay
;
;***************************************;***************************************
;{
Reset_TFT:
		bclr	LATA,	#RST
;
		mov		#63,	W0				; 16uS * 63 = 1mS
		call	Delay
;
		bset	LATA,	#RST
;
		mov		#2500,	W0				; 16uS * 2500 = 40mS	
		call	Delay
;
Reset_TFT_Exit:
		return
;}		
;***************************************;***************************************
;
;	Subroutine: Delay	(16mS * PR1)
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W0	=	PR1 value
;	Changes
;		W0
;		TMR1
;		PR1
;		IFS0.T1IF
;	Calls
;		none
;
;***************************************;***************************************
;{
Delay:		
		mov		W0,		PR1				; 16*63=1mS
		clr		TMR1
		bclr	IFS0,	#T1IF			; clear any interrupt	
;
Delay_Loop:	
		btss	IFS0,	#T1IF			; skip next if Timer 1 rollover
		goto	Delay_Loop
;
Delay_Exit:
		return				
;}
;***************************************;***************************************
;
;	Subroutine: Write to the Index Register
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		DataBus as Outputs
;		TFT_IR
;	Changes
;		W0
;		LATB.CD
;		LATB
;		LATB.WR_STROBE
;	Calls
;		none
;
;***************************************;***************************************
;{
Write_IR_TFT:
		bclr	LATB,	#CD				; control
		nop
;
Write_IR_TFT_UpperByte:		
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov.b	#0,		W0				; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;		
		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
		nop
;
Write_IR_TFT_LowerByte:		
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov.b	TFT_IR,	WREG			; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;		
		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
		nop
;
Write_IR_TFT_Exit:
		bset	LATB,	#CD				; data
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Write to a Register
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TFT_IR
;		TFT_WR_STROBE_Value
;	Changes
;		W0
;		W1
;		TRISB
;		LATB.CS
;		LATB
;		LATB.WR_STROBE
;	Calls
;		Write_IR_TFT
;
;***************************************;***************************************
;{
Write_TFT:
		mov		TRISB,	W1				; get direction bits
		mov		#0x00FF,	W0			; get mask 
		and		W0,		W1,		W1		; force high byte as outputs
		mov		W1,		TRISB			; replace direction bits
;
		nop
		bclr	LATB,	#CS		
		call	Write_IR_TFT
;		
Write_TFT1:
;
Write_TFT_UpperByte:		
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov		TFT_WR_STROBE_Value,	W1		; get word to write
		swap.w	W1						; swap bytes
		mov.b	W1,		W0				; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;		
		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
		nop
;
Write_TFT_LowerByte:		
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov.b	TFT_WR_STROBE_Value,	WREG	; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;		
		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
		nop
;
Write_TFT_Exit:
		bset	LATB,	#CS
		return
;}		
;***************************************;***************************************
;
;	Subroutine: Read from a Register
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none
;	Changes
;		W0
;		W1
;		TRISB
;		LATB.CS	
;		LATB.RD_STROBE
;		TFT_RD_STROBE_Value
;	Calls
;		none
;
;***************************************;***************************************
;{
Read_TFT:
		mov		TRISB,	W1				; get direction bits
		mov		#0x00FF,	W0			; get mask 
		and		W0,		W1,		W1		; force high byte as outputs
		mov		W1,		TRISB			; replace direction bits
		nop
		bclr	LATB,	#CS		
		call	Write_IR_TFT
;		
		mov		TRISB,	W1				; get direction bits
		mov		#0xFF00,	W0			; get mask 
		ior		W0,		W1,		W1		; force high byte as inputs
		mov		W1,		TRISB			; replace direction bits
;
Read_TFT1:
;
Read_TFT_UpperByte:	
		nop
		bclr	LATB,	#RD_STROBE		; read strobe
		nop
		mov		PORTB,	W0				; get port b
		bset	LATB,	#RD_STROBE		; read strobe
		nop
;			
		mov		W0,		TFT_RD_STROBE_Value	; save top byte (MSB read value)
;
Read_TFT_LowerByte:	
		nop
		bclr	LATB,	#RD_STROBE		; read strobe
		nop
		mov		PORTB,	W0				; get port b		
		bset	LATB,	#RD_STROBE		; read strobe
		nop
;			
		swap.w	W0						; swap bytes
		mov.b	WREG,	TFT_RD_STROBE_Value	; replace lower byte (LSB read value)
;
Read_TFT_Exit:
		nop
		bset	LATB,	#CS
		return	
;}		
;***************************************;***************************************
;
;	Subroutine: Display a Register
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TFT_IR
;	Changes
;		W0
;	Calls
;		SendMessage
;		SendHexWord
;		SendCharacter
;		SendCRLF
;
;***************************************;***************************************
;{		
DisplayRegister:
		mov		#tblpage(Register), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Register), W1	; Make a pointer to table
		call	SendMessage
;
		clr		W0
		mov.b	TFT_IR,	WREG
		call	SendHexWord
;
		mov.b	#' ',	W0
		call	SendCharacter
;
		mov		#tblpage(Value), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Value), W1	; Make a pointer to table
		call	SendMessage
;
		mov		TFT_RD_STROBE_Value,	WREG
		call	SendHexWord
;
		call	SendCRLF			
;
DisplayRegister_Exit:
		return			
;}
;***************************************;***************************************
;
;	Subroutine: Send a Message	-	ROM string to U1TXBuffer (Terminal Display)
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W1	=	Message Pointer
;	Changes
;		U1TXCharacter
;	Calls
;		U1TXBuffer_Add 
;
;***************************************;***************************************
;{
SendMessage:
		push.d	W0
;		
SendMessage_Loop: 
		tblrdl.b	[W1++],	W0			; Load the first data value
		cp0.b	W0
		bra		Z,	SendMessage_Exit
;
		mov.b	WREG,	U1TXCharacter
		call	U1TXBuffer_Add
;
		goto	SendMessage_Loop	
;		
SendMessage_Exit:
		pop.d	W0
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send a RAM Message		- RAM string to U1TXBuffer
;										  (Terminal Display)
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W1	=	Message Pointer 
;	Changes
;		U1TXCharacter
;	Calls
;		U1TXBuffer_Add
;
;***************************************;***************************************
;{
SendRAMMessage:
		push.d	W0
;		
SendRAMMessage_Loop: 
		mov.b	[W1++],	W0			; get the data value
		cp0.b	W0
		bra		Z,	SendRAMMessage_Exit
;
		mov.b	WREG,	U1TXCharacter
		call	U1TXBuffer_Add
;
		goto	SendRAMMessage_Loop	
;		
SendRAMMessage_Exit:
		pop.d	W0
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Send Hex Word to U1TXBuffer	
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W0	=	Value
;	Changes
;		W0
;		W1
;		TEMPW
;	Calls
;		SendCharacter 
;
;***************************************;***************************************
;{
SendHexWord:
		mov		W0,		TEMPW
		swap.w	W0
		swap.b	W0
		and		#0x000F,	W0
		add		#0x30,	W0
		mov		#0x39,	W1
		cpsgt	W0,		W1		
		goto	SendHexWord_HByte_HNibble
;
		mov		#7,		W1
		add		W1,		W0,		W0
;		
SendHexWord_HByte_HNibble:
		call	SendCharacter
;
		mov		TEMPW,	W0
		swap.w	W0
		and		#0x000F,	W0
		add		#0x30,	W0
		mov		#0x39,	W1
		cpsgt	W0,		W1		
		goto	SendHexWord_HByte_LNibble
;
		mov		#7,		W1
		add		W1,		W0,		W0
;		
SendHexWord_HByte_LNibble:
		call	SendCharacter
;
		mov		TEMPW,	W0
		swap.b	W0
		and		#0x000F,	W0
		add		#0x30,	W0
		mov		#0x39,	W1
		cpsgt	W0,		W1		
		goto	SendHexWord_LByte_HNibble
;
		mov		#7,		W1
		add		W1,		W0,		W0
;		
SendHexWord_LByte_HNibble:
		call	SendCharacter
;
		mov		TEMPW,	W0
		and		#0x000F,	W0
		add		#0x30,	W0
		mov		#0x39,	W1
		cpsgt	W0,		W1		
		goto	SendHexWord_LByte_LNibble
;
		mov		#7,		W1
		add		W1,		W0,		W0
;		
SendHexWord_LByte_LNibble:
		call	SendCharacter
;
SendHexWord_Exit:		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send Decimal Word to U1TXBuffer	
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W0	=	Value
;	Changes
;		W0
;		W1
;	Calls
;		SendCharacter 
;
;***************************************;***************************************
;{
SendDecimalWord:
		push.d	W2
		mov		W0,		W3				; save the value in W3
;
SendDecimalWord_Loop:		
		mov		W3,		W0				; get the value
		mov		#10,	W2					
		cp		W0,		W2				; compare the value to 10
		bra		LTU,	SendDecimalWord_Last	; branch if value<10
;
		repeat 	#17 					; Execute DIV.U 18 times			
		div.u	W0,		W2				; divide value by 10	
		mov		W1,		W3				; save the remainder 
		add		#0x30,	W0				; make dividend ASCII
		call	SendCharacter			; send digit
;
		goto	SendDecimalWord_Loop				
;		
SendDecimalWord_Last:
		add		#0x30,	W0				; make it ASCII
		call	SendCharacter			; send digit
;
SendDecimalWord_Exit:
		pop.d	W2		
		return		
;}
;***************************************;***************************************
;
;	Subroutine: Send a Character to U1TXBuffer	-	(Terminal Display)
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		W0	=	Character
;	Changes
;		U1TXCharacter
;	Calls
;		U1TXBuffer_Add 
;
;***************************************;***************************************
;{
SendCharacter:
		mov.b	WREG,	U1TXCharacter
		call	U1TXBuffer_Add
;		
		return
;}
;***************************************;***************************************
;
;	Subroutine: Send a CRLF to U1TXBuffer	-	(Terminal Display)
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		none
;	Changes
;		W0
;	Calls
;		SendCharacter 
;
;***************************************;***************************************
;{
SendCRLF:
		push	W0		
		mov.b	#0x0D,	W0
		call	SendCharacter
;						
		mov.b	#0x0A,	W0
		call	SendCharacter
;
		pop		W0
		return
;}
;***************************************;***************************************
;
;	Subroutine: Set Rotation			- set LCD Flags Bits
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_Rotation
;	Changes:
;		W0
;		W1
;		TFT_IR
;		LCDFlags.OrientationDisplay
;		LCDFlags.UpsideDownDisplay
;		LCDFlags.MirrorDisplay
;		Width
;		Height
; 	Calls:
;		Read_TFT
;		Write_TFT
;
;***************************************;***************************************
;{		
Set_Rotation:
		mov		#TFT_EntryModeRegister,	W0
		mov.b	WREG,	TFT_IR
		call	Read_TFT
;				
		btsc	TFT_RD_STROBE_Value,	#3		; skip next if Portrait
		goto	Set_Rotation_InL
;
Set_Rotation_InP:
		mov		#TFT_PHeight,	W0
		mov		W0,		Height
		mov		#TFT_PWidth,	W0
		mov		W0,		Width	
		bclr	LCDFlags,	#OrientationDisplay	; Portrait Flag
		btsc	TFT_RD_STROBE_Value,	#5	; skip next if UpsideDown		
		goto	Set_Rotation_InP_RightsideUp		
		
Set_Rotation_InP_UpsideDown:
		bset	LCDFlags,	#UpsideDownDisplay	; UpsideDown Flag
		btss	TFT_RD_STROBE_Value,	#4		; skip next if Mirrored
		goto	Set_Rotation_InP_UpsideDown_NotMirrored
;
Set_Rotation_InP_UpsideDown_Mirrored:
		bset	LCDFlags,	#MirrorDisplay	; Mirrored Flag
		goto	Set_Rotation_Exit
		
Set_Rotation_InP_UpsideDown_NotMirrored:
		bclr	LCDFlags,	#MirrorDisplay	; NotMirrored Flag
		goto	Set_Rotation_Exit						

Set_Rotation_InP_RightsideUp:
		bclr	LCDFlags,	#UpsideDownDisplay	; RightsideUp Flag
		btsc	TFT_RD_STROBE_Value,	#4		; skip next if Mirrored		
		goto	Set_Rotation_InP_RightsideUp_NotMirrored
;
Set_Rotation_InP_RightsideUp_Mirrored:
		bset	LCDFlags,	#MirrorDisplay	; Mirrored Flag
		goto	Set_Rotation_Exit
		
Set_Rotation_InP_RightsideUp_NotMirrored:
		bclr	LCDFlags,	#MirrorDisplay	; NotMirrored Flag
		goto	Set_Rotation_Exit
;				
Set_Rotation_InL:
		mov		#TFT_PHeight,	W0
		mov		W0,		Width
		mov		#TFT_PWidth,	W0
		mov		W0,		Height	
		bset	LCDFlags,	#OrientationDisplay	; Landscape Flag
		btss	TFT_RD_STROBE_Value,	#4	; skip next if UpsideDown		
		goto	Set_Rotation_InL_RightsideUp		
		
Set_Rotation_InL_UpsideDown:
		bset	LCDFlags,	#UpsideDownDisplay	; UpsideDown Flag
		btss	TFT_RD_STROBE_Value,	#5		; skip next if Mirrored
		goto	Set_Rotation_InL_UpsideDown_NotMirrored
;
Set_Rotation_InL_UpsideDown_Mirrored:
		bset	LCDFlags,	#MirrorDisplay	; Mirrored Flag
		goto	Set_Rotation_Exit
		
Set_Rotation_InL_UpsideDown_NotMirrored:
		bclr	LCDFlags,	#MirrorDisplay	; NotMirrored Flag
		goto	Set_Rotation_Exit						

Set_Rotation_InL_RightsideUp:
		bclr	LCDFlags,	#UpsideDownDisplay	; RightsideUp Flag
		btsc	TFT_RD_STROBE_Value,	#5		; skip next if Mirrored		
		goto	Set_Rotation_InL_RightsideUp_NotMirrored
;
Set_Rotation_InL_RightsideUp_Mirrored:
		bset	LCDFlags,	#MirrorDisplay	; Mirrored Flag
		goto	Set_Rotation_Exit
		
Set_Rotation_InL_RightsideUp_NotMirrored:
		bclr	LCDFlags,	#MirrorDisplay	; NotMirrored Flag
		goto	Set_Rotation_Exit
;				
Set_Rotation_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Set Address Window		- sets up TFTW_X1, TFTW_X2, TFTW_X,
;										 TFTW_Y1, TFTW_Y2, TFTW_Y based on
;										 Orientation, Upsidedown, Mirror, 
;										 TFTW_X1, TFTW_X2, TFTW_Y1, and TFTW_Y2
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		LCDFlags.Orientation
;		LCDFlags.Upsidedown
;		LCDFlags.Mirror
;		TFTW_X1
;		TFTW_X2
;		TFTW_Y1
;		TFTW_Y2
;	Changes:
;		W0
;		W1
;		LCDFlags.BMP2Large
;		TFTW_X1
;		TFTW_X2
;		TFT_X
;		TFTW_Y1
;		TFTW_Y2
;		TFTW_Y
;		TFT_IR
;		TFT_WR_STROBE_Value
;		TFT_HorizontalAddressStartPositionRegister
;		TFT_HorizontalAddressEndPositionRegister
;		TFT_VerticalAddressStartPositionRegister
;		TFT_VerticalAddressEndPositionRegister
;		TFT_HorizontalGRAMAddressRegister
;		TFT_VerticalGRAMAddressRegister
; 	Calls:
;		AdjustY
;		AdjustX 
;		Write_TFT
;
;***************************************;***************************************
;{
Set_Address_Window:	
		bclr	LCDFlags,	#BMP2Large	; initialize as OK
		call	CheckSize
;
		btsc	LCDFlags,	#BMP2Large	; skip next if not too large for display
		goto	Set_Address_Window_Exit
;		
		btsc	LCDFlags,	#OrientationDisplay	; skip next if Portrait		
		goto	Set_Address_Window_InL
;
Set_Address_Window_InP:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	Set_Address_Window_InP_RightSideUp
;
Set_Address_Window_InP_UpsideDown:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	Set_Address_Window_InP_UpsideDown_NotMirrored
;		
Set_Address_Window_InP_UpsideDown_Mirrored:
		call	AdjustY
;
		goto	Set_Address_Window_Rotation2
;
Set_Address_Window_InP_UpsideDown_NotMirrored:
		call	AdjustY
;
		call	AdjustX
;
		goto	Set_Address_Window_Rotation2
;
Set_Address_Window_InP_RightSideUp:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	Set_Address_Window_InP_RightSideUp_NotMirrored
;		
Set_Address_Window_InP_RightSideUp_Mirrored:
		call	AdjustX
;		
		goto	Set_Address_Window_Rotation0
;
Set_Address_Window_InP_RightSideUp_NotMirrored:
		goto	Set_Address_Window_Rotation0
;
Set_Address_Window_InL:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if UpsideDown
		goto	Set_Address_Window_InL_RightSideUp
;
Set_Address_Window_InL_UpsideDown:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	Set_Address_Window_InL_UpsideDown_NotMirrored
;		
Set_Address_Window_InL_UpsideDown_Mirrored:
		call	AdjustY
;
		goto	Set_Address_Window_Rotation3
;
Set_Address_Window_InL_UpsideDown_NotMirrored:
		call	AdjustX
;
		call	AdjustY
;
		goto	Set_Address_Window_Rotation3
;
Set_Address_Window_InL_RightSideUp:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Mirrored
		goto	Set_Address_Window_InL_RightSideUp_NotMirrored
;		
Set_Address_Window_InL_RightSideUp_Mirrored:
		call	AdjustX
;
		goto	Set_Address_Window_Rotation1
;
Set_Address_Window_InL_RightSideUp_NotMirrored:
		goto	Set_Address_Window_Rotation1
;
Set_Address_Window_Rotation0:			; Normal orientation portrait 
										; widest black border on bottom
		mov		TFTW_X1,	W0			; get TXTW_X1
		mov		W0,		W1				; X=TXTW_X1
;		
		mov		TFTW_Y1,	W0			; get TXTW_Y1
		mov		W0,		W2				; Y=TXTW_Y1
		goto	Set_Address_Window_Continue
;
Set_Address_Window_Rotation1:			; Normal orientation Landscape
										; widest black border on right
										; swap TFT_PHeight for TFT_PWidth										
		mov		TFTW_Y1,	W3			; temp=TFTW_Y1
;
		mov		TFTW_X1,	W0			; get TXTW_X1
		mov		W0,		TFTW_Y1			; TFTW_Y1=TXTW_X1
;		
		mov		Height,	W0				 
		mov		W0,		TFTW_X1			; TFTW_X1=Height
		dec		TFTW_X1					; TFTW_X1=TFTW_X1-1
		mov		TFTW_Y2,	W0			; get TFTW_Y2
		sub		TFTW_X1					; TFTW_X1=TFTW_X1-TFTW_Y2
;		
		mov		TFTW_X2,	W0			; get TFTW_X2
		mov		W0,		TFTW_Y2			; TFTW_Y2=TFTW_X2
;
		mov		Height,	W0			
		mov		W0,		TFTW_X2			; TFTW_X2=Height	
		dec		TFTW_X2					; TFTW_X2=TFTW_X2-1
		mov		W3,	W0					; get temp
		sub		TFTW_X2					; TFTW_X2=TFTW_X2-temp	
;			
		mov		TFTW_X2,	W1			; X=TFTW_X2
		mov		TFTW_Y1,	W2			; Y=TFTW_Y1
		goto	Set_Address_Window_Continue
;		
Set_Address_Window_Rotation2:			; UpsideDown Portrait
										; widest black border on top
										; swap TFT_PHeight for TFT_PWidth										
;
		mov		TFTW_X2,	W1			; X=TFTW_X2
		mov		TFTW_Y2,	W2			; Y=TFTW_Y2
		goto	Set_Address_Window_Continue
;		
Set_Address_Window_Rotation3:			; UpsideDown Landscape
										; widest black border on left
										; swap TFT_PHeight for TFT_PWidth										
		mov		TFTW_Y1,	W3			; temp=TFTW_Y1
;
		mov		TFTW_X1,	W0			; get TXTW_X1
		mov		W0,		TFTW_Y1			; TFTW_Y1=TXTW_X1
;		
		mov		Height,	W0				 
		mov		W0,		TFTW_X1			; TFTW_X1=Height
		dec		TFTW_X1					; TFTW_X1=TFTW_X1-1
		mov		TFTW_Y2,	W0			; get TFTW_Y2
		sub		TFTW_X1					; TFTW_X1=TFTW_X1-TFTW_Y2
;		
		mov		TFTW_X2,	W0			; get TFTW_X2
		mov		W0,		TFTW_Y2			; TFTW_Y2=TFTW_X2
;
		mov		Height,	W0			
		mov		W0,		TFTW_X2			; TFTW_X2=Height	
		dec		TFTW_X2					; TFTW_X2=TFTW_X2-1
		mov		W3,	W0					; get temp
		sub		TFTW_X2					; TFTW_X2=TFTW_X2-temp
;			
		mov		TFTW_X1,	W1			; X=TFTW_X1
		mov		TFTW_Y2,	W2			; Y=TFTW_Y2
		goto	Set_Address_Window_Continue				
;
Set_Address_Window_Continue:
		mov		W1,		TFTW_X
		mov		W2,		TFTW_Y
		mov		#TFT_HorizontalAddressStartPositionRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_X1,	W0 
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT 
;
		mov		#TFT_HorizontalAddressEndPositionRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_X2,	W0 
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT 
;
		mov		#TFT_VerticalAddressStartPositionRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_Y1,	W0 
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT 
;
		mov		#TFT_VerticalAddressEndPositionRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_Y2,	W0 
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT 
;
		mov		#TFT_HorizontalGRAMAddressRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_X,		W0
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT 
;
		mov		#TFT_VerticalGRAMAddressRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_Y,		W0
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;		
Set_Address_Window_Exit:
		return
;***************************************
CheckSize:		
		mov		Width,	W0
		cp		TFTW_X1
		bra		GEU,	CheckSize_Error	; branch if TFTW_X1 >= Width
;
		cp		TFTW_X2
		bra		GEU,	CheckSize_Error	; branch if TFTW_X2 >= Width
;
		mov		Height,	W0
		cp		TFTW_Y1
		bra		GEU,	CheckSize_Error	; branch if TFTW_Y1 >= Height
;
		cp		TFTW_Y2
		bra		GEU,	CheckSize_Error	; branch if TFTW_Y2 >= Height
;
		goto	CheckSize_Exit
;
CheckSize_Error:
		bset	LCDFlags,	#BMP2Large	; flag as Too Large For Display
CheckSize_Exit:		
		return		
;***************************************
AdjustX:
		mov		TFTW_X1,	W0
		mov		TFTW_X2,	W1
		sub		Width,		WREG
		dec		W0,		W0
		mov		W0,		TFTW_X2	
		mov		W1,		W0
		sub		Width,		WREG
		dec		W0,		W0
		mov		W0,		TFTW_X1	
		return
;***************************************
AdjustY:
		mov		TFTW_Y1,	W0
		mov		TFTW_Y2,	W1
		sub		Height,		WREG
		dec		W0,		W0
		mov		W0,		TFTW_Y2	
		mov		W1,		W0
		sub		Height,		WREG
		dec		W0,		W0
		mov		W0,		TFTW_Y1
		return					
;}	
;***************************************;***************************************
;
;	Subroutine: Fill Address Window		- fills rectangle (TFTW_X1, TFTW_X2,
;										 TFTW_Y1, TFTW_Y2) with TFTW_FillColor.
;										 (Rectangle's position is based on
;										 TFTW_PositionX and TFTW_PositionY)
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTW_FillColor
;		TFTW_X1
;		TFTW_X2
;		TFTW_Y1
;		TFTW_Y2
;		TFTW_IR
;	Changes:
;		W0
;		W1
;		LATB
;		LATB.WR_STROBE
;		LATB.CS		
;		TRISB
;		TEMP32L
;		TEMP32H
;		TFTW_WR_STROBE_Value
; 	Calls:
;		Write_IR_TFT			
;
;***************************************;***************************************
;{		
Fill_Address_Window:
		push.d	W2
		mov		TFTW_X1,	W0
		sub		TFTW_X2,	WREG
		inc		W0,		W0	
		mov		W0,		TEMP32L
;		
		mov		TFTW_Y1,	W0
		sub		TFTW_Y2,	WREG
		inc		W0,		W0
		mov		TEMP32L,	W1
		mul.uu	W0,		W1,		W2
		mov		W3,		TEMP32H
		mov		W2,		TEMP32L
;
Fill_Address_Window_Write_TFT:
		mov		TRISB,	W1				; get direction bits
		mov		#0x00FF,	W0			; get mask 
		and		W0,		W1,		W1		; force high byte as outputs
		mov		W1,		TRISB			; replace direction bits
;
		nop
		bclr	LATB,	#CS		
		call	Write_IR_TFT
;
		mov		TFTW_FillColor,	W0
		mov		W0,		TFT_WR_STROBE_Value		
;
Fill_Address_Window_Chk4L:
		cp0		TEMP32L
		bra		Z,		Fill_Address_Window_Chk4H
;
		dec		TEMP32L	
		goto	Fill_Address_Window_Write_Pixel	
;
Fill_Address_Window_Chk4H:
		cp0		TEMP32H
		bra		Z,		Fill_Address_Window_Exit
;
		dec		TEMP32H	
		dec		TEMP32L
;
Fill_Address_Window_Write_Pixel:				
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov		TFT_WR_STROBE_Value,	W1		; get word to write
		swap	W1						; swap bytes
		mov.b	W1,		W0				; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;	
;		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
;		nop
;
		mov		PORTB,	W0				; get port b
		swap.w	W0						; swap bytes
		mov.b	TFT_WR_STROBE_Value,	WREG	; replace (now) lower byte
		swap.w	W0						; swap bytes back				
		mov		W0,		LATB			; replace port b
;		
;		nop
		bclr	LATB,	#WR_STROBE				; write strobe
		nop
		bset	LATB,	#WR_STROBE				; write strobe
;		nop	
;
		goto	Fill_Address_Window_Chk4L
;
Fill_Address_Window_Exit:
		nop
		bset	LATB,	#CS	
		pop.d	W2
		return				
;}	
;***************************************;***************************************
;
;	Subroutine: NextX (CR)				- adjust TFTW_PositionX based
;										 on TFTW_PositionX and TFTB_TextSize
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;		TFTW_PositionX
;		Width
;		LCDFlags.WrapAround
;	Changes:
;		W0
;		W1
;		TFTW_PositionX
; 	Calls:
;		NextY
;		CR
;
;***************************************;***************************************
;{		
NextX:
		push.d	W2
		mov		#12,	W0				; two character widths
		mul.b	TFTB_TextSize
		mov		W2,		W0
		mov		TFTW_PositionX,	W1
		add		W0,		W1,		W1		
		mov		Width,	WREG		
		cp		W0,		W1
		bra		LTU,	NextX_Wrap		; exit if past right edge screen	
;		
		lsr		W2,		W0
		add		TFTW_PositionX
		goto	NextX_Exit
;
NextX_Wrap:
		btss	LCDFlags,	#WrapAround
		goto	NextX_Exit
;
		call	NextY
;		
 		call	CR
;		
NextX_Exit:
		pop.d	W2
		return
;
CR:										; force to beginning of line
		clr		TFTW_PositionX
		return		
;}	
;***************************************;***************************************
;
;	Subroutine: NextY (LF)				- adjust TFTW_PositionY based
;										 on TFTW_PositionY and TFTB_TextSize 
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;		TFTW_PositionY
;		Height
;		LCDFlags,	#WrapAround
;	Changes:
;		W0
;		W1
;		TFTW_PositionY
; 	Calls:
;		none
;
;***************************************;***************************************
;{		
NextY:
		push.d	W2
		mov		#16,		W0
		mul.b	TFTB_TextSize
		mov		W2,		W0
		mov		TFTW_PositionY,	W1
		add		W0,		W1,		W1		
		mov		Height,	WREG		
		cp		W0,		W1
		bra		LTU,	NextY_Wrap		; branch if past bottom of screen	
;	
		lsr		W2,		W0
		add		TFTW_PositionY
		goto	NextY_Exit				
;
NextY_Wrap:							
		btss	LCDFlags,	#WrapAround
		goto	NextY_Exit
;
 		clr		TFTW_PositionY
;		
NextY_Exit:
		pop.d	W2
		return
;}	
;***************************************;***************************************
;
;	Subroutine: DisplayACharacter		- creats a window (6x8 times TFTB_TextSize
;										 TFTW_X1, TFTW_X2, TFTW_Y1, TFTW_Y2)
;										 using TFTB_Character it looks up the data
;										 to diplay the character. Based on the 
;										 character's bit it sets the pixel color 
;										 using TFTW_FillColor or TFTW_TextColor
;										 to fill the window.  TextSize magnifies
;										 each bit.
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		FONT (table)
;		TFTB_TextSize
;		TFTB_Character
;		TFTW_FillColor
;		TFTW_TextColor
;		TFTW_X
;		TFTW_Y
;	Changes:
;		W0
;		W1
;		TFTW_PixelColor
;		TFTW_X1
;		TFTW_Y1
;		TFTW_X2
;		TFTW_Y2
;		TFTB_Column
;		TFTB_ColumnCount
;		TFTB_ColumnSize
;		TFTB_ColumnSizeCount
;		TFTB_Row
;		TFTB_RowCount
;		TFTB_RowSize
;		TFTB_RowSizeCount
; 	Calls:
;		SetNextPixel
;		Set_Address_Window
;		NextX
;
;***************************************;***************************************
;{		
DisplayACharacter:
		push.d	W2
		clr.b	TFTB_Count
;
; is there enough space to print char?
;
		mov		#6,		W0
		mul.b	TFTB_Character			; result in W3:2
		mov		#tblpage(FONT), W0
		mov		W0, 	_TBLPAG 		; Set TBLPAG to the page that contains
										; the table array
		mov		#tbloffset(FONT), W0	; Make a pointer to character table
		add		W0,		W2,		W2		; add offset
;
		mov		#TFTB_Char1,	W1		; pointer to RAM character buffer
;
DisplayACharacter_GetLoop:
		tblrdl.b	[W2++],		[W1++]	; Load the character data to RAM buffer
		inc.b	TFTB_Count
		mov.b	TFTB_Count,	WREG		
		cp.b	W0,		#6,
		bra		NZ,		DisplayACharacter_GetLoop		
;										
		mov		#6,		W0
		mov.b	WREG,	TFTB_Column
		mul.b	TFTB_TextSize			; result in W3:2
		mov		W2,		W0
		mov.b	WREG,	TFTB_ColumnSize	; width of window
;
		mov		#8,		W0
		mov.b	WREG,	TFTB_Row
		mul.b	TFTB_TextSize			; result in W3:2
		mov		W2,		W0
		mov.b	WREG,	TFTB_RowSize	; height of window
;						
		clr		W0
		mov.b	TFTB_ColumnSize,	WREG
		mov		W0,		W1				
		mov		TFTW_X,	W0
		mov		W0,		TFTW_X1			; starting at TFTW_X
		add		W0,		W1,		W0		; width of window
		dec		W0,		W0
		mov		W0,		TFTW_X2			; ending at TFTW_X+width-1
;
		clr		W0
		mov.b	TFTB_RowSize,	WREG
		mov		W0,		W1
		mov		TFTW_Y,	W0
		mov		W0,		TFTW_Y1			; starting at TFTW_Y
		add		W0,		W1,		W0		; height of window
		dec		W0,		W0
		mov		W0,		TFTW_Y2			; ending at TFTW_X+height-1
		call	Set_Address_Window
;
DisplayACharacter_Fill:
		mov		#0x8000,		W3
		clr.b	TFTB_RowCount		
;
DisplayACharacter_RowLoop:
		clr.b	TFTB_RowSizeCount	
		rlnc	W3,		W3
;
DisplayACharacter_RowSizeLoop:
		mov		#TFTB_Char1,	W2		; pointer to RAM character buffer
		clr.b	TFTB_ColumnCount
;		
DisplayACharacter_ColumnLoop:
		mov.b	[W2++],		W0
		and.b	W0,		W3,		W0
		cp0.b	W0
		bra		Z,		DisplayACharacter_Bit0		
;
DisplayACharacter_Bit1:
		mov		TFTW_TextColor,	W0
		goto	DisplayACharacter_Continue

DisplayACharacter_Bit0:
		mov		TFTW_FillColor,	W0
		goto	DisplayACharacter_Continue

DisplayACharacter_Continue:
		mov		W0,		TFTW_PixelColor
		clr.b	TFTB_ColumnSizeCount		
;
DisplayACharacter_ColumnSizeLoop:		
		call	SetNextPixel
;
		inc.b	TFTB_ColumnSizeCount
		mov.b	TFTB_ColumnSizeCount,	WREG
		cp.b	TFTB_TextSize	
		bra		NZ,		DisplayACharacter_ColumnSizeLoop
;
		inc.b	TFTB_ColumnCount
		mov.b	TFTB_ColumnCount,	WREG
		cp.b	TFTB_Column	
		bra		NZ,		DisplayACharacter_ColumnLoop
;		
		inc.b	TFTB_RowSizeCount
		mov.b	TFTB_RowSizeCount,	WREG
		cp.b	TFTB_TextSize	
		bra		NZ,		DisplayACharacter_RowSizeLoop		
;		
		inc.b	TFTB_RowCount
		mov.b	TFTB_RowCount,	WREG
		cp.b	TFTB_Row	
		bra		NZ,		DisplayACharacter_RowLoop		
;
DisplayACharacter_Exit:
		pop.d	W2
		call	NextX
;		
		return						
;}	
;***************************************;***************************************
;
;	Subroutine: Draw a Pixel			@ TFTW_CharacterX, TFTW_CharacterY,
;										 with TFTW_PixelColor,
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;		TFTW_CharacterX
;		TFTW_CharacterY
;	Changes:
;		W0
;		W1
;		TFTB_Row
;		TFTB_Column
;		TFTB_RowCount
;		TFTB_ColumnCount
;		TFTW_PixelX
;		TFTW_PixelY
; 	Calls:
;		SetPixel
;
;***************************************;***************************************
;{		
DrawPixel:
		clr.b	TFTB_Row
		clr.b	TFTB_Column
		mov.b	TFTB_TextSize,	WREG
		dec.b	W0,		W0
		mov.b	WREG,	TFTB_RowCount
		mov.b	WREG,	TFTB_ColumnCount
;
DrawPixel_Multiple_ColumnLoop:
		clr		W0
		mov.b	TFTB_Column,	WREG
		add		TFTW_CharacterX,	WREG
		mov		W0,		TFTW_PixelX
;
		clr		W0
		mov.b	TFTB_Row,	WREG
		add		TFTW_CharacterY,	WREG
		mov		W0,		TFTW_PixelY
;						
		call	SetPixel
		mov.b	TFTB_ColumnCount,	WREG
		cp.b	TFTB_Column
		bra		Z,	DrawPixel_Multiple_Row
;
		inc.b	TFTB_Column
		goto	DrawPixel_Multiple_ColumnLoop
;
DrawPixel_Multiple_Row:				
		mov.b	TFTB_RowCount,	WREG
		cp.b	TFTB_Row
		bra		Z,	DrawPixel_Exit
;
		inc.b	TFTB_Row
		clr.b	TFTB_Column
		goto	DrawPixel_Multiple_ColumnLoop
;
DrawPixel_Exit:				
		return	
;}	
;***************************************;***************************************
;
;	Subroutine: Set a Pixel			@ TFTW_PixelX, TFTW_PixelY,
;										 with TFTW_PixelColor,
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTW_PixelX
;		TFTW_PixelY
;		TFTW_PixelColor
;		#TFT_PWidth
;		#TFT_PHeight
;		LCDFlags.OrientationDisplay
;		LCDFlags.UpsideDownDisplay
;	Changes:
;		W0
;		W1
;		TFTW_PixelX
;		TFTW_PixelY
;		TFT_IR
;		TFT_WR_STROBE_Value
; 	Calls:
;		SetPixelPosition
;		SetNextPixel
;
;***************************************;***************************************
;{		
SetPixel:
;
SetPixel_AdjustLocation:
		btsc	LCDFlags,	#OrientationDisplay		; skip next if Portrait		
		goto	SetPixel_AdjustLocation_InL
;
SetPixel_AdjustLocation_InP:
		btss	LCDFlags,	#UpsideDownDisplay		; skip next if UpsideDown			
		goto	SetPixel_AdjustLocation_InP_RightsideUp
;		
SetPixel_AdjustLocation_InP_UpsideDown:
		mov		TFTW_PixelX,	W0
		mov		#TFT_PWidth,	W1
		mov		W1,	TFTW_PixelX
		dec		TFTW_PixelX
		sub		TFTW_PixelX
;
		mov		TFTW_PixelY,	W0
		mov		#TFT_PHeight,	W1
		mov		W1,		TFTW_PixelY
		dec		TFTW_PixelY
		sub		TFTW_PixelY		
		goto	SetPixel_Continue
		
SetPixel_AdjustLocation_InP_RightsideUp:		
		goto	SetPixel_Continue		
;		
SetPixel_AdjustLocation_InL:		
		btss	LCDFlags,	#UpsideDownDisplay		; skip next if UpsideDown	
		goto	SetPixel_AdjustLocation_InL_RightsideUp
;		
SetPixel_AdjustLocation_InL_UpsideDown:
		mov		TFTW_PixelX,	W0
		mov		TFTW_PixelY,	W1
		mov		W1,		TEMPW
		mov		#TFT_PHeight,	W1
		mov		W1,		TFTW_PixelY
		dec		TFTW_PixelY
		sub		TFTW_PixelY
;
		mov		TEMPW,	W0
		mov		W0,		TFTW_PixelX
		goto	SetPixel_Continue

SetPixel_AdjustLocation_InL_RightsideUp:
		mov		TFTW_PixelY,	W0
		mov		TFTW_PixelX,	W1
		mov		W1,		TEMPW
		mov		#TFT_PWidth,	W1
		mov		W1,		TFTW_PixelX
		dec		TFTW_PixelX
		sub		TFTW_PixelX
;		
		mov		TEMPW,	W0
		mov		W0,		TFTW_PixelY
		goto	SetPixel_Continue
;
SetPixel_Continue:
		call	SetPixelPosition
;
		call	SetNextPixel
;
		return				
;}	
;***************************************;***************************************
;
;	Subroutine: Set a Pixel			@ TFTW_PixelX, TFTW_PixelY,
;										 with TFTW_PixelColor,
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#TFT_HorizontalGRAMAddressRegister
;		TFTW_PixelX
;		#TFT_VerticalGRAMAddressRegister
;		TFTW_PixelY
;	Changes:
;		W0
;		TFT_IR
;		TFT_WR_STROBE_Value
; 	Calls:
;		Write_TFT
;
;***************************************;***************************************
;{
SetPixelPosition:		
		mov		#TFT_HorizontalGRAMAddressRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_PixelX,	WREG
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT		
;
		mov		#TFT_VerticalGRAMAddressRegister,	W0
		mov.b	WREG,	TFT_IR
		mov		TFTW_PixelY,	WREG 
		mov		W0,		TFT_WR_STROBE_Value
		call	Write_TFT
;
		return		
;}	
;***************************************;***************************************
;
;	Subroutine: Set a Pixel			@ TFTW_PixelX, TFTW_PixelY,
;										 with TFTW_PixelColor,
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#TFT_WriteDataToGRAMRegister
;		TFTW_PixelColor
;	Changes:
;		W0
;		TFT_IR
;		TFT_WR_STROBE_Value
; 	Calls:
;		Write_TFT
;
;***************************************;***************************************
;{
SetNextPixel:
		mov		#TFT_WriteDataToGRAMRegister,	W0
		mov.b	WREG,		TFT_IR
		mov		TFTW_PixelColor,	W1 
		mov		W1,		TFT_WR_STROBE_Value
		call	Write_TFT		
		return	
;}	
;***************************************;***************************************
;
;	Subroutine: ROM To RAM
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#CommandBuffer
;	Changes:
;		W0
;		W1
; 	Calls:
;		none
;
;***************************************;***************************************
;{		
ROMToRAM:
		push	W2
		mov		#CommandBuffer,	W2
;
ROMToRAM_Loop: 
		tblrdl.b	[W1++],	W0			; Load the first data value
		cp0.b	W0
		bra		Z,		ROMToRAM_Exit		
		mov.b	W0,	[W2++]
		goto	ROMToRAM_Loop
;
ROMToRAM_Exit:
		mov		#0x0D,	W0	
		mov.b	W0,	[W2++]
		pop		W2
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Display Button			- Use TFTB_RowCount and TFTB_ColumnCount
;										 to draw a button outline at
;										 TFTW_PositionX and TFTW_PositionY.
;										 Use TFTW_TEXTCOLOR and TFTW_FILLCOLOR 
;
;										- TFTW_X+3 and TFTW_X+4 are
;										 used to save buttonID into the 
;										 ButtonMatrix position/x where x is
;										 based on width/16 and height/16
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTW_PositionX
;		TFTW_PositionY
;		TFTB_TextColor
;		TFTB_FillColor
;		TFTB_RowCount
;		TFTB_ColumnCount
;		TFTB_Rotation
;	Changes:
;		W0
;		W1
;		TFTW_PositionX
;		TFTW_PositionY
;		TFTB_ButtonRowCount
;		TFTB_ButtonColumnCount
;		TFTW_X
;		TFTW_Y
;		TFTB_Character
;		ButtonID
; 	Calls:
;		SaveButtonIDToMatrix
;		DisplayACharacter
;		SendMessage
;		SendDecimalWord
;		SendCRLF
;
;***************************************;***************************************
;{		
Display_Button:
;
Display_Button_TopRow:
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_OriginX
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_OriginY		
		dec.b	TFTB_KeypadRowCount
		dec.b	TFTB_KeypadRowCount
		dec.b	TFTB_KeypadColumnCount
		dec.b	TFTB_KeypadColumnCount		
		clr.b	TFTB_KeypadRow
;
Display_Button_TopRow_LeftCorner:		
		mov		#0xC9,		W0			; upper left corner character	
		mov.b	WREG,		TFTB_Character
		mov		TFTW_OriginX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_OriginY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
Display_Button_TopRow_TopEdge:
		mov		#0xCD,		W0			; top of button character
		mov.b	WREG,		TFTB_Character
		clr.b	TFTB_KeypadColumn				
;
Display_Button_TopRow_TopEdge_Loop:
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;		
		call	DisplayACharacter
;		
		inc.b	TFTB_KeypadColumn
		mov.b	TFTB_KeypadColumnCount,	WREG
		cp.b	TFTB_KeypadColumn
		bra		NZ,		Display_Button_TopRow_TopEdge_Loop
;
Display_Button_TopRow_RightCorner:
		mov		#0xBB,		W0			; upper right corner character
		mov.b	WREG,		TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
Display_Button_MiddleRows_Loop:
		mov		TFTW_OriginX,	W0
		mov		W0,		TFTW_PositionX
 		call	NextY
;
Display_Button_MiddleRows_LeftBorder:		
		mov		#0xBA,		W0			; left side character
		mov.b	WREG,		TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
Display_Button_MiddleRows_Text:
		clr.b	TFTB_KeypadColumn									
;
Display_Button_MiddleRows_TextColumn_Loop:
		mov.b	#' ',		W0			; blank text
		mov.b	WREG,		TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
		inc.b	TFTB_KeypadColumn
		mov.b	TFTB_KeypadColumnCount,	WREG
		cp.b	TFTB_KeypadColumn	
		bra		NZ,		Display_Button_MiddleRows_TextColumn_Loop
;
Display_Button_MiddleRows_RightBorder:
		mov		#0xBA,		W0			; right side character
		mov.b	WREG,		TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
		inc.b	TFTB_KeypadRow
		mov.b	TFTB_KeypadRowCount,	WREG
		cp.b	TFTB_KeypadRow	
		bra		NZ,		Display_Button_MiddleRows_Loop		
;
Display_Button_Bottom:
		mov		TFTW_OriginX,	W0
		mov		W0,		TFTW_PositionX
 		call	NextY			
;
Display_Button_Bottom_LeftCorner:	
		mov		#0xC8,	W0				; lower left corner character
		mov.b	WREG,	TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
Display_Button_Bottom_Edge:
		mov		#0xCD,	W0				; lower edge character
		mov.b	WREG,	TFTB_Character
		clr.b	TFTB_KeypadColumn		
;
Display_Button_Bottom_EdgeLoop:
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;		
		call	DisplayACharacter
;		
		inc.b	TFTB_KeypadColumn
		mov.b	TFTB_KeypadColumnCount,	WREG
		cp.b	TFTB_KeypadColumn
		bra		NZ,		Display_Button_Bottom_EdgeLoop
;
Display_Button_Bottom_RightCorner:
		mov		#0xBC,	W0				; lower right corner character
		mov.b	WREG,	TFTB_Character
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y
		call	SaveButtonIDToMatrix
;				
		call	DisplayACharacter
;
Display_Button_Exit:
		mov		#tblpage(Button_Command), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Button_Command), W1	; Make a pointer to table
		call	SendMessage
;
		mov.b	ButtonID,	WREG
		call	SendDecimalWord
;
		call	SendCRLF
;
		inc.b	ButtonID
;				
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Send_Files			- Dump SD Card directory
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		RootDirSector
;		SectorCount
;	Changes:
;		W0
;		W1
;		SPI1STAT.SPIEN
;		SectorCount
;		BlockAddressL
;		BlockAddressH
;		DirectoryCount
; 	Calls:
;		Init_Media
;		SendMessage
;		SendCRLF
;		CMD17
;		SendRAMMessage
;
;***************************************;***************************************
;{			
Send_Files:
		mov		TRISB,	W1				; get direction bits
		mov		#0xFF00,	W0			; get mask 
		ior		W0,		W1,		W1		; force high byte as inputs
		mov		#0xD7FF,	W0			; get mask 
		and		W0,		W1,		W1		; force low byte as outputs		
		mov		W1,		TRISB			; replace direction bits		
		bset	SPI1STAT,	#SPIEN		; enable SPI funtions
;
		call	Init_Media
;
		btsc	SDFlags,	#BadMedia
		goto	Send_Files_NoCard
;
		btss	SDFlags,	#MediaReady
		goto	Send_Files_MediaErrorCard		
;
Send_Files_DirSearch:
		mov		#tblpage(DirSearch), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(DirSearch), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		clr		SectorCount
;		
Send_Files_SectorLoop:
		mov		RootDirSector,	W0		; First Sector		
		add		SectorCount,	WREG
		mov		W0,		BlockAddressL
		clr		BlockAddressH
		call	CMD17
;
		clr		DirectoryCount
		mov		#SPI_Data_buffer+1,	W4	; Beginning of Sector Data				
;
Send_Files_DirectoryLoop:	
		mov		#0x20,	W0	
		mul		DirectoryCount			; Directory offset (result in W3:W2)
		add		W2,		W4,		W2		; into Sector Data
		mov.b	[W2],	W0				; get the byte
		cp0.b	W0						; is it zero
		bra		Z,		Send_Files_Done	; yes, we're done (no more entries)
;
		mov		#0xE5,	W3				; deleted entry
		cp.b	W0,		W3
		bra		Z,		Send_Files_Next
;
		add		W2,	#0x0B,	W2			; else add offset to dir attributes
		mov.b	[W2],	W0				; get the byte
		mov		#0x20,	W3				; must be 0x10 or it's not a file
		cp.b	W0,		W3
		bra		NZ,		Send_Files_Next
;
Send_Files_FileName:				; looks like a file
		sub		W2,	#0x0B,	W2			; go back to the file name
		mov		#StringBuffer,	W3
;
Send_Files_LetterLoop:
		mov.b	[W2++],	[W3++]
		mov		#StringBuffer+8,		W1
		cp.b	W3,		W1
		bra		NZ,		Send_Files_LetterLoop
;
		mov		#'.',	W0				; period
		mov.b	W0,	[W3++]
;
Send_Files_ExtentionLoop:
		mov.b	[W2++],	[W3++]		
		mov		#StringBuffer+12,	W1
		cp.b	W3,		W1
		bra		NZ,		Send_Files_ExtentionLoop
;
		mov		#0,		W0
		mov.b	W0,	[W3]
		mov		#StringBuffer,	W1
		call	SendRAMMessage
;		
		call	SendCRLF
;
Send_Files_Next:
		inc		DirectoryCount
		mov		NumDirInSector,	W0
		cp		DirectoryCount
		bra		NZ,		Send_Files_DirectoryLoop
;
		inc		SectorCount
		mov		NumSecInCluster,	W0
		cp		SectorCount
		bra		NZ,		Send_Files_SectorLoop							

Send_Files_Done:
		mov		#tblpage(OKSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(OKSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Send_Files_Exit
;		
Send_Files_NoCard:
		mov		#tblpage(NoSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(NoSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Send_Files_Exit	
;		
Send_Files_MediaErrorCard:
		mov		#tblpage(MediaErrorSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MediaErrorSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Send_Files_Exit 		
;
Send_Files_Exit:
		bclr	SPI1STAT,	#SPIEN		; disable SPI funtions
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Search4File				- Search SD Card directory for a
;										 specific file.  If found display the 
;										 image on the graphics display
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		RootDirSector
;		SectorCount
;		NumSecInCluster
;		NumDirInSector
;	Changes:
;		W0
;		W1
;		SPI1STAT.SPIEN
;		SectorCount
;		BlockAddressL
;		BlockAddressH
;		DirectoryCount
;		LCDFlags.Match
; 	Calls:
;		Init_Media
;		SendMessage
;		SendCRLF
;		CMD17
;		SendRAMMessage
;		PullOutFileEntry
;		DisplayBMP
;{
;***************************************;***************************************
;{			
Search4File:
		mov		TRISB,	W1				; get direction bits
		mov		#0xFF00,	W0			; get mask 
		ior		W0,		W1,		W1		; force high byte as inputs
		mov		#0xD7FF,	W0			; get mask 
		and		W0,		W1,		W1		; force low byte as outputs		
		mov		W1,		TRISB			; replace direction bits		
;
		call	Init_Media
;
		btsc	SDFlags,	#BadMedia
		goto	Search4File_NoCard
;
		btss	SDFlags,	#MediaReady
		goto	Search4File_MediaErrorCard		
;
Search4File_DirSearch:
		mov		#tblpage(DirSearch), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(DirSearch), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;		
		clr		SectorCount
;		
Search4File_SectorLoop:
		mov		RootDirSector,	W0		; First Sector		
		add		SectorCount,	WREG
		mov		W0,		BlockAddressL
		clr		BlockAddressH
		call	CMD17
;
		clr		DirectoryCount
		mov		#SPI_Data_buffer+1,	W4	; Beginning of Sector Data				
;
Search4File_DirectoryLoop:	
		mov		#0x20,	W0	
		mul		DirectoryCount			; Directory offset (result in W3:W2)
		add		W2,		W4,		W10		; into Sector Data
		mov		W10,	W2
		mov.b	[W2],	W0				; get the byte
		cp0.b	W0						; is it zero
		bra		Z,		Search4File_NotFound	; yes, we're done (no more entries)
;
		mov		#0xE5,	W3				; deleted entry
		cp.b	W0,		W3
		bra		Z,		Search4File_Next
;
		add		W2,	#0x0B,	W2			; else add offset to dir attributes
		mov.b	[W2],	W0				; get the byte
		mov		#0x20,	W3				; must be 0x10 or it's not a file
		cp.b	W0,		W3
		bra		NZ,		Search4File_Next
;
Search4File_FileName:				; looks like a file
		mov		W10,	W2			; go back to the file name
		mov		#StringBuffer,	W3
;
Search4File_FileNameLoop:
		mov.b	[W2],	W0
		mov.b	[W3++],	W1
		cpseq.b	W0,		W1
		goto	Search4File_Next
;
		inc		W2,		W2
		mov		#StringBuffer+11,		W1
		cp.b	W3,		W1
		bra		NZ,		Search4File_FileNameLoop
;
		goto	Search4File_Done				
;
Search4File_Next:
		inc		DirectoryCount
		mov		NumDirInSector,	W0
		cp		DirectoryCount
		bra		NZ,		Search4File_DirectoryLoop
;
		inc		SectorCount
		mov		NumSecInCluster,	W0
		cp		SectorCount
		bra		NZ,		Search4File_SectorLoop							
;
Search4File_NotFound:
 		bclr	LCDFlags,	#Match		; no match found
		goto	Search4File_Exit
;
Search4File_Done:						; found match
		call	PullOutFileEntry		; get all file info
;				
		call	DisplayBMP
;
		mov		#tblpage(OKSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(OKSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Search4File_Exit
;		
Search4File_NoCard:
		mov		#tblpage(NoSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(NoSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Search4File_NotFound	
;		
Search4File_MediaErrorCard:
		mov		#tblpage(MediaErrorSD), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(MediaErrorSD), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
 		goto	Search4File_NotFound 		
;
Search4File_Exit:
		return
;}
;***************************************;**************************************
;
;	PullOutFileEntry:	move the file entry from the sector to the file
;						 storage area
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10				pointer to file entry
;		W11				pointer to File_Name
;	Changes:
;		W0
;		File_ATT
;		File_NTRES
;		File_CTimeF
;		File_CTime
;		File_CDate
;		File_LADate
;		File_FClusterH
;		File_WTime
;		File_WDate
;		File_FClusterL
;		File_Size+2
;		File_Size
; 	Calls:
;		none
;
;***************************************;**************************************
;{
PullOutFileEntry:
		mov		#File_Name,W11
		mov.b	[W10++],[W11++]
		mov.b	[W10++],[W11++]			
		mov.b	[W10++],[W11++]
		mov.b	[W10++],[W11++]
		mov.b	[W10++],[W11++]			
		mov.b	[W10++],[W11++]		
		mov.b	[W10++],[W11++]
		mov.b	[W10++],[W11++]
		mov		#0x2E,	W0
		mov.b	W0,		[W11++]
		mov.b	[W10++],[W11++]			
		mov.b	[W10++],[W11++]
		mov.b	[W10++],[W11]
		clr		W0
		mov.b	[W10++],W0
		mov		W0,		File_ATT		
		mov.b	[W10++],W0
		mov		W0,		File_NTRES
		mov.b	[W10++],W0
		mov		W0,		File_CTimeF		
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov		W0,		File_CTime				
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov		W0,		File_CDate		
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov 	W0,		File_LADate
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov 	W0,		File_FClusterH				
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov 	W0,		File_WTime				
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov 	W0,		File_WDate		
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0
		mov 	W0,		File_FClusterL
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0
		mov 	W0,		File_Size+2	
		mov.b	[W10++],W0
		swap	W0
		mov.b	[W10++],W0
		swap	W0		
		mov 	W0,		File_Size	
;
PullOutFileEntry_Exit:
		return
;}
;***************************************;**************************************
;
;	GetFirstSector/NextSector	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		File_Size
;		File_Size+2
;		File_FClusterL
;		FATRegionSectorH
;		FATRegionSectorL
;		RootDirSector
;	Changes:
;		W0
;		W1
;		FileLength
;		FileLength+2
;		ClusterOfInterest
;		NumSecInClusterCount
;		NumSecInCluster
;		SectorOfInterestH
;		SectorOfInterestL
;		FATEntry
;		BlockAddressL
;		BlockAddressH
; 	Calls:
;		GotoAFATEntry
;		ReadAFATEntry
;		CMD17
;		
;***************************************;**************************************
;{
GetFirstSector:
		push.d	W2
		mov		File_Size,W0
		mov		W0,		FileLength
		mov		File_Size+2,W0
		mov		W0,		FileLength+2		
;
GetFirstSector_FirstCluster:
		mov		File_FClusterL,	W0
		mov		W0,		ClusterOfInterest
		goto	GetSector_Cluster_Loop
;
GetNextSector:
		push.d	W2
		inc		NumSecInClusterCount
		mov		NumSecInClusterCount,W0
		mov		NumSecInCluster,W1
		cpseq	W0,		W1
		bra		GetSector_Loop
;
GetSector_NextCluster:
		mov		FATRegionSectorH,	W0
		mov		W0,		SectorOfInterestH
		mov		FATRegionSectorL,	W0
		mov		W0,		SectorOfInterestL
		mov		ClusterOfInterest,	W0
		mov		W0,		FATEntry
		call	GotoAFATEntry
;
		call	ReadAFATEntry
;
		mov		FATEntryValue,	W0
		mov		W0,		ClusterOfInterest				
		bra		GetSector_Cluster_Loop		
;
GetSector_Cluster_Loop:		
		clr		NumSecInClusterCount
;
GetSector_Loop:
		mov		RootDirSector,W0
		mov		W0,		SectorOfInterestL
		clr		SectorOfInterestH
;		
		mov		ClusterOfInterest,W0
		cp0		W0
		bra		z,		GetSector_Continue
		dec		W0,		W0
;		
GetSector_Continue:		
		mov		NumSecInCluster,W1
		mul.UU	W0,		W1,		W0
		mov		NumSecInClusterCount,W2
		add		W2,		W0,		W0
		clr		W2
		addc	W2,		W1,		W1
		mov		SectorOfInterestL,W2
		add		W0,		W2,		W2
		mov		W2,		SectorOfInterestL
		mov		SectorOfInterestH,W2
		addc	W1,		W2,		W2
		mov		W2,		SectorOfInterestH
;
		mov		SectorOfInterestL,W0
		mov		W0,		BlockAddressL
		mov		SectorOfInterestH,W0
		mov		W0,		BlockAddressH
;
		call	CMD17
;
		mov		#SPI_Data_buffer+1,W10
		pop.d	W2
		return
;}
;***************************************;**************************************
;
;	Chk4EndOfSectorBuffer	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
; 	Calls:
;		GetNextSector
;
;***************************************;**************************************
;{		
Chk4EndOfSectorBuffer:
		mov		#SPI_Data_buffer+0x201,	W0
		cpseq	W10,	W0
		goto	Chk4EndOfSectorBuffer_Exit	
;
EndOfSectorBuffer_NeedAnotherSector:
		call	GetNextSector		
;
Chk4EndOfSectorBuffer_Exit:
		return		
				
;}
;***************************************;**************************************
;
;	DisplayBMP	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#BitMapType
;		W10
;		BitMapOffsetL
;		#ColorTable1
;		TFTW_PixelColor
;		TFTW_PositionX
;		TFTW_PositionY
;		BitMapWidthL
;		BitMapHeightL
;		BitMapBitsPerPixel
;	Changes:
;		W2
;		W1
;		W0
;		BitMapColorTableSizeL
;		TFTW_X1
;		TFTW_X2
;		TFTW_Y1
;		TFTW_Y2
;		LCDFlags.BMP2Large
;		BMP_Width_Count
;		BMP_Height_Count
;		BMP_ColumnMax
;		BMP_Width_DataMax
;		BMP_RowMax
;		
; 	Calls:
;		GetFirstSector
;		Chk4EndOfSectorBuffer
;		BuildAColor
;		FlipUpsideDown
;		MirrorImage
;		CorrectOrigin
;		Set_Address_Window
;		SendMessage
;		SendCRLF
;		DisplayBMP_1bbp
;		DisplayBMP_4bbp
;		DisplayBMP_8bbp
;		DisplayBMP_16bbp
;		DisplayBMP_24bbp
;		DisplayBMP_32bbp
;
;***************************************;**************************************
;{		
DisplayBMP:		
		call	GetFirstSector	
;			
;***************************************
BMPFileHeader:		
		mov		#BitMapType,	W2
;
BMPFileHeader_Loop:		
		clr		W1
		mov.b	[W10++],	W1
		swap	W1
		clr		W0
		mov.b	[W10++],	W0
		ior		W1,		W0,		W0
		swap	W0		
		mov		W0,		[W2++]
;
		mov		#BitMapType+0x36,	W1
		cpseq	W1,		W2
		goto	BMPFileHeader_Loop		; read in all header info
;
		mov		BitMapOffsetL,	W0
		sub		#0x36,	W0
		mov		W0,		BitMapColorTableSizeL
;
		mov		#ColorTable1,	W2
		mov		BitMapColorTableSizeL,	W1
		bclr	SR,		#C
		rrc		W1,		W1					; divide by 2
		mov		#ColorTable1,	W0		
		add		W0,		W1,		W3			; end of palette				
;***************************************
StoringPalette_Loop:			; now store away the color palette
		cp		W2,		W3
		bra		Z,		StoringPalette_Done
;		
		mov.b	[W10++],	W0				; read the blue byte
		mov.b	WREG,	TFTB_Blue
		call	Chk4EndOfSectorBuffer
;		
		mov.b	[W10++],	W0				; read the green byte
		mov.b	WREG,	TFTB_Green
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; read the red byte	
		mov.b	WREG,	TFTB_Red
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; 4th byte is always zero not used
;		
		call	Chk4EndOfSectorBuffer
;				
		call	BuildAColor
;
		mov		TFTW_PixelColor,	W0
		mov		W0,		[W2++]
		goto	StoringPalette_Loop
;
StoringPalette_Done:
;***************************************
SettingAddressWinDow4BMP:
		mov		TFTW_PositionX,	W0		; PositionX
		mov		W0,	TFTW_X1				; start of window width
		mov		BitMapWidthL,	W1		; width of bitmap
		add		W0,		W1,		W0
		dec		W0,		W0
		mov		W0,	TFTW_X2				; end of window width
		mov		TFTW_PositionY,	W0		; PositionY 
		mov		W0,	TFTW_Y1				; start of window height
		mov		BitMapHeightL,	W1		; height of bitmap
		add		W0,		W1,		W0
		dec		W0,		W0
		mov		W0,	TFTW_Y2				; end of window height
;
		call	FlipUpsideDown
;
		call	MirrorImage
;
		call	CorrectOrigin
;				
		call	Set_Address_Window		; set the window
;
		btss	LCDFlags,	#BMP2Large	; skip next if BMP Too Large for Display
		goto	DisplayingBMP
;
		mov		#tblpage(BMPTooLarge), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(BMPTooLarge), W1	; Make a pointer to table
		call	SendMessage
;
		call	SendCRLF
;
		goto	DisplayingBMP_Exit	
;
;***************************************
DisplayingBMP:
		clr		BMP_Width_Count
		clr		BMP_Height_Count
		mov		BitMapWidthL,	W0
		mov		W0,		BMP_ColumnMax
		mov		#0x20,	W0
		mov		BitMapBitsPerPixel,	W2
		repeat	#17
		div.u	W0,		W2
;assume W0 has quotient
		mov		W0,		W2	
		mov		BitMapWidthL,	W0
		repeat	#17
		div.u	W0,		W2
;assume W0 has quotient
		cp0		W1			
		btss	SR,		#Z				; skip next if remainder=0		
		inc		W0,		W0				; add a pad word		
		mul.uu	W0,		W2,		W0
;assume W0 has LSW
		btsc	W0,		#0				; skip next if even
		inc		W0,		W0
		btsc	W0,		#1				; skip next if even
		inc2		W0,		W0		
;				
		mov		W0,		BMP_Width_DataMax	; the max bytes of width data
		mov		BitMapHeightL,	W0
		mov		W0,		BMP_RowMax
;
DisplayingBMP_Chk1bbp:
		mov		#1,		W0
		cp		BitMapBitsPerPixel
		bra		NZ,		DisplayingBMP_Chk4bpp
;		
DisplayingBMP_1bpp:						; monochrome bipmap (2 color palette)
		call	DisplayBMP_1bbp
		
		goto	DisplayingBMP_Exit
;		
DisplayingBMP_Chk4bpp:
		mov		#4,		W0
		cp		BitMapBitsPerPixel
		bra		NZ,		DisplayingBMP_Chk8bpp
;		
DisplayingBMP_4bpp:						; 16 color bipmap (16 color palette)
		call	DisplayBMP_4bbp
		
		goto	DisplayingBMP_Exit
;
DisplayingBMP_Chk8bpp:
		mov		#8,		W0
		cp		BitMapBitsPerPixel
		bra		NZ,		DisplayingBMP_Chk16bpp
;		
DisplayingBMP_8bpp:						; 256 color bipmap (256 color palette)
		call	DisplayBMP_8bbp
		
		goto	DisplayingBMP_Exit
;
DisplayingBMP_Chk16bpp:
		mov		#16,		W0
		cp		BitMapBitsPerPixel
		bra		NZ,		DisplayingBMP_Chk24bpp
;		
DisplayingBMP_16bpp:					; 65535 color bipmap (no palette)
		call	DisplayBMP_16bbp
		
		goto	DisplayingBMP_Exit
;
DisplayingBMP_Chk24bpp:
		mov		#24,		W0
		cp		BitMapBitsPerPixel
		bra		NZ,		DisplayingBMP_32bpp
;		
DisplayingBMP_24bpp:					; 2^24 color bitmap (no palette)	
;		
		call	DisplayBMP_24bbp
		
		goto	DisplayingBMP_Exit
;
DisplayingBMP_32bpp:					; 2^32 color bitmap (no palette)
;		
		call	DisplayBMP_32bbp
;		
		goto	DisplayingBMP_Exit
;		
DisplayingBMP_Exit:
		call	MirrorImage
;
 		call	FlipUpsideDown	
;
		call	FullScreenWindow		; reset window size to full screen
;
		return
;}
;***************************************;**************************************
;
;	Correct Origin						- shift BMP within screen
;										 
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		LCDFlags.UpsideDownDisplay
;		LCDFlags.MirrorDisplay
;	Changes:
;		none		
; 	Calls:
;		AdjustY
;		AdjustX
;		
;***************************************;**************************************
;{		
CorrectOrigin:
		btss	LCDFlags,	#UpsideDownDisplay	; skip next if Landscape
		goto	CorrectOrigin_RightSideUp		
;
CorrectOrigin_UpsideDown:	; after upsidedown and mirror
		btss	LCDFlags,	#MirrorDisplay	; skip next if Landscape
		goto	CorrectOrigin_UpsideDown_NotMirrored
;
CorrectOrigin_UpsideDown_Mirrored:	; after upsidedown and mirror
		call	AdjustY
;
		goto	CorrectOrigin_Exit
;
CorrectOrigin_UpsideDown_NotMirrored:	; after upsidedown and mirror
		call	AdjustX
;
		call	AdjustY
;
		goto	CorrectOrigin_Exit
;
CorrectOrigin_RightSideUp:
		btss	LCDFlags,	#MirrorDisplay	; skip next if Landscape
		goto	CorrectOrigin_RightSideUp_NotMirrored
;
CorrectOrigin_RightSideUp_Mirrored:	; after upsidedown and mirror
		call	AdjustX
;
		goto	CorrectOrigin_Exit
;
CorrectOrigin_RightSideUp_NotMirrored:	; after upsidedown and mirror
		goto	CorrectOrigin_Exit
;
CorrectOrigin_Exit:
		return		
;}
;***************************************;**************************************
;
;	BuildAColor							- build a 16-bit color value from 8
;										 8-bit colors
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_Red
;		TFTB_Green
;		TFTB_Blue
;	Changes:
;		W0
;		TFTW_PixelColor		
; 	Calls:
;		none
;
;***************************************;**************************************
;{		
BuildAColor:		
		mov		#0x00F8,	W0			; want the MS 6-bits							
		and.b	TFTB_Red,	WREG
		swap	W0						; move 6-bits to the MSByte						
		mov		W0,		TFTW_PixelColor	; save red 
;
		mov		#0x00FC,	W0			; want the MS 5-bits
		and.b	TFTB_Green,	WREG		
		bclr	SR,		#C				; Carry=0
		rlc		W0,		W0				; W1 times 2
		bclr	SR,		#C				; Carry=0
		rlc		W0,		W0				; W1 times 2		 
		bclr	SR,		#C				; Carry=0
		rlc		W0,		W0				; W1 times 2 (total *8)												
		ior		TFTW_PixelColor			; add green in it
;
		clr		W0
		mov.b	TFTB_Blue,	WREG
		bclr	SR,		#C				; Carry=0
		rrc		W0,		W0				; W1 / 2 							
		bclr	SR,		#C				; Carry=0
		rrc		W0,		W0				; W1 / 2 
		bclr	SR,		#C				; Carry=0
		rrc		W0,		W0				; W1 / 2 (total /8)		
		ior		TFTW_PixelColor			; add green in it
		return				
;}
;***************************************;**************************************
;
;	DisplayBMP_1bbp						- monochrome bipmap (2 color palette)
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		BMP_RowMax
;		BMP_Height_Count
;		W10
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;		#ColorTable1+2
;		#ColorTable1
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;
;***************************************;**************************************
;{		
DisplayBMP_1bbp:		
		mov.b	[W10++],	W2
		clr		W3						; bit counter
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W1,		W0	
		bra		Z,		DisplayBMP_1bbp_Exit	; branch if done		
;
DisplayBMP_1bbp_ChkByte:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		LTU,	DisplayBMP_1bbp_ChkColumn	; branch if more bytes in row
;
		clr		BMP_Width_Count
		inc		BMP_Height_Count
		goto	DisplayBMP_1bbp_DoPixel
;				
DisplayBMP_1bbp_ChkColumn:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GEU,	DisplayBMP_1bbp_SkipPixel	; branch if no move pixels in row
;		
DisplayBMP_1bbp_DoPixel:				
		rlc.b	W2,		W2
		btss	SR,		#C				; skip next if Carry=1
		goto	Byte_Moving_1bpp_0		
;
Byte_Moving_1bpp_1:
		mov		#ColorTable1+2,	W1
		goto	Byte_Moving_1bpp_Continue
;
Byte_Moving_1bpp_0:
		mov		#ColorTable1,	W1
		goto	Byte_Moving_1bpp_Continue
;
Byte_Moving_1bpp_Continue:
		mov		[W1++],	W0				; read the color
		mov		W0,		TFTW_PixelColor		
;						
		call	SetNextPixel
;
DisplayBMP_1bbp_SkipPixel:
		inc		W3,		W3
		inc		BMP_Width_Count
		cp		W3,		#8
		bra		NZ,		DisplayBMP_1bbp_ChkByte		
;
		call	Chk4EndOfSectorBuffer		
;
		goto	DisplayBMP_1bbp
;	
DisplayBMP_1bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	DisplayBMP_4bbp	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		BMP_RowMax
;		BMP_Height_Count
;		W10
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;		#ColorTable1
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;
;***************************************;**************************************
;{		
DisplayBMP_4bbp:
		mov.b	[W10++],	W2
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W0,		W1	
		bra		Z,		DisplayBMP_4bbp_Exit	; branch if done
;
DisplayBMP_4bbp_ChkByte:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		LTU,	DisplayBMP_4bbp_ChkColumnFirstNibblePixel	; branch if more bytes in row
;
		clr		BMP_Width_Count
		inc		BMP_Height_Count
		goto	DisplayBMP_4bbp_DoFirstNibblePixel
;				
DisplayBMP_4bbp_ChkColumnFirstNibblePixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GEU,	DisplayBMP_4bbp_SkipFirstNibblePixel	; branch if no move pixels in row
;		
DisplayBMP_4bbp_DoFirstNibblePixel:				
		mov.b	W2,		W1				; get the value in W1
		and		#0x000F,	W1			; we want just the lower 4 bits
		bclr	SR,		#C				; Carry=0
		rlc		W1,		W1				; W1 times 2
		mov		#ColorTable1,	W0		; beginning of color table
		add		W1,		W0,		W1		; add W1 as the offset
		mov		[W1++],	W0				; read the blue byte
		mov		W0,		TFTW_PixelColor		
;				
		call	SetNextPixel
;
DisplayBMP_4bbp_SkipFirstNibblePixel:
		inc		BMP_Width_Count
;
DisplayBMP_4bbp_ChkColumnSecondNibblePixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GEU,	DisplayBMP_4bbp_SkipSecondNibblePixel	; branch if no move pixels in row
;				
DisplayBMP_4bbp_DoSecondNibblePixel:
		mov.b	W2,		W1
		swap.b	W1
		and		#0x000F,	W1
		bclr	SR,		#C
		rlc		W1,		W1
		mov		#ColorTable1,	W0		; beginning of color table
		add		W1,		W0,		W1		; add W1 as the offset
		mov		[W1++],	W0				; read the blue byte
		mov		W0,		TFTW_PixelColor				
;						
		call	SetNextPixel
;
DisplayBMP_4bbp_SkipSecondNibblePixel:
		inc		BMP_Width_Count
;		
		call	Chk4EndOfSectorBuffer		
;
		goto	DisplayBMP_4bbp		
;
DisplayBMP_4bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	DisplayBMP_8bbp	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		BMP_RowMax
;		BMP_Height_Count
;		W10
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;		#ColorTable1
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;
;***************************************;**************************************
;{		
DisplayBMP_8bbp:
		clr		W2
		mov.b	[W10++],	W2
;
Byte_Moving_8bpp_Loop:
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W1,		W0	
		bra		Z,		DisplayBMP_8bbp_Exit	; branch if done
;
DisplayBMP_8bbp_ChkByte:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		LEU,	DisplayBMP_8bbp_ChkColumnPixel	; branch if more bytes in row
;
		clr		BMP_Width_Count
		inc		BMP_Height_Count
;				
DisplayBMP_8bbp_ChkColumnPixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GTU,	DisplayBMP_8bbp_SkipPixel	; branch if no move pixels in row
;				
DisplayBMP_8bbp_DoPixel:	
		mov		#ColorTable1,	W1
		add		W2,		W1,		W1
		add		W2,		W1,		W1		
		mov		[W1++],	W0				; read the color
		mov		W0,		TFTW_PixelColor			
;						
		call	SetNextPixel
;
DisplayBMP_8bbp_SkipPixel:
		inc		BMP_Width_Count
;		
		call	Chk4EndOfSectorBuffer		
;
		goto	DisplayBMP_8bbp		
;	
DisplayBMP_8bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	DisplayBMP_16bbp	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;		TFTB_Blue
;		TFTB_Green
;		TFTB_Red
;		BMP_RowMax
;		BMP_Height_Count
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;		BuildAColor
;
;***************************************;**************************************
;{		
DisplayBMP_16bbp:
		mov.b	[W10++],	W0				; read the blue byte
		mov.b	WREG,	TFTB_Blue
		call	Chk4EndOfSectorBuffer
;		
		mov.b	[W10++],	W0				; read the green byte
		mov.b	WREG,	TFTB_Green
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; read the red byte	
		mov.b	WREG,	TFTB_Red
		call	Chk4EndOfSectorBuffer
;				
		call	BuildAColor
;
Byte_Moving_16bpp_Loop:
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W0,		W1	
		bra		Z,		DisplayBMP_16bbp_Exit	; branch if done
;
DisplayBMP_16bbp_ChkByte:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W0,		W1	
		bra		GTU,	DisplayBMP_16bbp_ChkColumnPixel	; branch if more bytes in row
;
		clr		BMP_Width_Count
		inc		BMP_Height_Count
;				
DisplayBMP_16bbp_ChkColumnPixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GTU,	DisplayBMP_16bbp_SkipPixel	; branch if no move pixels in row
;				
DisplayBMP_16bbp_DoPixel:
		call	SetNextPixel
;
DisplayBMP_16bbp_SkipPixel:
		inc		BMP_Width_Count
		goto	DisplayBMP_16bbp		
;	
DisplayBMP_16bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	DisplayBMP_24bbp	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;		TFTB_Blue
;		TFTB_Green
;		TFTB_Red
;		BMP_RowMax
;		BMP_Height_Count
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;		BuildAColor
;
;***************************************;**************************************
;{		
DisplayBMP_24bbp:
		mov		#3,		W0
		mul		BMP_ColumnMax
		and		W2,		#3,		W2
		mov		#4,		W0
		sub		W0,		W2,		W0
		add		BMP_ColumnMax,	WREG
		mov		W0,		BMP_Width_DataMax			
;
DisplayBMP_24bbp_Loop:
		mov.b	[W10++],	W0				; read the blue byte
		mov.b	WREG,	TFTB_Blue
		call	Chk4EndOfSectorBuffer
;		
		mov.b	[W10++],	W0				; read the green byte
		mov.b	WREG,	TFTB_Green
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; read the red byte	
		mov.b	WREG,	TFTB_Red
		call	Chk4EndOfSectorBuffer
;				
		call	BuildAColor
;
DisplayBMP_24bbp_DoPixel:
		call	SetNextPixel
;
		inc		BMP_Width_Count
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W1,		W0	
		bra		Z,		DisplayBMP_24bbp_Exit	; branch if done
;				
DisplayBMP_24bbp_ChkColumnPixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GEU,	DisplayBMP_24bbp_SkipPixel	; branch if no move pixels in row		
;		
		goto	DisplayBMP_24bbp_Loop		
;
DisplayBMP_24bbp_SkipPixel:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GEU,	DisplayBMP_24bbp_NewRow	; branch if more bytes in row
;
		inc		BMP_Width_Count
		mov.b	[W10++],	W0			; dummy read (should be zero)
		call	Chk4EndOfSectorBuffer
;				
		goto	DisplayBMP_24bbp_SkipPixel
;		
DisplayBMP_24bbp_NewRow:
		clr		BMP_Width_Count
		inc		BMP_Height_Count
		goto	DisplayBMP_24bbp_Loop
;
DisplayBMP_24bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	DisplayBMP_32bbp					- 2^32 color bitmap (no palette)
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;		TFTB_Blue
;		TFTB_Green
;		TFTB_Red
;		BMP_RowMax
;		BMP_Height_Count
;		BMP_Width_DataMax
;		BMP_Width_Count
;		BMP_ColumnMax
;	Changes:
;		W0
;		W1
;		W2
;		W3
;		BMP_Width_Count	
;		BMP_Height_Count
;		TFTW_PixelColor
; 	Calls:
;		SetNextPixel
;		Chk4EndOfSectorBuffer
;		BuildAColor
;
;***************************************;**************************************
;{		
DisplayBMP_32bbp:					
		mov.b	[W10++],	W0				; read the blue byte
		mov.b	WREG,	TFTB_Blue
		call	Chk4EndOfSectorBuffer
;		
		mov.b	[W10++],	W0				; read the green byte
		mov.b	WREG,	TFTB_Green
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; read the red byte	
		mov.b	WREG,	TFTB_Red
		call	Chk4EndOfSectorBuffer
;				
		mov.b	[W10++],	W0				; 4th byte is always zero not used
;		
		call	Chk4EndOfSectorBuffer
;				
		call	BuildAColor
;
Byte_Moving_32bpp_Loop:
		mov		BMP_RowMax,	W0
		mov		BMP_Height_Count,	W1
		cp		W0,		W1	
		bra		Z,		DisplayBMP_32bbp_Exit	; branch if done
;
DisplayBMP_32bbp_ChkByte:
		mov		BMP_Width_DataMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W0,		W1	
		bra		GTU,	DisplayBMP_32bbp_ChkColumnPixel	; branch if more bytes in row
;
		clr		BMP_Width_Count
		inc		BMP_Height_Count
;				
DisplayBMP_32bbp_ChkColumnPixel:
		mov		BMP_ColumnMax,	W0
		mov		BMP_Width_Count,	W1
		cp		W1,		W0	
		bra		GTU,	DisplayBMP_32bbp_SkipPixel	; branch if no move pixels in row
;				
DisplayBMP_32bbp_DoPixel:
		call	SetNextPixel
;
DisplayBMP_32bbp_SkipPixel:
		inc		BMP_Width_Count
		goto	DisplayBMP_32bbp		
;
DisplayBMP_32bbp_Exit:		
		return		
;}
;***************************************;**************************************
;
;	Goto A FAT Entry
;
;	SectorOfInterestH/L:	First Sector (LBA) of a FAT Region
;	FATEntry:				Cluster position to look for in FAT
;	FATOffsetCount:			Present Word position in the LBA
;	FATSectorCount:			Present FAT Sector
;	BlockAddressH/L:		LBA for Sector read or write
;	#SPI_Data_buffer:		Working Sector buffer
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		FATEntry
;		SectorOfInterestH
;		SectorOfInterestL
;		FATSectorCount
;	Changes:
;		W0
;		W1
;		W2
;		FATOffsetCount
;		BlockAddressL
;		BlockAddressH
; 	Calls:
;		CMD17
;
;***************************************;**************************************
;{
GotoAFATEntry:
		mov		FATEntry,W0				; Cluster to look for
		and		#0x00FF,W0				; lower byte becomes offset into Cluster
		mov		W0,		FATOffsetCount	; save this offset
		mov		FATEntry,W0				; Cluster to look for
		swap	W0						; upper byte now becomes lower byte
		and		#0x00FF,W0				; lower byte becomes sequential Sector	
		mov		W0,		FATSectorCount	; save the Sector Count
		mov		SectorOfInterestH,W0	; take the first Sector of a FAT
		mov		SectorOfInterestL,W1	;
		mov		FATSectorCount,W2		; and the Sector offset 
		add		W2,		W1,		W1		; and add them
		clr		W2
		addc	W2,		W0,		W0
		mov		W1,		BlockAddressL	; this becomes the Sector of the FATEntry
		mov		W0,		BlockAddressH
		call	CMD17					; get this Sector
;
GotoAFATEntry_Exit:
		return		
;}
;***************************************;**************************************
;
;	ReadAFATEntry:		Read A FAT Entry	
;
;	FATOffsetCount:			Present Word position in the LBA
;	#SPI_Data_buffer:		Working Sector buffer
;	FATEntryValue:			Word read at FATOffsetCount of the SPI_Data_buffer
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#SPI_Data_buffer+1
;		FATOffsetCount
;	Changes:
;		W0
;		W10
;		FATEntryValue
; 	Calls:
;		CMD17
;
;***************************************;**************************************
;{
ReadAFATEntry:
		mov		#SPI_Data_buffer+1,W10	; get a pointer to the buffer
		mov		FATOffsetCount,W0		; and the offset into the buffer
		sl		W0,		W0				; offset is times 2 (WORD_STROBE)
		add		W0,		W10,	W10		; add then up
			
		mov.b	[W10++],W0				; get the LSByte
		swap	W0						; make it the MSB
		mov.b	[W10--],	W0			; get the MSByte
		swap	W0						; swap them again
		mov		W0,		FATEntryValue	; this becomes the FATEntryValue		
;
ReadAFATEntry_Exit:
		return							
;}
;***************************************;**************************************
;
;	Initialize Media	
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		#SPI_CMD_Response_inputbuffer
;		VBSH
;		VBSL
;		NumDirEntries
;		NumDirInSector
;		NumSecInFAT
;		FATRegionSectorL
;	Changes:
;		W0
;		W10
;		SDFlags.MediaReady
;		BlockAddressH
;		BlockAddressL
;		NumSecInCluster
;		NumOfFATs
;		RootDirSector
;		LastDirectoryCluster		
;		SectorOfInterestL
;		SectorOfInterestH
; 	Calls:
;		SD_Reset
;		CMD0
;		CMD1
;		CMD8
;		CMD58
;		CMD55
;		CMDA41
;		CMD17
;
;***************************************;**************************************
;{
Init_Media:
		bclr	SDFlags,	#MediaReady
		call	SD_Reset
;		
		call	SD_Reset
;
Init_Media_CMD0:		
		call	CMD0					; reset and enter idle state
;		
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;		
Init_Media_CMD1:						
		call	CMD1					; Sends host capacity support information		
;		
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;
Init_Media_CMD8:							
		call	CMD8					; Sends SD Memory Card interface
										; condition that includes host supply
										; voltage information and asks the
										; accessed card whether card can
										; operate in supplied voltage range. 
;		
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;
Init_Media_CMD58:						
		call	CMD58					; Read OCR (Operating Conditions Register) 
;		
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;
Init_Media_CMD55:						; Next Command will be application-specific (ACMDXX)		
		call	CMD55
;
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;		
Init_Media_CMD41:						; Requests the capacity support information		
		call	CMDA41
;
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;	
		mov		#SPI_CMD_Response_inputbuffer,W10
		mov		[W10],	W0
		btsc	W0,		#InIdleState
		bra		Init_Media_CMD55
;
Init_Media_Set_Sector_Address_To_Zero:
		mov		#BlockAddressH,W10
		mov		#0x0000,W0
		mov		W0,		[W10++]
		mov		W0,		[W10]
;
Init_Media_CMD17:							
		call	CMD17					; Read a Sector (BlockAddressH:L)
;
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit
;
Init_Media_Check_MSBs_First_Data_Value:
		mov		#SPI_Data_buffer+1,W10
		mov.b	[W10],	W0
		cp0.b	W0
		bra		z,		Init_Media_GetPartionInfo
;
		bset	SDFlags,	#BadMedia		
		bra		Init_Media_Exit
;
Init_Media_GetPartionInfo:
		mov		#SPI_Data_buffer+0x1CA,W10
		mov		#VBSH+1,W11
		mov.b	[W10--],[W11--]
		mov.b	[W10--],[W11++]
		mov		#VBSL+1,W11		
		mov.b	[W10--],[W11--]
		mov.b	[W10],[W11]						
;
Init_Media_Set_VBS:
		mov		VBSH,W0
		mov		W0,		BlockAddressH
		mov		VBSL,W0
		mov		W0,		BlockAddressL
;
Init_Media_GetFATRegionSector:	
		call	CMD17					; Read a Sector (BlockAddressH:L)
;
		btsc	SDFlags,	#BadMedia	; skip if Media OK
		bra		Init_Media_Exit	
;
Init_Media_Check_FATRegions_First_Data_Value:
		mov		#SPI_Data_buffer+1,W10
		mov.b	[W10],	W0
		mov		#0xEB,	W1
		cp.b	W0,		W1
		bra		z,		Init_Media_GotFATRegionSector	
;
		bra		Init_Media_Exit
;				
Init_Media_GotFATRegionSector:			; found the FAT
		mov		#SPI_Data_buffer+18,W10
		mov		#NumDirEntries,W11
		mov.b	[W10++],[W11++]
		mov.b	[W10],[W11]
;		
		mov		#(512/32),W0
		mov		W0,		NumDirInSector
;		
		mov		NumDirEntries,W0
		mov		NumDirInSector,W2
		repeat	#17
		div.u	W0,		W2
		mov		W0,		NumSecInCluster
;		
		mov		#SPI_Data_buffer+17,W10
		clr		W0
		mov.b	[W10],	W0
		mov		W0,		NumFATs
;		
		mov		#SPI_Data_buffer+23,W10
		mov		#NumSecInFAT,W11			
		mov.b	[W10++],[W11++]
		mov.b	[W10],[W11]
;		
		mov		#SPI_Data_buffer+15,W10
		mov		#ReservedSectorCount,W11
		mov.b	[W10++],	[W11++]
		mov.b	[W10],	[W11]
		mov		ReservedSectorCount,W10
;
		clr		FATRegionSectorL
		clr		FATRegionSectorH
		mov		VBSL,	W0
		add		W10,	W0,		W10
		mov		W10,	FATRegionSectorL
		mov		VBSH,	W0
		mov		FATRegionSectorH,W10
		addc	W10,	W0,		W10
		mov		W10,	FATRegionSectorH
;		
		mov		NumFATs,W0
		mov		NumSecInFAT,W1
		mul.uu	W0,W1,W2
		mov		FATRegionSectorL,W0
		add		W0,		W2,		W0
		mov		W0,		RootDirSector
		clr		LastDirectoryCluster		
		mov		W0,		SectorOfInterestL
		clr		SectorOfInterestH				
;
		bset	SDFlags,	#MediaReady
;		
Init_Media_Exit:
		return				
;}	
;***************************************;***************************************
;
;	Subroutine: Send_Registers			- Dump of all registers to UART1
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		W0
;		W1
;		TFT_IR
; 	Calls:
;		Read_TFT
;		DisplayRegister
;
;***************************************;***************************************
;{		
Send_Registers:
		push	W2
		mov		#tblpage(TFT_Registers), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(TFT_Registers), W2	; Make a pointer to table
;
Send_Registers_Loop:
		tblrdl.b	[W2++],	W0			; get a register from the table
		mov.b	WREG,	TFT_IR			; TFT_IR = what we want
;
		call	Read_TFT				; go get it
;
		call	DisplayRegister			; send it to the UART
;
		mov		#0xA5,	W0				; this is the end of the table
		cp.b	TFT_IR					; TFT_IR=0xA5?
		bra		NZ,		Send_Registers_Loop	; yes, more to do
;
Send_Registers_Exit:					; else done
		pop		W2
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Read_TouchX
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TouchFlags
;	Changes
;		TouchFlags
;		ANSA.PlusY
;		ANSA.MinusY
;		ANSA.PlusX
;		ANSA.MinusX
;		LATA.PlusY
;		LATA.MinusY
;		LATA.PlusX
;		LATA.MinusX
;		TRISA.PlusY
;		TRISA.MinusY
;		TRISA.PlusX
;		TRISA.MinusX
;		AD1CON1.SAMP
;		AD1CHS
;	Calls
;		none
;
;***************************************;***************************************
;{
Read_TouchX:
		mov		#0x000F,W0
		ior		TRISA,	WREG	
		mov		W0,		TRISA
;
		mov		#0xFFF0,W0
		and		LATA,	WREG
		mov		W0,		LATA
;
		mov		#0xFFF0,	W0
		and		TouchFlags,	WREG
		ior		#1<<ReadingX,	W0
		mov		W0,		TouchFlags
;		
		mov		ANSA,	W0
		bset	W0,		#PlusY			; analog
		bset	W0,		#MinusY			; analog
		bclr	W0,		#PlusX			; digital
		bclr	W0,		#MinusX			; digital
		mov		W0,		ANSA
;
		mov		LATA,	W0
		bclr	W0,		#PlusY			; low
		bclr	W0,		#MinusY			; low		
		bset	W0,		#PlusX			; high
		bclr	W0,		#MinusX			; low
		mov		W0,		LATA				
;
		mov		TRISA,	W0
		bset	W0,		#PlusY			; input
		bset	W0,		#MinusY			; input
		bclr	W0,		#PlusX			; output
		bclr	W0,		#MinusX			; output
		mov		W0,		TRISA
;
		mov		#0x0001,	W0			; CH0NB=000, CH0SB=00000, 
										; CH0NA=000, CH0SA=00001
		mov		W0,		AD1CHS			; Select Channel A1 (PlusY)
;
		bset	AD1CON1,	#SAMP		; Start sampling											
;
Read_TouchX_Exit:
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Read_TouchY
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TouchFlags
;	Changes
;		TouchFlags
;		ANSA.PlusY
;		ANSA.MinusY
;		ANSA.PlusX
;		ANSA.MinusX
;		LATA.PlusY
;		LATA.MinusY
;		LATA.PlusX
;		LATA.MinusX
;		TRISA.PlusY
;		TRISA.MinusY
;		TRISA.PlusX
;		TRISA.MinusX
;		AD1CON1.SAMP
;		AD1CHS
;	Calls
;		none
;
;***************************************;***************************************
;{
Read_TouchY:
		mov		#0x000F,W0
		ior		TRISA,	WREG	
		mov		W0,		TRISA
;
		mov		#0xFFF0,W0
		and		LATA,	WREG
		mov		W0,		LATA
;
		mov		#0xFFF0,	W0
		and		TouchFlags,	WREG
		ior		#1<<ReadingY,	W0
		mov		W0,		TouchFlags						
;
		mov		ANSA,	W0
		bclr	W0,		#PlusY			; digital
		bclr	W0,		#MinusY			; digital
		bset	W0,		#PlusX			; analog
		bset	W0,		#MinusX			; analog
		mov		W0,		ANSA
;
		mov		LATA,	W0
		bset	W0,		#PlusY			; low
		bclr	W0,		#MinusY			; high		
		bclr	W0,		#PlusX			; low
		bclr	W0,		#MinusX			; low
		mov		W0,		LATA				
;
		mov		TRISA,	W0
		bclr	W0,		#PlusY			; output
		bclr	W0,		#MinusY			; output
		bset	W0,		#PlusX			; input
		bset	W0,		#MinusX			; input	
		mov		W0,		TRISA
;
		mov		#0x0000,	W0			; CH0NB=000, CH0SB=00000, 
										; CH0NA=000, CH0SA=00000
		mov		W0,		AD1CHS			; Select Channel A1 (MinusX)
;
		bset	AD1CON1,	#SAMP		; Start sampling											
;
Read_TouchY_Exit:
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Read_TouchZ1
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TouchFlags
;	Changes
;		TouchFlags
;		ANSA.PlusY
;		ANSA.MinusY
;		ANSA.PlusX
;		ANSA.MinusX
;		LATA.PlusY
;		LATA.MinusY
;		LATA.PlusX
;		LATA.MinusX
;		TRISA.PlusY
;		TRISA.MinusY
;		TRISA.PlusX
;		TRISA.MinusX
;		AD1CON1.SAMP
;		AD1CHS
;	Calls
;		none
;
;***************************************;***************************************
;{
Read_TouchZ1:
		mov		#0x000F,W0
		ior		TRISA,	WREG	
		mov		W0,		TRISA
;
		mov		#0xFFF0,W0
		and		LATA,	WREG
		mov		W0,		LATA
;
		mov		#0xFFF0,	W0
		and		TouchFlags,	WREG
		ior		#1<<ReadingZ1,	W0
		mov		W0,		TouchFlags						
;
		mov		ANSA,	W0
		bset	W0,		#PlusY			; analog
		bclr	W0,		#MinusY			; digital
		bclr	W0,		#PlusX			; digital
		bset	W0,		#MinusX			; analog
		mov		W0,		ANSA
;
		mov		LATA,	W0
		bclr	W0,		#PlusY			; low
		bset	W0,		#MinusY			; high		
		bclr	W0,		#PlusX			; low
		bclr	W0,		#MinusX			; low
		mov		W0,		LATA				
;
		mov		TRISA,	W0
		bset	W0,		#PlusY			; input
		bclr	W0,		#MinusY			; output
		bclr	W0,		#PlusX			; output
		bset	W0,		#MinusX			; input	
		mov		W0,		TRISA
;
		mov		#0x0001,	W0			; CH0NB=000, CH0SB=00000, 
										; CH0NA=000, CH0SA=00001
		mov		W0,		AD1CHS			; Select Channel A1 (PlusY)
;
		bset	AD1CON1,	#SAMP		; Start sampling
Read_TouchZ1_Exit:
		return
;}	
;***************************************;***************************************
;
;	Subroutine: Read_TouchZ2
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TouchFlags
;	Changes
;		TouchFlags
;		ANSA.PlusY
;		ANSA.MinusY
;		ANSA.PlusX
;		ANSA.MinusX
;		LATA.PlusY
;		LATA.MinusY
;		LATA.PlusX
;		LATA.MinusX
;		TRISA.PlusY
;		TRISA.MinusY
;		TRISA.PlusX
;		TRISA.MinusX
;		AD1CON1.SAMP
;		AD1CHS
;	Calls
;		none
;
;***************************************;***************************************
;{
Read_TouchZ2:
		mov		#0x000F,W0
		ior		TRISA,	WREG	
		mov		W0,		TRISA
;
		mov		#0xFFF0,W0
		and		LATA,	WREG
		mov		W0,		LATA
;		
		mov		#0xFFF0,	W0
		and		TouchFlags,	WREG
		ior		#1<<ReadingZ2,	W0
		mov		W0,		TouchFlags						
;
		mov		ANSA,	W0
		bset	W0,		#PlusY			; analog
		bclr	W0,		#MinusY			; digital
		bclr	W0,		#PlusX			; digital
		bset	W0,		#MinusX			; analog
		mov		W0,		ANSA
;
		mov		LATA,	W0
		bclr	W0,		#PlusY			; low
		bset	W0,		#MinusY			; high		
		bclr	W0,		#PlusX			; low
		bclr	W0,		#MinusX			; low
		mov		W0,		LATA				
;
		mov		TRISA,	W0
		bset	W0,		#PlusY			; input
		bclr	W0,		#MinusY			; output
		bclr	W0,		#PlusX			; output
		bset	W0,		#MinusX			; input	
		mov		W0,		TRISA
;
		mov		#0x0000,	W0			; CH0NB=000, CH0SB=00000, 
										; CH0NA=000, CH0SA=00000
		mov		W0,		AD1CHS			; Select Channel A1 (MinusX)
;
Read_TouchZ2_Exit:
		return
;}	
;***************************************;***************************************
;
;	Subroutine: SaveButtonIDToMatrix	- The ButtonID is saved into the Matrix
;										 for this Button character
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		TFTB_CharacterWidth
;		TFTW_X
;		Width
;		TFTB_CharacterHeight
;		TFTW_Y
;		Height
;		ButtonID
;		TFTB_TextSize		
;	Changes
;		W0
;		W1
;		TEMPB
;	Calls
;		none
;
;***************************************;***************************************
;{		
SaveButtonIDToMatrix:
		push.d	W2
		clr		W3
;
SaveButtonIDToMatrix_Loop:		
		mov		W3,		W0
		mul.b	TFTB_CharacterWidth	
		mov		W2,		W0
		add		TFTW_X,	WREG
		add		#3,		W0
		mov		W0,		W1				; save offset
		mov		#15,	W2
		mov		#240,	W0
		cp		Width
		btss	SR,		#Z				; skip next if Width=240
		mov		#20,	W2
		mov		W1,		W0				; move the offset		
		repeat	#17
		div.u	W0,		W2
		mov.b	WREG,	TEMPB
;		
		mov		W3,		W0
		mul.b	TFTB_CharacterHeight	
		mov		W2,		W0
		add		TFTW_Y,	WREG	
		add		#4,		W0
		mov		W0,		W1				; save the offset
		mov		#15,	W2
		mov		#240,	W0
		cp		Height
		btss	SR,		#Z				; skip next if Height=240
		mov		#20,	W2
		mov		W1,		W0				; move the offset	
		repeat	#17
		div.u	W0,		W2
		mov		W0,		W1
		swap.b	W1		
		mov.b	TEMPB,	WREG
		ior.b	W0,		W1,		W1
		mov		#TouchMatrix,	W0
		add		W0,		W1,		W1
		mov.b	ButtonID,	WREG	
		mov.b	W0,		[W1]
;
		inc		W3,		W3
		mov.b	TFTB_TextSize,	WREG
		cp.b	W0,		W3
		bra		NZ,		SaveButtonIDToMatrix_Loop		

SaveButtonIDToMatrix_Exit:
		pop.d	W2
		return											
;}	
;***************************************;***************************************
;
;	Subroutine: Check the U1RX Buffer for characters
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		U1RXBuffer_Head
;		U1RXBuffer_Tail
;	Changes
;		W0
;		W1
;		SystemFlags.U1RXEmpty
;	Calls
;		none
;
;***************************************;***************************************
;{
U1RXBuffer_Check:
		mov		U1RXBuffer_Head,	W0
		mov		U1RXBuffer_Tail,	W1
		cpseq	W0,	W1					; skip next if buffer is empty
		goto	U1RXBuffer_Check_NotEmpty
;
U1RXBuffer_Check_Empty:
		bset	SystemFlags,	#U1RXEmpty
		goto	U1RXBuffer_Check_Exit
;
U1RXBuffer_Check_NotEmpty:
		bclr	SystemFlags,	#U1RXEmpty
		goto	U1RXBuffer_Check_Exit
;
U1RXBuffer_Check_Exit:
		return
;}		
;***************************************;***************************************
;
;	Subroutine: Remove a character from the U1RX Buffer
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		U1RXBuffer_Tail
;	Changes
;		W0
;		W1
;		U1RXBuffer_Tail
;		U1RXCharacter
;		SystemFlags.U1RXEmpty
;		SystemFlags.U1RXCharacterReady
;	Calls
;		U1RXBuffer_Check
;
;***************************************;***************************************
;{
U1RXBuffer_Remove:
		disi	#20
		call	U1RXBuffer_Check
;
		btsc	SystemFlags,	#U1RXEmpty	; skip next if its not Empty
		goto	U1RXBuffer_Remove_Exit
;
		mov		U1RXBuffer_Tail,	W1
		mov.b	[W1++],		W0
		mov.b	WREG,		U1RXCharacter
		mov		#U1RXBuffer+UARTBuffer_Length,	W0
		cpseq	W0,			W1			; skip next if past the Buffer
		goto	U1RXBuffer_Remove_Room_NoWrap
;
U1RXBuffer_Remove_Room_Wrap:
		mov		#U1RXBuffer,	W1
;
U1RXBuffer_Remove_Room_NoWrap:
		mov		W1,	U1RXBuffer_Tail	
		bset	SystemFlags,	#U1RXCharacterReady					
;
U1RXBuffer_Remove_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine: Add a character to the U1TX Buffer
;
;	DateLastEdited:	4/29/2013
;
;	Requires
;		U1TXCharacter	-	Character to add
;		U1TXBuffer_Head
;		U1TXBuffer_Tail
;	Changes
;		U1TXBuffer_Head
;		IEC0.U1TXIE
;	Calls
;		none
;
;***************************************;***************************************
;{
U1TXBuffer_Add:
		push.d	W0
;
U1TXBuffer_Add_Loop:		
		disi	#9
		mov		U1TXBuffer_Head,	W1
		inc		W1,	W1
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cpsne	W0,			W1			; skip next if past the U1TXBuffer
;
		mov		#U1TXBuffer,	W1
;				
		mov		U1TXBuffer_Tail,	W0
		cpseq	W0,			W1			; skip next if there is no room
		goto	U1TXBuffer_Add_Room
;
U1TXBuffer_Add_NoRoom:
		nop
		nop
		nop
		nop
		goto	U1TXBuffer_Add_Loop
;
U1TXBuffer_Add_Room:
		disi	#9
		mov		U1TXBuffer_Head,	W1
		mov.b	U1TXCharacter,	WREG
		mov.b	W0,			[W1++]		
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cpseq	W1,			W0			; skip next if past the U1TXBuffer
		goto	U1TXBuffer_Add_Room_NoWrap	
;
U1TXBuffer_Add_Room_Wrap:
		mov		#U1TXBuffer,	W1
;
U1TXBuffer_Add_Room_NoWrap:
		mov		W1,			U1TXBuffer_Head
		bset	IEC0,		#U1TXIE		; Enable U1Transmit Interrupt	
;
U1TXBuffer_Add_Exit:
		pop.d	W0			
		return
;}
;***************************************;***************************************
;
;	UART1 TX Interrupt Service Routine
;
;	DateLastEdited:	4/29/2013
;
;	Requires:	
;		U1TXBuffer_Head
;		U1TXBuffer_Tail
;	Changes:
;		U1TXBuffer_Tail
;		IFS0.U1TXIF
;		U1TXREG
;		IEC0.U1TXIE
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U1TXInterrupt:
        push.d 	W0 						; Save context using double-word PUSH
;
U1TXInterrupt_Loop:                         	
		mov		U1TXBuffer_Head,	W0
		mov		U1TXBuffer_Tail,	W1
		cp		W0,	W1
		bra		Z,	U1TXInterrupt_NoData	; branch if Head=Tail (no data)      
;
        bclr 	IFS0,	#U1TXIF         ; Clear the U1TX Interrupt flag bit 		
		mov.b	[W1],	W0
		mov.b	WREG,	U1TXREG 
		inc		W1,	W1
		mov		#U1TXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1
		bra		NZ,	U1TXInterrupt_NoWrap
;
		mov		#U1TXBuffer,	W1        
;
U1TXInterrupt_NoWrap:
		mov		W1,	U1TXBuffer_Tail       
		goto	U1TXInterrupt_Exit
;
U1TXInterrupt_NoData:
		bclr	IEC0,	#U1TXIE         ; Prevent further U1TX interrupts
		goto  	U1TXInterrupt_Exit
;
U1TXInterrupt_Exit:       
		pop.d 	W0                   	;Retrieve context POP-ping from Stack
        retfie                     		;Return from Interrupt Service routine
;}
;***************************************;***************************************
;
;	UART1 RX Interrupt Service Routine
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		U1RXBuffer_Head
;		U1RXBuffer_Tail
;	Changes:
;		U1RXBuffer_Head
;		IFS0.U1RXIF
;		U1TXREG
;		U1STA.OERR
;		U1STA.FERR
;		U1STA.PERR
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__U1RXInterrupt:
        push.D 	W0                  	; Save context using double-word PUSH
        push.D 	W2                  	; Save context using double-word PUSH        
        bclr 	IFS0,	#U1RXIF         ; Clear the U1RX Interrupt flag bit.
;                
		btsc	U1STA,	#OERR			; skip next if no OERR
		goto	U1RXInterrupt_OERR
;		
		btsc	U1STA,	#FERR			; skip next if no FERR
		goto	U1RXInterrupt_FERR		
;		
		btsc	U2STA,	#PERR			; skip next if no PERR
		goto	U1RXInterrupt_PERR	
;
		mov		U1RXBuffer_Head,	WREG
		inc		W0,	W1
		mov		W0,	W2
		mov		#U1RXBuffer+UARTBuffer_Length,	W0
		cp		W0,	W1		
		bra		NZ,	U1RXInterrupt_NoWrap
;
		mov		#U1RXBuffer,	W1
;
U1RXInterrupt_NoWrap:				
		mov		U1RXBuffer_Tail,	W0
		cp		W0,	W1		
		bra		Z,	U1RXInterrupt_NoRoom
;
		mov.b	U1RXREG,	WREG
		mov.b	W0,	[W2]
		mov		W1,	U1RXBuffer_Head		
;
		goto	U1RXInterrupt_Exit	
;				
U1RXInterrupt_OERR:
		bclr	U1STA,	#OERR
		goto	U1RXInterrupt_Exit
;		
U1RXInterrupt_FERR:
		mov.b	U1RXREG,	WREG		; dummy read to toss out bad data
		goto	U1RXInterrupt_Exit
;		
U1RXInterrupt_PERR:	
		mov.b	U1RXREG,	WREG		; dummy read to toss out bad data	
		goto	U1RXInterrupt_Exit
;
U1RXInterrupt_NoRoom:
		mov.b	U1RXREG,	WREG		; toss out data, no room in U1RXBuffer
		goto	U1RXInterrupt_Exit
;
U1RXInterrupt_Exit:
        pop.D 	W2                   	; Retrieve context POP-ping from Stack 
        pop.D 	W0                   	; Retrieve context POP-ping from Stack        
        retfie                     		; Return from Interrupt Service routine
;}
;***************************************;***************************************
;
;	Timer2 Interrupt Service Routine	-	Touch scan
;
;	tick=16MHz/256=16uS
; 	tick*3125=50mS (overflow)
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		IFS0.T2IF
;		AD1CON1.SAMP
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__T2Interrupt:
	    bclr 	IFS0,	#T2IF         	; Clear the T2 Interrupt flag bit.
        bset	AD1CON1,	#SAMP		; start a new round of samples
;
T2Interrupt_Exit:
		retfie         
;}
;***************************************;***************************************
;
;	A/D Interrupt Service Routine	-	all channels have been read and stored
;										so just stop further samples
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TouchFlags.ReadingX
;		TouchFlags.ReadingY
;		TouchFlags.ReadingZ1
;		TouchFlags.ReadingZ2
;	Changes:
;		IFS0.AD1IF
;		AD1CON1.ASAM
;		TouchFlags.TouchReady
; 	Calls:
;		Read_TouchX
;		Read_TouchY
;		Read_TouchZ1
;		Read_TouchZ2
;
;***************************************;***************************************
;{			
__ADC1Interrupt:
		push.d	W0
		bclr	IFS0,	#AD1IF			; clear the interrupt
;
ADC1Interrupt_Chk4X:		
		btss	TouchFlags,	#ReadingX
		goto	ADC1Interrupt_Chk4Y
;
ADC1Interrupt_X:
		mov		ADC1BUF1,	W0
		swap	W0
		mov.b	WREG,	TFTB_TouchX
		call	Read_TouchY
;		
		goto	ADC1Interrupt_Exit
;	
ADC1Interrupt_Chk4Y:
		btss	TouchFlags,	#ReadingY
		goto	ADC1Interrupt_Chk4Z1
;
ADC1Interrupt_Y:
		mov		ADC1BUF0,	W0
		swap	W0
		mov.b	WREG,	TFTB_TouchY
		call	Read_TouchZ1
;
		goto	ADC1Interrupt_Exit
;	
ADC1Interrupt_Chk4Z1:
		btss	TouchFlags,	#ReadingZ1
		goto	ADC1Interrupt_Z2
;
ADC1Interrupt_Z1:
		mov		ADC1BUF1,	W0
		swap	W0
		mov.b	WREG,	TFTB_TouchZ1
		call	Read_TouchZ2
;
		goto	ADC1Interrupt_Exit
;	
ADC1Interrupt_Z2:
		mov		ADC1BUF0,	W0
		swap	W0
		mov.b	WREG,	TFTB_TouchZ2
		call	Read_TouchX
;		
		bset	TouchFlags,	#TouchReady	
		goto	ADC1Interrupt_Exit

ADC1Interrupt_Exit:
		pop.d	W0	
		retfie  
;}
;***************************************;***************************************
;
;	Function:		SPI1Interrupt
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI1BUF
;		SPI_Data_Length
;		SPI_Data_Delay
;		SDFlags.MediaMode
;		SPI_Data_buffer_pointer
;		SPI_CMD_Response_inputbuffer_pointer
;		SDFlags.MediaMode
;		SPI_Data_Response_Length
;		SPI_Data_Response_Delay
;	Changes:
;		IFS0.AD1IF
;		AD1CON1.ASAM
;		TouchFlags.TouchReady
;		SPI_Data_buffer_pointer
;		SPI1BUF
;		SPI_CMD_Response_Length
;		SPI_CMD_Response_inputbuffer_pointer
;		SPI_CMD_Response_Delay
;		LATA.SD_CS
;		IFS0.SPI1IF
;		SPI_DATA_Response_Length
;		SPI_DATA_Response_inputbuffer_pointer
;		SPI_DATA_Response_Delay
; 	Calls:
;		none
;
;***************************************;***************************************
;{
__SPI1Interrupt:
		push.d	W0
		push.d	W10
		push.d	W12
		cp0		SPI_CMD_Length
		bra		nz,		__SPI1Interrupt_NextCMDChk	; branch if SPI_CMD_Length<>0
;
		cp0		SPI_CMD_Response_Length
		bra		nz,		__SPI1Interrupt_NextResponseChk	
										; branch if SPI_CMD_Response_Length<>0
;		
		mov		SPI1BUF,	W12
		cp0		SPI_Data_Length
		bra		z,		__SPI1Interrupt_Done	; branch if SPI_Data_Length=0
;
__SPI1Interrupt_NextDataChk:
		cp0		SPI_Data_Delay
		bra		nz,		__SPI1Interrupt_DataDelay	; skip if DataDelay<>0
;
		btss	SDFlags,	#MediaMode		; skip next if Media Read
		bra		__SPI1Interrupt_WriteData
;		
__SPI1Interrupt_ReadData:
		mov		SPI_Data_buffer_pointer,W10
		mov.b	W12,	[W10]
		inc		SPI_Data_buffer_pointer
;
		cp0		SPI_Data_Length
		bra		z,		__SPI1Interrupt_Done
;
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF
		dec		SPI_Data_Length		
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_DataDelay:	
		mov		#0xFF,	W11				
		cpsne	W11,	W12				; skip if SPI_Data<>0xFF
		bra		__SPI1Interrupt_DataDelayLoop
;
		clr		SPI_Data_Delay
		bra		__SPI1Interrupt_ReadData
;
__SPI1Interrupt_DataDelayLoop:			
		dec		SPI_Data_Delay
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF		
		bra		__SPI1Interrupt_Exit				
;
__SPI1Interrupt_NextResponseChk:
		mov		SPI1BUF,	W12
		cp0		SPI_CMD_Response_Delay
		bra		nz,		__SPI1Interrupt_ResponseDelay	; skip if DataDelay<>0
;
__SPI1Interrupt_Response:
		mov		SPI_CMD_Response_inputbuffer_pointer,W10
		mov.b	W12,	[W10]
		inc		SPI_CMD_Response_inputbuffer_pointer
		dec		SPI_CMD_Response_Length
		bra		z,		__SPI1Interrupt_ResponseLast	; branch if SPI_CMD_Response_Length=0	
;		
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_ResponseLast:	
		mov		#SPI_Data_buffer,W10
		mov		W10,	SPI_Data_buffer_pointer
		mov		#DelayCount,W11
		mov		W11,	SPI_Data_Delay
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF		
		bra		__SPI1Interrupt_Exit			
;
__SPI1Interrupt_ResponseDelay:	
		btsc	W12,	#7				; Legal Response
		bra		__SPI1Interrupt_ResponseDelayLoop
;
		clr		SPI_CMD_Response_Delay
		bra		__SPI1Interrupt_Response
;
__SPI1Interrupt_ResponseDelayLoop:			
		dec		SPI_CMD_Response_Delay
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF		
		bra		__SPI1Interrupt_Exit				
;
__SPI1Interrupt_NextCMDChk:
		mov		SPI1BUF,	W12
		cp0		SPI_CMD_Delay
		bra		nz,		__SPI1Interrupt_CMDDelay	; skip if CMDDelay<>0
;
__SPI1Interrupt_CMD:
		mov		SPI_CMD_inputbuffer_pointer,W10
		mov.b	W12,	[W10]
		inc		SPI_CMD_inputbuffer_pointer
		dec		SPI_CMD_Length
		bra		z,		__SPI1Interrupt_CMDLast; branch if SPI_Data_Length0	
;		
		mov		SPI_CMD_outputbuffer_pointer,W10
		mov.b	[W10],	W0
		mov.b	WREG,	SPI1BUF
		inc		SPI_CMD_outputbuffer_pointer		
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_CMDLast:	
		mov		#SPI_CMD_Response_inputbuffer,W10
		mov		W10,	SPI_CMD_Response_inputbuffer_pointer
		mov		#DelayCount,W11
		mov		W11,	SPI_CMD_Response_Delay
		mov		#0xFF,	W11
		mov		W11,	SPI1BUF		
		bra		__SPI1Interrupt_Exit				
;
__SPI1Interrupt_CMDDelay:	
		clr		SPI_CMD_Delay
		mov		SPI_CMD_outputbuffer_pointer,W10
		mov.b	[W10],	W11
		mov		W11,	SPI1BUF
		inc		SPI_CMD_outputbuffer_pointer			
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_WriteData:
		mov		SPI_Data_buffer_pointer,W10
		mov.b	[W10],	W12
		mov		W12,	SPI1BUF
		inc		SPI_Data_buffer_pointer
;
		cp0		SPI_Data_Length
		bra		z,		__SPI1Interrupt_Done
;
		dec		SPI_Data_Length		
		bra		__SPI1Interrupt_Exit
		
;
__SPI1Interrupt_Done:
		btss	SDFlags,	#MediaMode		; skip next if Media Read
		bra		__SPI1Interrupt_DataResponse		
;
__SPI1Interrupt_Done_Really:		
		bset	LATA,		#SD_CS
		bra		__SPI1Interrupt_Exit		
;
__SPI1Interrupt_DataResponse:
		cp0		SPI_Data_Response_Length
		bra		z,		__SPI1Interrupt_Done_Really
;
		cp0		SPI_Data_Response_Delay
		bra		z,		__SPI1Interrupt_DataResponse_Loop
;
		btss	W12,	#0					
		bra		__SPI1Interrupt_DataResponseDelay
;
		btsc	W12,	#4					
		bra		__SPI1Interrupt_DataResponseDelay				
;
		clr		SPI_Data_Response_Delay
		bra		__SPI1Interrupt_DataResponse_Loop
;
__SPI1Interrupt_DataResponseDelay:
		dec		SPI_Data_Response_Delay		
		mov		#0xFF,	W12
		mov		W12,	SPI1BUF
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_DataResponse_Loop:
		mov		SPI_Data_Response_inputbuffer_pointer,W10
		mov.b	W12,	[W10]
		inc		SPI_Data_Response_inputbuffer_pointer
		dec		SPI_Data_Response_Length
		bra		z,		__SPI1Interrupt_WaitForNotBusy
;
		mov		#0xFF,	W12
		mov		W12,	SPI1BUF
		bra		__SPI1Interrupt_Exit
;
__SPI1Interrupt_WaitForNotBusy:
		mov		#0xFF,	W0
		mov		W0,		SPI1BUF
		bclr	IFS0,	#SPI1IF
;
__SPI1Interrupt_Busy:		
		btss	IFS0,	#SPI1IF
		bra		__SPI1Interrupt_Busy
;
		mov		SPI1BUF,W1
		mov		#0xFF,	W0
		cpseq.b	W0,		W1	
		bra		__SPI1Interrupt_WaitForNotBusy
;
		bra		__SPI1Interrupt_Done_Really											
;
__SPI1Interrupt_Exit:		
		bclr	IFS0,		#SPI1IF		; Clear SPI interrupt
		pop.d	W12									; 
		pop.d	W10
		pop.d	W0
		retfie
;}
;***************************************;***************************************
;
;	Function:		SD_Reset
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		LATB.D3
; 	Calls:
;		none
;
;***************************************;***************************************
;{
SD_Reset:
	bclr	SPI1STAT,#SPIEN
	
	mov		#74,	W0	
		
SD_Reset_Loop:
	bclr	LATB,	#D3					; SCLK
	nop
	nop
	nop
	nop	
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop		
	bset	LATB,	#D3					; SCLK
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop	
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop
	nop			
	dec		W0,		W0
	bra		nz,		SD_Reset_Loop	
;	
	return	
;}
;***************************************;***************************************
;
;	Function:		CMD0
; 	Overview:       Tell the card to reset and enter its idle state 
;					Command: 1 Command byte (0x40+0)
;							 4 Argument Bytes (0,0,0,0)
;							 1 CRC byte (0x95)
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD0:
	bset	SPI1STAT,#SPIEN	
	mov		#Media_Retries,	W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD0_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x40,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0x95,	W0
	mov.b	W0,		[W10++]	
	mov		#0xFF,	W0	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#1,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode		; Media Read			
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD0_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD0_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7					; skip if no legal response
	bra		CMD0_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD0_Loop
;
	bset	SDFlags,	#BadMedia	

CMD0_Exit:
	bclr	SPI1STAT,#SPIEN	
	return
;}
;***************************************;***************************************
;
;	Function:		CMD1
; 	Overview:       Sends host capacity support information 
;					Command: 1 Command byte (0x40+1)
;							 4 Argument Bytes (0, 0, 0, 0)
;							 1 CRC byte 
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD1:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD1_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x41,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD1_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD1_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD1_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD1_Loop
;
	bset	SDFlags,	#BadMedia	

CMD1_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD8
; 	Overview:       Sends SD Memory Card interface condition that includes
;					 host supply voltage information and asks the accessed card
;					 whether card can operate in supplied voltage range. 
;					Command: 1 Command byte (0x40+8)
;							 4 Argument Bytes (0, 0, 0-0-0-0-supply voltage,
;								check pattern)
;							 1 CRC byte 
;					Response:5 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State
;							 4 condition bytes
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD8:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD8_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x48,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#5,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command
	bset	SDFlags,	#MediaMode			; Media Read					
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD8_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD8_Wait
;
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD8_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD8_Loop
;
	bset	SDFlags,	#BadMedia	

CMD8_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD9
; 	Overview: 		Request the card-specific data (CSD)
;					Command: 1 Command byte (0x40+9)
;							 4 Argument Bytes (0, 0, 0, 0) 
;							 1 CRC byte  
;					Response:1 Status byte3
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State 
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD9:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD9_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x49,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	mov		#18,		W0		
	mov		W0,		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD9_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD9_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD9_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD9_Loop
;
	bset	SDFlags,	#BadMedia	

CMD9_Exit:
	bclr	SPI1STAT,#SPIEN
	return	
;}
;***************************************;***************************************
;
;	Function:		CMD10
; 	Overview:		Request the card identification (CID)
;					Command: 1 Command byte (0x40+0x0A)
;							 4 Argument Bytes (0, 0, 0, 0)
;							 1 CRC byte 
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State  
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD10:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD10_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x4A,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	mov		#18,		W0		
	mov		W0,		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD10_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD10_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD10_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD10_Loop
;
	bset	SDFlags,	#BadMedia	

CMD10_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD17 Read a Sector
; 	Overview:		Read a single block
;					Command: 1 Command byte (0x40+0x11)
;							 4 Argument Bytes (32-bit block address)
;							 1 CRC byte 
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State         
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD17:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD17_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x51,	W0
	mov.b	W0,		[W10++]	
	mov		#BlockAddressH,W11	
	mov.b	[W11],	[W10++]
	mov		#BlockAddressL+1,W11		
	mov.b	[W11--],[W10++]
	mov.b	[W11],	[W10++]
	mov		#0,		W0
	mov.b	W0,		[W10]	
;
	bclr	SR,		#C
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10]	
	inc2	W10,	W10	
	inc2	W10,	W10
;		
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	mov		#515,	W0		
	mov		W0,		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD17_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD17_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD17_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD17_Loop
;
	bset	SDFlags,	#BadMedia	

CMD17_Exit:
	bclr	SPI1STAT,#SPIEN
	return	
;}
;***************************************;***************************************
;
;	Function:		CMD24 Write a Sector
; 	Overview:  		Write a single block
;					Command: 1 Command byte (0x40+0x18)
;							 4 Argument Bytes (32-bit block address)
;							 1 CRC byte 
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State      
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
;		SPI1STAT.SPIEN
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD24:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD24_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x58,	W0
	mov.b	W0,		[W10++]	
	mov		#BlockAddressH,W11	
	mov.b	[W11],	[W10++]
	mov		#BlockAddressL+1,W11		
	mov.b	[W11--],[W10++]
	mov.b	[W11],	[W10++]
	mov		#0,		W0
	mov.b	W0,		[W10]	
;
	bclr	SR,		#C
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10--]
	mov.b	[W10],	W0	
	rlc.b	W0,		W0
	mov.b	W0,		[W10]	
	inc2	W10,	W10	
	inc2	W10,	W10
;		
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	mov		#1,		W0		
	mov		W0,		SPI_Data_Response_Length	
	mov		#515,	W0		
	mov		W0,		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay
	mov		#BusyCount,W0		
	mov		W0,		SPI_Data_Response_Delay		
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#SPI_Data_Response_inputbuffer,W10
	mov		W10,	SPI_Data_Response_inputbuffer_pointer	
	mov		#0xFF,	W11					; delay byte before command	
	bclr	SDFlags,	#MediaMode			; Media Write				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD24_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD24_Wait
;		
	mov		#SPI_Data_Response_inputbuffer,W10
	mov		[W10],	W0
	btsc	W0,		#1				; skip if good response
	bra		CMD24_Error
;
	btss	W0,		#2				; skip if good response
	bra		CMD24_Error
;
	btsc	W0,		#3				; skip if good response
	bra		CMD24_Error
;
	bra		CMD24_Exit	
;
CMD24_Error:
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD24_Loop
;
	bset	SDFlags,	#BadMedia	

CMD24_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD41
; 	Overview:       Requests the capacity support information 
;					Command: 1 Command byte (0x40+0x29)
;							 4 Argument Bytes (0, 0, 0, 0)
;							 1 CRC byte
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
;		SPI1STAT.SPIEN
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMDA41:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMDA41_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x41,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMDA41_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMDA41_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMDA41_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMDA41_Loop
;
	bset	SDFlags,	#BadMedia	

CMDA41_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD55
; 	Overview:     	Next Command will be application-specific (ACMDXX)
;					Command: 1 Command byte (0x40+0x37)
;							 4 Argument Bytes (0,0,0,0)
;							 1 CRC byte 
;					Response:1 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State   
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
;		SPI1STAT.SPIEN
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD55:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD55_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x78,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#1,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command
	bset	SDFlags,	#MediaMode			; Media Read					
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD55_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD55_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD55_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD55_Loop
;
	bset	SDFlags,	#BadMedia	

CMD55_Exit:
	bclr	SPI1STAT,#SPIEN
	return
;}
;***************************************;***************************************
;
;	Function:		CMD58
; 	Overview:       Read OCR (Operating Conditions Register) 
;					Command: 1 Command byte (0x40+0x3A)
;							 4 Argument Bytes (0,0,0,0)
;							 1 CRC byte 
;					Response:5 Status byte
;								Bit7: 0
;								Bit6: parameter error
;								Bit5: address error
;								Bit4: erase sequence error
;								Bit3: CRC error
;								Bit2: Illegal command
;								Bit1: Erase Reset
;								Bit0: In Idle State
;							 4 OCR bytes
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		W10
;	Changes:
;		W0
;		SPI1STAT.SPIEN
;		SDFlags.BadMedia
;		Media_Retry_Count
;		LATA.SD_CS
;		SPI_CMD_Length
;		SPI_CMD_Response_Length
;		SPI_CMD_Delay
;		SPI_CMD_inputbuffer_pointer
;		SPI_CMD_outputbuffer_pointer
;		SDFlags.MediaMode
;		SPI1BUF
;		PORTA.SD_CS
;		SDFlags.BadMedia
;		SPI1STAT.SPIEN
; 	Calls:
;		none
;
;***************************************;***************************************
;{
CMD58:
	bset	SPI1STAT,#SPIEN
	mov		#Media_Retries,W0
	mov		W0,		Media_Retry_Count
	bclr	SDFlags,	#BadMedia
;
CMD58_Loop:
	bclr	LATA,	#SD_CS				; chip select
	mov		#SPI_CMD_outputbuffer,W10
	mov		#0x7A,	W0
	mov.b	W0,		[W10++]
	mov		#0x00,	W0
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]
	mov.b	W0,		[W10++]			
	mov		#0xFF,	W0
	mov.b	W0,		[W10++]	
	mov.b	W0,		[W10++]	
;				
	mov		#6,		W0		
	mov		W0,		SPI_CMD_Length
	mov		#5,		W0		
	mov		W0,		SPI_CMD_Response_Length
	clr		SPI_Data_Length
;	
	mov		#DelayCount,W0		
	mov		W0,		SPI_CMD_Delay	
	mov		#SPI_CMD_inputbuffer,W10
	mov		W10,	SPI_CMD_inputbuffer_pointer		
	mov		#SPI_CMD_outputbuffer,W10
	mov		W10,	SPI_CMD_outputbuffer_pointer
	mov		#0xFF,	W11					; delay byte before command	
	bset	SDFlags,	#MediaMode			; Media Read				
	mov		W11,	SPI1BUF				; write to SPI
;	
CMD58_Wait:	
	btss	PORTA,	#SD_CS
	bra		CMD58_Wait
;		
	mov		#SPI_CMD_Response_inputbuffer,W10
	mov		[W10],	W0
	btss	W0,		#7				; skip if no legal response
	bra		CMD58_Exit
;
	dec		Media_Retry_Count
	cp0		Media_Retry_Count
	bra		nz,		CMD58_Loop
;
	bset	SDFlags,	#BadMedia	

CMD58_Exit:
	bclr	SPI1STAT,#SPIEN
	return			       
;}
;***************************************;***************************************
;
;	Subroutine:	Print Character Set		- begins at TFTW_PositionX and 
;										 TFTW_PositionX and uses TFTW_TextColor,
;										 TFTW_FillColor, and TFTB_TextSize		
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTW_PositionX
;		TFTW_PositionY
;	Changes:
;		W0
;		TFTB_Character
;		TFTW_X
;		TFTW_Y
; 	Calls:
;		DisplayACharacter
;
;***************************************;***************************************
;{
Display_Character_Set:
		mov		#0x00,	W0
		mov.b	WREG,	TFTB_Character
;
Display_Character_Set_Loop:		
		mov		TFTW_PositionX,	W0
		mov		W0,		TFTW_X
		mov		TFTW_PositionY,	W0
		mov		W0,		TFTW_Y			
		call	DisplayACharacter
;
		inc.b	TFTB_Character
		cp0.b	TFTB_Character
		bra		NZ,		Display_Character_Set_Loop
;
Display_Character_Set_Exit:		
		return
;}
;***************************************;***************************************
;
;	Subroutine:	Display Keypad			- display twelve keys of a telephone pad
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;		LCDFlags.OrientationDisplay
;	Changes:
;		W0
;		W1
;		TFTB_KeypadRowCount
;		TFTB_KeypadColumnCount
;		TFTW_PositionX
;		TFTW_PositionY
;		CommandBufferPointer
; 	Calls:
;		Print
;
;***************************************;***************************************
;{
Display_Keypad:
		push.d	W2
		push.d	W4
;***************************************
Display_B1:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;		
		mov		#0*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#0*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1		
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'1',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B1_Exit:						
;***************************************
Display_B2:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;		
		mov		#1*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#0*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button		
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer		
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'2',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B2_Exit:						
;***************************************		
Display_B3:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#2*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#0*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'3',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print	
;
Display_B3_Exit:						
;***************************************
Display_B4:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#0*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#1*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1		
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'4',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B4_Exit:						
;***************************************
Display_B5:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#1*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#1*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1		
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'5',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B5_Exit:						
;***************************************		
Display_B6:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#2*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#1*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY		
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'6',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B6_Exit:						
;***************************************
Display_B7:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#0*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#2*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_RowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_ColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'7',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B7_Exit:						
;***************************************
Display_B8:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#1*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#2*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_RowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_ColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'8',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B8_Exit:						
;***************************************		
Display_B9:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		mov		#2*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#2*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_RowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_ColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'9',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B9_Exit:						
;***************************************
Display_B10:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		btsc	LCDFlags,	#OrientationDisplay		; skip next if Portrait 
		goto	Display_B10_4x3
;
Display_B10_3x4:			
		mov		#0*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#3*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B10_Continue		
;
Display_B10_4x3:
		mov		#3*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#0*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B10_Continue
;
Display_B10_Continue:		
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'*',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B10_Exit:						
;***************************************
Display_B11:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		btsc	LCDFlags,	#OrientationDisplay		; skip next if Portrait  
		goto	Display_B11_4x3
;
Display_B11_3x4:
		mov		#1*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#3*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B11_Continue		
;
Display_B11_4x3:
		mov		#3*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#1*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B11_Continue
;		
Display_B11_Continue:		
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'0',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print		
;		
Display_B11_Exit:						
;***************************************
Display_B12:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#4,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
;
		btsc	LCDFlags,	#OrientationDisplay		; skip next if Portrait  
		goto	Display_B12_4x3
;
Display_B12_3x4:
		mov		#2*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#3*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount				
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B12_Continue		
;
Display_B12_4x3:
		mov		#3*ColumnOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetX,		W0
		add		W0,		W2,		W4
		mov		W4,		TFTW_PositionX
;		
		mov		#2*RowOffset,	W0
		mul.b	TFTB_TextSize
		mov		#OriginOffsetY,		W0
		add		W0,		W2,		W5
		mov		W5,		TFTW_PositionY	
		mov		#ButtonRows,		W0
		mov.b	WREG,	TFTB_KeypadRowCount
		mov		#ButtonColumns,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount			
		call	Display_Button
;
		mov		#CharacterColumnSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W4,		W0
		mov		W0,		TFTW_PositionX
		mov		#CharacterRowSpacing,	W0
		mul.b	TFTB_TextSize		
		add		W2,		W5,		W0		
		mov		W0,		TFTW_PositionY
		goto	Display_B12_Continue
;
Display_B12_Continue:		
		mov		#CommandBuffer,	W1
		mov		W1,		CommandBufferPointer
		mov		#' ',	W0
		mov.b	W0,		[W1++]
		mov		#'#',	W0
		mov.b	W0,		[W1++]
		mov		#0x0D,	W0
		mov.b	W0,		[W1++]
		call	Print
;
Display_B12_Exit:						
;***************************************
Display_Keypad_Exit:
		pop.d	W4
		pop.d	W2		
		return
;}
;***************************************;***************************************
;
;	Subroutine:	Clear Screen			- uses TFTW_FillColor
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		W0
;		W1
;		TFT_IR
;		ButtonID
; 	Calls:
;		FullScreenWindow
;		Fill_Address_Window
;		
;***************************************;***************************************
;{ 
ClearScreen:
		call	FullScreenWindow
;
		mov		#TFT_WriteDataToGRAMRegister,	W0
		mov.b	WREG,	TFT_IR
		call	Fill_Address_Window	
;
ClearScreen_ClearAnyButtons:
		mov		#1,		W0
		mov.b	WREG,	ButtonID
;		
		mov		#TouchMatrix,	W0
		mov		#256,		W1
		add		W0,		W1,		W1
;
ClearScreen_ClearMatrixLoop:
		clr.b	[W0++]
		cpseq	W0,		W1
		goto	ClearScreen_ClearMatrixLoop		
;
		return
;}
;***************************************;***************************************
;
;	Subroutine:	SwapOrientation	swap orientation before/after fill
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		LCDFlags
; 	Calls:
;		ChangeOrientation
;
;***************************************;***************************************
;{ 		
SwapOrientation:		
		mov		#1<<OrientationDisplay,		W0
		xor		LCDFlags
		call	ChangeOrientation
;
		return				
;}
;***************************************;***************************************
;
;	Subroutine:	SwapOrientation	swap orientation before/after fill
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		LCDFlags
; 	Calls:
;		ChangeUpsideDown
;
;***************************************;***************************************
;{ 		
FlipUpsideDown:		
		mov		#1<<UpsideDownDisplay,		W0
		xor		LCDFlags
		call	ChangeUpsideDown
;
		return
;}
;***************************************;***************************************
;
;	Subroutine:	mirror the image before/after fill
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		LCDFlags
; 	Calls:
;		ChangeMirror
;
;***************************************;***************************************
;{ 		
MirrorImage:		
		mov		#1<<MirrorDisplay,		W0
		xor		LCDFlags
		call	ChangeMirror
;
		return				
;}
;***************************************;***************************************
;
;	Subroutine:	Full Screen Window		- set window to full screen
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		Width
;		Height
;	Changes:
;		W0
;		TFTW_PositionX
;		TFTW_PositionY
;		TFT_IR
;		TFTW_X1
;		TFTW_X2
;		TFTW_Y1
;		TFTW_Y2
; 	Calls:
;		Set_Address_Window
;		
;***************************************;***************************************
;{ 		
FullScreenWindow:
		clr		TFTW_PositionX			; X1=0
		clr		TFTW_PositionY			; Y1=0		
		clr		TFTW_X1					; X1=0
		clr		TFTW_Y1					; Y1=0
		mov		Width,	W0				; get Width
		mov		W0,		TFTW_X2			; TFTW_X2=Width
		dec		TFTW_X2					; TFTW_X2=Width-1
		mov		Height,	W0				; get Height
		mov		W0,		TFTW_Y2			; TFTW_Y2=Height
		dec		TFTW_Y2					; TFTW_Y2=Height-1
		call	Set_Address_Window	
;
FullScreenWindow_Exit:	
		return	
;}
;***************************************;***************************************
;
;	Subroutine:	Help
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;	Changes:
;		W0
;		W1
;		TFTB_TextSize
; 	Calls:
;		ClearScreen
;		ROMToRAM
;		Print
;		NextY
;		CR
;		
;***************************************;***************************************
;{ 
Help:
		push.d	W2
		call	ClearScreen
;		
		mov.b	TFTB_TextSize,	WREG
		mov		W0,		W2
		mov		#2,		W0
		mov.b	WREG,	TFTB_TextSize		
;		
		mov		#tblpage(Help1), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help1), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help2), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help2), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
				;		
		mov		#tblpage(Help3), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help3), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help4), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help4), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help5), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help5), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help6), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help6), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help7), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help7), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help8), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help8), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help9), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help9), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help10), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help10), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help11), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help11), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help12), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help12), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help13), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help13), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help14), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help14), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print		
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(Help15), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(Help15), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
		mov		W2,		W0
		mov.b	WREG,	TFTB_TextSize
;
		mov		#CommandBuffer,	W0
		mov		W0,	CommandBufferPointer				
;				
Help_Exit:
		pop.d	W2
		return		
;}
;***************************************;***************************************
;
;	Subroutine:	ImagineThat
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;	Changes:
;		W0
;		W1
;		TFTB_TextSize
; 	Calls:
;		ROMToRAM
;		Print
;		NextY
;		CR
;		
;***************************************;***************************************
;{ 
ImagineThat:
		push.d	W2
;		
		mov.b	TFTB_TextSize,	WREG
		mov		W0,		W2
		mov		#2,		W0
		mov.b	WREG,	TFTB_TextSize		
;		
		mov		#tblpage(SignON1), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignON1), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(SignON2), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignON2), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR				
;		
		mov		#tblpage(SignON3), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignON3), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	NextY		
;
		call	CR								
;
		mov		#tblpage(SignON4), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignON4), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		call	NextY
;
		call	CR
;		
		mov		#tblpage(SignON5), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(SignON5), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		mov		W2,		W0
		mov.b	WREG,	TFTB_TextSize		
;				
		mov		#CommandBuffer,	W0
		mov		W0,	CommandBufferPointer
;		
ImagineThat_Exit:
		pop.d	W2
		return
;}
;***************************************;***************************************
;
;	Subroutine:	Calibrate the Touch Screen
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTB_TextSize
;		Width
;		Height
;	Changes:
;		W0
;		W1
;		TFTB_TextSize
;		LCDFlags.WrapAround
;		TFTW_PositionX
;		TFTW_PositionY
;		TEMPB
;		TouchFlags.Button1
;		TouchFlags.Button2
;		TouchFlags.Calibrate
; 	Calls:
;		ClearScreen
;		TargetButton
;		ROMToRAM
;		Print
;
;***************************************;***************************************
;{
CalibrateScreen:
		call	ClearScreen
;
		bclr	LCDFlags,	#WrapAround
;		
		mov.b	TFTB_TextSize,	WREG
		mov.b	WREG,	TEMPB		
		mov		#1,		W0
		mov.b	WREG,		TFTB_TextSize
;		
		mov		#0x00,	W0
		mov		W0,		TFTW_PositionX
		mov		#0x00,	W0
		mov		W0,		TFTW_PositionY
		call	TargetButton
;
		mov		Width,	W0
		sub		#19,	W0
		mov		W0,		TFTW_PositionX
		mov		Height,	W0
		sub		#25,	W0		
		mov		W0,		TFTW_PositionY
		call	TargetButton
;
		mov		#3,		W0
		mov.b	WREG,		TFTB_TextSize
;		
		mov		#0x00,	W0
		mov		W0,		TFTW_PositionX
		mov		#0x64,	W0
		mov		W0,		TFTW_PositionY						
;		
		mov		#tblpage(TouchInstructions), W1
		mov		W1, _TBLPAG 	    	; Set TBLPAG to the page that contains
										; the table array.
		mov		#tbloffset(TouchInstructions), W1	; Make a pointer to table
		call	ROMToRAM
;
		mov		#CommandBuffer,		W0
		mov		W0,		CommandBufferPointer
		call	Print
;
		mov.b	TEMPB,	WREG		
		mov.b	WREG,		TFTB_TextSize
		bclr	TouchFlags,	#Button1
		bclr	TouchFlags,	#Button2
		bset	TouchFlags,	#Calibrate								
;
CalibrateScreen_Exit:
		return
;}
;***************************************;***************************************
;
;	Subroutine:	Calibrate Done
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		none
;	Changes:
;		none
; 	Calls:
;		ClearScreen
;		
;***************************************;***************************************
;{
CalibrateDone:
		call	ClearScreen	
;
CalibrateDone_Exit:
		return			
;}
;***************************************;***************************************
;
;	Subroutine:	Target Button
;
;	DateLastEdited:	4/29/2013
;
;	Requires:
;		TFTW_X
;		TFTW_Y
;	Changes:
;		TFTB_KeypadColumnCount
;		TFTB_KeypadRowCount
;		TFTB_Character
;		TFTW_X
;		TFTW_Y
; 	Calls:
;		DisplayACharacter
;		
;***************************************;***************************************
;{		
TargetButton:
		mov		#3,		W0
		mov.b	WREG,	TFTB_KeypadColumnCount
		mov.b	WREG,	TFTB_KeypadRowCount		
		call	Display_Button
;
		mov		#0xC5,	W0				; (cross) character
		mov		W0,		TFTB_Character
;		
		mov		TFTW_X,	W0
		sub		#6,	W0
		mov		W0,		TFTW_X
		mov		TFTW_Y,	W0
		sub		#8,	W0
		mov		W0,		TFTW_Y		
		call	DisplayACharacter
;	
TargetButton_Exit:
		return		
;}
;***************************************;***************************************
;
;	Commands							- User commands
;
;***************************************;***************************************
;{  
Register:
		.asciz 	"Register:"
Value:
		.asciz 	"Value:"      
CLS_Command:
		.asciz 	"CLS"
Pixel_Command:
		.asciz 	"P="
LF_Command:
		.asciz 	"LF"
CR_Command:
		.asciz 	"CR"
Display_Command:
		.asciz 	"DISPLAY="
Invert_Command:
		.asciz 	"INVERT="				
Wrap_Command:
		.asciz 	"WR_STROBEAP="
BackLight_Command:
		.asciz 	"BACKLIGHT="		
IT_Command:
		.asciz 	"IT"
OK_Command:
		.asciz 	"OK"								
FColor_Command:
		.asciz 	"FCOLOR="
BColor_Command:
		.asciz 	"BCOLOR="
ATX_Command:
		.asciz 	"@X="
ATY_Command:
		.asciz 	"@Y="
Button_Command:
		.asciz 	"BUTTON="
BitMap_Command:
		.asciz 	"BITMAP="		
TextSize_Command:
		.asciz 	"TEXTSIZE="
Orientation_Command:
		.asciz 	"ORIENTATION="
Print_Command:
		.asciz 	"PRINT="
Mirror_Command:
		.asciz 	"MIRROR="
UpsideDown_Command:
		.asciz 	"UPSIDEDOWN="				
Test_Command:
		.asciz 	"TEST="
BMP_Command:
		.asciz 	"BMP="		
XTouch_Command:
		.asciz 	"XTouch="
YTouch_Command:
		.asciz 	"YTouch="
ZTouch_Command:
		.asciz 	"ZTouch="																				
Black_Color:
		.asciz 	"BLACK"
Blue_Color:
		.asciz 	"BLUE"
Green_Color:
		.asciz 	"GREEN"
Cyan_Color:
		.asciz 	"CYAN"
Red_Color:
		.asciz 	"RED"
Magenta_Color:
		.asciz 	"MAGENTA"
Yellow_Color:
		.asciz 	"YELLOW"
White_Color:
		.asciz 	"WHITE"
SignON1:
		.asciz 	"   Imagine That"
SignON2:
		.asciz 	"  Jeff Bachiochi"
SignON3:
		.asciz 	"imaginethatnow.com"		
SignON4:
		.asciz 	"  from the bench"
SignON5:
		.asciz 	"Circuit Cellar #276"
Help1:
		.asciz 	"Help..."		
Help2:
		.asciz 	"@X=d or @Y=d"
Help3:
		.asciz 	"CLS, LF, CR"
Help4:
		.asciz 	"PRINT=text"
Help5:
		.asciz 	"BUTTON=cols,rows"		
Help6:
		.asciz 	"FCOLOR=hhhh"
Help7:
		.asciz 	"BCOLOR=hhhh"
Help8:
		.asciz 	"DISPLAY=b INVERT=b"
Help9:
		.asciz 	"BACKLIGHT=b"		
Help10:
		.asciz 	"ORIENTATION=P/L"
Help11:
		.asciz 	"TEXTSIZE=d TEST=d"		
Help12:
		.asciz 	"P=hhhh"
Help13:
		.asciz 	"touch returns #"
Help14:
		.asciz 	"BMP=cols,rows"
Help15:
		.asciz 	"Bitmap=filename"				
NoSD:
		.asciz 	"No SD Card Found"
MediaErrorSD:
		.asciz 	"Media Error"
OKSD:
		.asciz 	"SD Card Ready"
DirSearch:
		.asciz 	"Searching..."
File:
		.asciz 	"File"	
Portrait:
		.asciz 	"PORTRAIT"
Landscape:
		.asciz 	"LANDSCAPE"	
TouchInstructions:
		.asciz 	"Touch Targets"	
BMPTooLarge:
		.asciz 	"BMP Too Large"																																						
;}
;***************************************;***************************************
;
;	TFT Registers						
;
;***************************************;***************************************
;{
TFT_Registers:
		.byte	0x00, 0x01, 0x02, 0x03, 0x04, 0x07
		.byte	0x08, 0x09, 0x0A, 0x0C, 0x0D, 0x0F
		.byte	0x10, 0x11, 0x12, 0x13
		.byte	0x20, 0x21, 0x22 
		.byte	0x2B
		.byte	0x30, 0x31, 0x32, 0x35, 0x36, 0x37
		.byte	0x38, 0x39, 0x3C, 0x3D
		.byte	0x50, 0x51, 0x52, 0x53
		.byte	0x60, 0x61
		.byte	0x6A
		.byte	0x80, 0x81, 0x82, 0x83, 0x84, 0x85
		.byte	0x90, 0x92,0x95
		.byte	0xA1, 0xA2, 0xA5																													
;}
;***************************************;***************************************
;
;	Font
;
;***************************************;***************************************
;				
;{
FONT:
Char0x00:
	.byte	0x00, 0x00, 0x00, 0x00, 0x00, 0x00  
	.byte	0x3E, 0x5B, 0x4F, 0x5B, 0x3E, 0x00 	
	.byte	0x3E, 0x6B, 0x4F, 0x6B, 0x3E, 0x00	
	.byte	0x1C, 0x3E, 0x7C, 0x3E, 0x1C, 0x00 
	.byte	0x18, 0x3C, 0x7E, 0x3C, 0x18, 0x00 
	.byte	0x1C, 0x57, 0x7D, 0x57, 0x1C, 0x00 
	.byte	0x1C, 0x5E, 0x7F, 0x5E, 0x1C, 0x00 
	.byte	0x00, 0x18, 0x3C, 0x18, 0x00, 0x00 
	.byte	0xFF, 0xE7, 0xC3, 0xE7, 0xFF, 0x00 
	.byte	0x00, 0x18, 0x24, 0x18, 0x00, 0x00 
	.byte	0xFF, 0xE7, 0xDB, 0xE7, 0xFF, 0x00 
	.byte	0x30, 0x48, 0x3A, 0x06, 0x0E, 0x00 
	.byte	0x26, 0x29, 0x79, 0x29, 0x26, 0x00 
	.byte	0x40, 0x7F, 0x05, 0x05, 0x07, 0x00 
	.byte	0x40, 0x7F, 0x05, 0x25, 0x3F, 0x00 
	.byte	0x5A, 0x3C, 0xE7, 0x3C, 0x5A, 0x00
Char0x10:
	.byte	0x7F, 0x3E, 0x1C, 0x1C, 0x08, 0x00 
	.byte	0x08, 0x1C, 0x1C, 0x3E, 0x7F, 0x00 
	.byte	0x14, 0x22, 0x7F, 0x22, 0x14, 0x00 
	.byte	0x5F, 0x5F, 0x00, 0x5F, 0x5F, 0x00 
	.byte	0x06, 0x09, 0x7F, 0x01, 0x7F, 0x00 
	.byte	0x00, 0x66, 0x89, 0x95, 0x6A, 0x00 
	.byte	0x60, 0x60, 0x60, 0x60, 0x60, 0x00 
	.byte	0x94, 0xA2, 0xFF, 0xA2, 0x94, 0x00 
	.byte	0x08, 0x04, 0x7E, 0x04, 0x08, 0x00 
	.byte	0x10, 0x20, 0x7E, 0x20, 0x10, 0x00 
	.byte	0x08, 0x08, 0x2A, 0x1C, 0x08, 0x00 
	.byte	0x08, 0x1C, 0x2A, 0x08, 0x08, 0x00 
	.byte	0x1E, 0x10, 0x10, 0x10, 0x10, 0x00 
	.byte	0x0C, 0x1E, 0x0C, 0x1E, 0x0C, 0x00 
	.byte	0x30, 0x38, 0x3E, 0x38, 0x30, 0x00 
	.byte	0x06, 0x0E, 0x3E, 0x0E, 0x06, 0x00
Char0x20:
	.byte	0x00, 0x00, 0x00, 0x00, 0x00, 0x00 
	.byte	0x00, 0x00, 0x5F, 0x00, 0x00, 0x00 
	.byte	0x00, 0x07, 0x00, 0x07, 0x00, 0x00 
	.byte	0x14, 0x7F, 0x14, 0x7F, 0x14, 0x00 
	.byte	0x24, 0x2A, 0x7F, 0x2A, 0x12, 0x00 
	.byte	0x23, 0x13, 0x08, 0x64, 0x62, 0x00 
	.byte	0x36, 0x49, 0x56, 0x20, 0x50, 0x00 
	.byte	0x00, 0x08, 0x07, 0x03, 0x00, 0x00 
	.byte	0x00, 0x1C, 0x22, 0x41, 0x00, 0x00 
	.byte	0x00, 0x41, 0x22, 0x1C, 0x00, 0x00 
	.byte	0x2A, 0x1C, 0x7F, 0x1C, 0x2A, 0x00 
	.byte	0x08, 0x08, 0x3E, 0x08, 0x08, 0x00 
	.byte	0x00, 0x80, 0x70, 0x30, 0x00, 0x00 
	.byte	0x08, 0x08, 0x08, 0x08, 0x08, 0x00 
	.byte	0x00, 0x00, 0x60, 0x60, 0x00, 0x00 
	.byte	0x20, 0x10, 0x08, 0x04, 0x02, 0x00
Char0x30:
	.byte	0x3E, 0x51, 0x49, 0x45, 0x3E, 0x00 
	.byte	0x00, 0x42, 0x7F, 0x40, 0x00, 0x00 
	.byte	0x72, 0x49, 0x49, 0x49, 0x46, 0x00 
	.byte	0x21, 0x41, 0x49, 0x4D, 0x33, 0x00 
	.byte	0x18, 0x14, 0x12, 0x7F, 0x10, 0x00 
	.byte	0x27, 0x45, 0x45, 0x45, 0x39, 0x00 
	.byte	0x3C, 0x4A, 0x49, 0x49, 0x31, 0x00 
	.byte	0x41, 0x21, 0x11, 0x09, 0x07, 0x00 
	.byte	0x36, 0x49, 0x49, 0x49, 0x36, 0x00 
	.byte	0x46, 0x49, 0x49, 0x29, 0x1E, 0x00 
	.byte	0x00, 0x00, 0x14, 0x00, 0x00, 0x00 
	.byte	0x00, 0x40, 0x34, 0x00, 0x00, 0x00 
	.byte	0x00, 0x08, 0x14, 0x22, 0x41, 0x00 
	.byte	0x14, 0x14, 0x14, 0x14, 0x14, 0x00 
	.byte	0x00, 0x41, 0x22, 0x14, 0x08, 0x00
	.byte	0x02, 0x01, 0x59, 0x09, 0x06, 0x00
Char0x40:
	.byte	0x3E, 0x41, 0x5D, 0x59, 0x4E, 0x00
	.byte	0x7C, 0x12, 0x11, 0x12, 0x7C, 0x00 
	.byte	0x7F, 0x49, 0x49, 0x49, 0x36, 0x00 
	.byte	0x3E, 0x41, 0x41, 0x41, 0x22, 0x00 
	.byte	0x7F, 0x41, 0x41, 0x41, 0x3E, 0x00 
	.byte	0x7F, 0x49, 0x49, 0x49, 0x41, 0x00 
	.byte	0x7F, 0x09, 0x09, 0x09, 0x01, 0x00 
	.byte	0x3E, 0x41, 0x41, 0x51, 0x73, 0x00 
	.byte	0x7F, 0x08, 0x08, 0x08, 0x7F, 0x00 
	.byte	0x00, 0x41, 0x7F, 0x41, 0x00, 0x00 
	.byte	0x20, 0x40, 0x41, 0x3F, 0x01, 0x00 
	.byte	0x7F, 0x08, 0x14, 0x22, 0x41, 0x00 
	.byte	0x7F, 0x40, 0x40, 0x40, 0x40, 0x00 
	.byte	0x7F, 0x02, 0x1C, 0x02, 0x7F, 0x00 
	.byte	0x7F, 0x04, 0x08, 0x10, 0x7F, 0x00 
	.byte	0x3E, 0x41, 0x41, 0x41, 0x3E, 0x00
Char0x50:
	.byte	0x7F, 0x09, 0x09, 0x09, 0x06, 0x00 
	.byte	0x3E, 0x41, 0x51, 0x21, 0x5E, 0x00
	.byte	0x7F, 0x09, 0x19, 0x29, 0x46, 0x00 
	.byte	0x26, 0x49, 0x49, 0x49, 0x32, 0x00 
	.byte	0x03, 0x01, 0x7F, 0x01, 0x03, 0x00 
	.byte	0x3F, 0x40, 0x40, 0x40, 0x3F, 0x00 
	.byte	0x1F, 0x20, 0x40, 0x20, 0x1F, 0x00 
	.byte	0x3F, 0x40, 0x38, 0x40, 0x3F, 0x00 
	.byte	0x63, 0x14, 0x08, 0x14, 0x63, 0x00 
	.byte	0x03, 0x04, 0x78, 0x04, 0x03, 0x00 
	.byte	0x61, 0x59, 0x49, 0x4D, 0x43, 0x00 
	.byte	0x00, 0x7F, 0x41, 0x41, 0x41, 0x00 
	.byte	0x02, 0x04, 0x08, 0x10, 0x20, 0x00 
	.byte	0x00, 0x41, 0x41, 0x41, 0x7F, 0x00 
	.byte	0x04, 0x02, 0x01, 0x02, 0x04, 0x00 
	.byte	0x40, 0x40, 0x40, 0x40, 0x40, 0x00
Char0x60:
	.byte	0x00, 0x03, 0x07, 0x08, 0x00, 0x00 
	.byte	0x20, 0x54, 0x54, 0x78, 0x40, 0x00 
	.byte	0x7F, 0x28, 0x44, 0x44, 0x38, 0x00 
	.byte	0x38, 0x44, 0x44, 0x44, 0x28, 0x00 
	.byte	0x38, 0x44, 0x44, 0x28, 0x7F, 0x00 
	.byte	0x38, 0x54, 0x54, 0x54, 0x18, 0x00 
	.byte	0x00, 0x08, 0x7E, 0x09, 0x02, 0x00 
	.byte	0x18, 0xA4, 0xA4, 0x9C, 0x78, 0x00 
	.byte	0x7F, 0x08, 0x04, 0x04, 0x78, 0x00
	.byte	0x00, 0x44, 0x7D, 0x40, 0x00, 0x00 
	.byte	0x20, 0x40, 0x40, 0x3D, 0x00, 0x00 
	.byte	0x7F, 0x10, 0x28, 0x44, 0x00, 0x00 
	.byte	0x00, 0x41, 0x7F, 0x40, 0x00, 0x00 
	.byte	0x7C, 0x04, 0x78, 0x04, 0x78, 0x00 
	.byte	0x7C, 0x08, 0x04, 0x04, 0x78, 0x00 
	.byte	0x38, 0x44, 0x44, 0x44, 0x38, 0x00
Char0x70:
	.byte	0xFC, 0x18, 0x24, 0x24, 0x18, 0x00 
	.byte	0x18, 0x24, 0x24, 0x18, 0xFC, 0x00 
	.byte	0x7C, 0x08, 0x04, 0x04, 0x08, 0x00 
	.byte	0x48, 0x54, 0x54, 0x54, 0x24, 0x00 
	.byte	0x04, 0x04, 0x3F, 0x44, 0x24, 0x00 
	.byte	0x3C, 0x40, 0x40, 0x20, 0x7C, 0x00 
	.byte	0x1C, 0x20, 0x40, 0x20, 0x1C, 0x00 
	.byte	0x3C, 0x40, 0x30, 0x40, 0x3C, 0x00 
	.byte	0x44, 0x28, 0x10, 0x28, 0x44, 0x00 
	.byte	0x4C, 0x90, 0x90, 0x90, 0x7C, 0x00 
	.byte	0x44, 0x64, 0x54, 0x4C, 0x44, 0x00 
	.byte	0x00, 0x08, 0x36, 0x41, 0x00, 0x00 
	.byte	0x00, 0x00, 0x77, 0x00, 0x00, 0x00 
	.byte	0x00, 0x41, 0x36, 0x08, 0x00, 0x00 
	.byte	0x02, 0x01, 0x02, 0x04, 0x02, 0x00 
	.byte	0x3C, 0x26, 0x23, 0x26, 0x3C, 0x00
Char0x80:
	.byte	0x1E, 0xA1, 0xA1, 0x61, 0x12, 0x00 
	.byte	0x3A, 0x40, 0x40, 0x20, 0x7A, 0x00 
	.byte	0x38, 0x54, 0x54, 0x55, 0x59, 0x00 
	.byte	0x21, 0x55, 0x55, 0x79, 0x41, 0x00 
	.byte	0x21, 0x54, 0x54, 0x78, 0x41, 0x00 
	.byte	0x21, 0x55, 0x54, 0x78, 0x40, 0x00 
	.byte	0x20, 0x54, 0x55, 0x79, 0x40, 0x00 
	.byte	0x0C, 0x1E, 0x52, 0x72, 0x12, 0x00 
	.byte	0x39, 0x55, 0x55, 0x55, 0x59, 0x00 
	.byte	0x39, 0x54, 0x54, 0x54, 0x59, 0x00 
	.byte	0x39, 0x55, 0x54, 0x54, 0x58, 0x00 
	.byte	0x00, 0x00, 0x45, 0x7C, 0x41, 0x00 
	.byte	0x00, 0x02, 0x45, 0x7D, 0x42, 0x00 
	.byte	0x00, 0x01, 0x45, 0x7C, 0x40, 0x00 
	.byte	0xF0, 0x29, 0x24, 0x29, 0xF0, 0x00 
	.byte	0xF0, 0x28, 0x25, 0x28, 0xF0, 0x00
Char0x90:
	.byte	0x7C, 0x54, 0x55, 0x45, 0x00, 0x00 
	.byte	0x20, 0x54, 0x54, 0x7C, 0x54, 0x00 
	.byte	0x7C, 0x0A, 0x09, 0x7F, 0x49, 0x00
	.byte	0x32, 0x49, 0x49, 0x49, 0x32, 0x00 
	.byte	0x32, 0x48, 0x48, 0x48, 0x32, 0x00 
	.byte	0x32, 0x4A, 0x48, 0x48, 0x30, 0x00 
	.byte	0x3A, 0x41, 0x41, 0x21, 0x7A, 0x00 
	.byte	0x3A, 0x42, 0x40, 0x20, 0x78, 0x00 
	.byte	0x00, 0x9D, 0xA0, 0xA0, 0x7D, 0x00 
	.byte	0x39, 0x44, 0x44, 0x44, 0x39, 0x00 
	.byte	0x3D, 0x40, 0x40, 0x40, 0x3D, 0x00 
	.byte	0x3C, 0x24, 0xFF, 0x24, 0x24, 0x00 
	.byte	0x48, 0x7E, 0x49, 0x43, 0x66, 0x00 
	.byte	0x2B, 0x2F, 0xFC, 0x2F, 0x2B, 0x00 
	.byte	0xFF, 0x09, 0x29, 0xF6, 0x20, 0x00 
	.byte	0xC0, 0x88, 0x7E, 0x09, 0x03, 0x00
Char0xA0:
	.byte	0x20, 0x54, 0x54, 0x79, 0x41, 0x00 
	.byte	0x00, 0x00, 0x44, 0x7D, 0x41, 0x00 
	.byte	0x30, 0x48, 0x48, 0x4A, 0x32, 0x00 
	.byte	0x38, 0x40, 0x40, 0x22, 0x7A, 0x00 
	.byte	0x00, 0x7A, 0x0A, 0x0A, 0x72, 0x00 
	.byte	0x7D, 0x0D, 0x19, 0x31, 0x7D, 0x00 
	.byte	0x26, 0x29, 0x29, 0x2F, 0x28, 0x00 
	.byte	0x26, 0x29, 0x29, 0x29, 0x26, 0x00 
	.byte	0x30, 0x48, 0x4D, 0x40, 0x20, 0x00 
	.byte	0x38, 0x08, 0x08, 0x08, 0x08, 0x00 
	.byte	0x08, 0x08, 0x08, 0x08, 0x38, 0x00 
	.byte	0x2F, 0x10, 0xC8, 0xAC, 0xBA, 0x00 
	.byte	0x2F, 0x10, 0x28, 0x34, 0xFA, 0x00 
	.byte	0x00, 0x00, 0x7B, 0x00, 0x00, 0x00 
	.byte	0x08, 0x14, 0x2A, 0x14, 0x22, 0x00 
	.byte	0x22, 0x14, 0x2A, 0x14, 0x08, 0x00
Char0xB0:
	.byte	0x00, 0x55, 0x00, 0x55, 0x00, 0x55 
	.byte	0xAA, 0x00, 0x55, 0x00, 0xAA, 0x00 
	.byte	0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55 
	.byte	0x00, 0x00, 0x00, 0xFF, 0x00, 0x00 
	.byte	0x10, 0x10, 0x10, 0xFF, 0x00, 0x00 
	.byte	0x14, 0x14, 0x14, 0xFF, 0x00, 0x00 
	.byte	0x10, 0x10, 0xFF, 0x00, 0xFF, 0x00 
	.byte	0x10, 0x10, 0xF0, 0x10, 0xF0, 0x00 
	.byte	0x14, 0x14, 0x14, 0xFC, 0x00, 0x00 
	.byte	0x14, 0x14, 0xF7, 0x00, 0xFF, 0x00 
	.byte	0x00, 0x00, 0xFF, 0x00, 0xFF, 0x00 
	.byte	0x14, 0x14, 0xF4, 0x04, 0xFC, 0x00 
	.byte	0x14, 0x14, 0x17, 0x10, 0x1F, 0x00 
	.byte	0x10, 0x10, 0x1F, 0x10, 0x1F, 0x00 
	.byte	0x14, 0x14, 0x14, 0x1F, 0x00, 0x00 
	.byte	0x10, 0x10, 0x10, 0xF0, 0x00, 0x00 
Char0xC0:	
	.byte	0x00, 0x00, 0x00, 0x1F, 0x10, 0x10
	.byte	0x10, 0x10, 0x10, 0x1F, 0x10, 0x10
	.byte	0x10, 0x10, 0x10, 0xF0, 0x10, 0x10 
	.byte	0x00, 0x00, 0x00, 0xFF, 0x10, 0x10 
	.byte	0x10, 0x10, 0x10, 0x10, 0x10, 0x10 
	.byte	0x10, 0x10, 0x10, 0xFF, 0x10, 0x10 
	.byte	0x00, 0x00, 0x00, 0xFF, 0x14, 0x14 
	.byte	0x00, 0x00, 0xFF, 0x00, 0xFF, 0x10 
	.byte	0x00, 0x00, 0x1F, 0x10, 0x17, 0x14 
	.byte	0x00, 0x00, 0xFC, 0x04, 0xF4, 0x14 
	.byte	0x14, 0x14, 0x17, 0x10, 0x17, 0x14 
	.byte	0x14, 0x14, 0xF4, 0x04, 0xF4, 0x14 
	.byte	0x00, 0x00, 0xFF, 0x00, 0xF7, 0x14 
	.byte	0x14, 0x14, 0x14, 0x14, 0x14, 0x14 
	.byte	0x14, 0x14, 0xF7, 0x00, 0xF7, 0x14 
	.byte	0x14, 0x14, 0x14, 0x17, 0x14, 0x14
Char0xD0:	
	.byte	0x10, 0x10, 0x1F, 0x10, 0x1F, 0x10
	.byte	0x14, 0x14, 0x14, 0xF4, 0x14, 0x14 
	.byte	0x10, 0x10, 0xF0, 0x10, 0xF0, 0x10 
	.byte	0x00, 0x00, 0x1F, 0x10, 0x1F, 0x10 
	.byte	0x00, 0x00, 0x00, 0x1F, 0x14, 0x14 
	.byte	0x00, 0x00, 0x00, 0xFC, 0x14, 0x14 
	.byte	0x00, 0x00, 0xF0, 0x10, 0xF0, 0x10 
	.byte	0x10, 0x10, 0xFF, 0x10, 0xFF, 0x10 
	.byte	0x14, 0x14, 0x14, 0xFF, 0x14, 0x14 
	.byte	0x10, 0x10, 0x10, 0x1F, 0x00, 0x00 
	.byte	0x00, 0x00, 0x00, 0xF0, 0x10, 0x10 
	.byte	0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF 
	.byte	0xF0, 0xF0, 0xF0, 0xF0, 0xF0, 0xF0 
	.byte	0xFF, 0xFF, 0xFF, 0x00, 0x00, 0x00 
	.byte	0x00, 0x00, 0x00, 0xFF, 0xFF, 0xFF 
	.byte	0x0F, 0x0F, 0x0F, 0x0F, 0x0F, 0x0F 
Char0xE0:	
	.byte	0x38, 0x44, 0x44, 0x38, 0x44, 0x00
	.byte	0x7C, 0x2A, 0x2A, 0x3E, 0x14, 0x00 
	.byte	0x7E, 0x02, 0x02, 0x06, 0x06, 0x00 
	.byte	0x02, 0x7E, 0x02, 0x7E, 0x02, 0x00 
	.byte	0x63, 0x55, 0x49, 0x41, 0x63, 0x00 
	.byte	0x38, 0x44, 0x44, 0x3C, 0x04, 0x00 
	.byte	0x40, 0x7E, 0x20, 0x1E, 0x20, 0x00 
	.byte	0x06, 0x02, 0x7E, 0x02, 0x02, 0x00 
	.byte	0x99, 0xA5, 0xE7, 0xA5, 0x99, 0x00 
	.byte	0x1C, 0x2A, 0x49, 0x2A, 0x1C, 0x00 
	.byte	0x4C, 0x72, 0x01, 0x72, 0x4C, 0x00 
	.byte	0x30, 0x4A, 0x4D, 0x4D, 0x30, 0x00 
	.byte	0x30, 0x48, 0x78, 0x48, 0x30, 0x00 
	.byte	0xBC, 0x62, 0x5A, 0x46, 0x3D, 0x00 
	.byte	0x3E, 0x49, 0x49, 0x49, 0x00, 0x00 
	.byte	0x7E, 0x01, 0x01, 0x01, 0x7E, 0x00 
Char0xF0:	
	.byte	0x2A, 0x2A, 0x2A, 0x2A, 0x2A, 0x00
	.byte	0x44, 0x44, 0x5F, 0x44, 0x44, 0x00 
	.byte	0x40, 0x51, 0x4A, 0x44, 0x40, 0x00 
	.byte	0x40, 0x44, 0x4A, 0x51, 0x40, 0x00 
	.byte	0x00, 0x00, 0xFF, 0x01, 0x03, 0x00 
	.byte	0xE0, 0x80, 0xFF, 0x00, 0x00, 0x00 
	.byte	0x08, 0x08, 0x6B, 0x6B, 0x08, 0x00
	.byte	0x36, 0x12, 0x36, 0x24, 0x36, 0x00 
	.byte	0x06, 0x0F, 0x09, 0x0F, 0x06, 0x00 
	.byte	0x00, 0x00, 0x18, 0x18, 0x00, 0x00 
	.byte	0x00, 0x00, 0x10, 0x10, 0x00, 0x00 
	.byte	0x30, 0x40, 0xFF, 0x01, 0x01, 0x00 
	.byte	0x00, 0x1F, 0x01, 0x01, 0x1E, 0x00 
	.byte	0x00, 0x19, 0x1D, 0x17, 0x12, 0x00 
	.byte	0x00, 0x3C, 0x3C, 0x3C, 0x3C, 0x00 
	.byte	0x00, 0x00, 0x00, 0x00, 0x00, 0x00
;}
;***************************************;***************************************
;
;	End
;
;***************************************;***************************************
;

.end                               ;End of program code in this file